@@ -79,18 +79,164 @@
 
 #include <vips/vips.h>
 
-#include "arithmetic.h"
 #include "binary.h"
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
+typedef VipsBinary VipsAdd;
+typedef VipsBinaryClass VipsAddClass;
+
+G_DEFINE_TYPE( VipsAdd, vips_add, VIPS_TYPE_BINARY );
+
+#define LOOP( IN, OUT ) { \
+	IN *left = (IN *) in[0]; \
+	IN *right = (IN *) in[1]; \
+	OUT *q = (OUT *) out; \
+	\
+	for( x = 0; x < sz; x++ ) \
+		q[x] = left[x] + right[x]; \
+}
+
+static void
+add_buffer( VipsArithmetic *arithmetic, VipsPel *out, VipsPel **in, int width )
+{
+	VipsArithmeticClass *class = VIPS_ARITHMETIC_GET_CLASS( arithmetic );
+	VipsImage *im = arithmetic->ready[0];
+
+	/* Complex just doubles the size.
+	 */
+	const int sz = width * vips_image_get_bands( im ) * 
+		(vips_band_format_iscomplex( vips_image_get_format( im ) ) ? 
+		 	2 : 1);
+
+	VipsVector *v;
+
+	if( (v = vips_arithmetic_get_vector( class, 
+		vips_image_get_format( im ) )) ) {
+		VipsExecutor ex;
+
+		vips_executor_set_program( &ex, v, sz );
+		vips_executor_set_array( &ex, v->s[0], in[0] );
+		vips_executor_set_array( &ex, v->s[1], in[1] );
+		vips_executor_set_destination( &ex, out );
+
+		vips_executor_run( &ex );
+	}
+	else {
+		int x;
+
+		/* Add all input types. Keep types here in sync with 
+		 * bandfmt_add[] below.
+		 */
+		switch( vips_image_get_format( im ) ) {
+		case VIPS_FORMAT_UCHAR: 	
+			LOOP( unsigned char, unsigned short ); break; 
+		case VIPS_FORMAT_CHAR: 	
+			LOOP( signed char, signed short ); break; 
+		case VIPS_FORMAT_USHORT: 
+			LOOP( unsigned short, unsigned int ); break; 
+		case VIPS_FORMAT_SHORT: 	
+			LOOP( signed short, signed int ); break; 
+		case VIPS_FORMAT_UINT: 	
+			LOOP( unsigned int, unsigned int ); break; 
+		case VIPS_FORMAT_INT: 	
+			LOOP( signed int, signed int ); break; 
+
+		case VIPS_FORMAT_FLOAT: 		
+		case VIPS_FORMAT_COMPLEX: 
+			LOOP( float, float ); break; 
+
+		case VIPS_FORMAT_DOUBLE:	
+		case VIPS_FORMAT_DPCOMPLEX: 
+			LOOP( double, double ); break;
+
+		default:
+			g_assert( 0 );
+		}
+	}
+}
+
+/* Save a bit of typing.
+ */
+#define UC VIPS_FORMAT_UCHAR
+#define C VIPS_FORMAT_CHAR
+#define US VIPS_FORMAT_USHORT
+#define S VIPS_FORMAT_SHORT
+#define UI VIPS_FORMAT_UINT
+#define I VIPS_FORMAT_INT
+#define F VIPS_FORMAT_FLOAT
+#define X VIPS_FORMAT_COMPLEX
+#define D VIPS_FORMAT_DOUBLE
+#define DX VIPS_FORMAT_DPCOMPLEX
+
+/* Type promotion for addition. Sign and value preserving. Make sure these
+ * match the case statement in add_buffer() above.
+ */
+static const VipsBandFormat bandfmt_add[10] = {
+/* UC  C   US  S   UI  I  F  X  D  DX */
+   US, S,  UI, I,  UI, I, F, X, D, DX
+};
+
+static void
+vips_add_class_init( VipsAddClass *class )
+{
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsArithmeticClass *aclass = VIPS_ARITHMETIC_CLASS( class );
+	VipsVector *v;
+
+	object_class->nickname = "add";
+	object_class->description = _( "add two images" );
+
+	vips_arithmetic_set_format_table( aclass, bandfmt_add );
+
+	v = vips_arithmetic_get_program( aclass, VIPS_FORMAT_UCHAR );
+	vips_vector_asm2( v, "convubw", "t1", "s1" );
+	vips_vector_asm2( v, "convubw", "t2", "s2" );
+	vips_vector_asm3( v, "addw", "d1", "t1", "t2" ); 
+
+	v = vips_arithmetic_get_program( aclass, VIPS_FORMAT_CHAR );
+	vips_vector_asm2( v, "convsbw", "t1", "s1" );
+	vips_vector_asm2( v, "convsbw", "t2", "s2" );
+	vips_vector_asm3( v, "addw", "d1", "t1", "t2" ); 
+
+	v = vips_arithmetic_get_program( aclass, VIPS_FORMAT_USHORT );
+	vips_vector_asm2( v, "convuwl", "t1", "s1" );
+	vips_vector_asm2( v, "convuwl", "t2", "s2" );
+	vips_vector_asm3( v, "addl", "d1", "t1", "t2" );
+
+	v = vips_arithmetic_get_program( aclass, VIPS_FORMAT_SHORT );
+	vips_vector_asm2( v, "convswl", "t1", "s1" );
+	vips_vector_asm2( v, "convswl", "t2", "s2" );
+	vips_vector_asm3( v, "addl", "d1", "t1", "t2" );
+
+	/*
+
+	   uint/int are a little slower than C, on a c2d anyway
+
+	   float/double/complex are not handled well
+
+	v = vips_arithmetic_get_vector( aclass, VIPS_FORMAT_UINT );
+	vips_vector_asm3( v, "addl", "d1", "s1", "s2" );
+
+	v = vips_arithmetic_get_vector( aclass, VIPS_FORMAT_INT );
+	vips_vector_asm3( v, "addl", "d1", "s1", "s2" );
+
+	 */
+
+	vips_arithmetic_compile( aclass );
+
+	aclass->process_line = add_buffer;
+}
+
+static void
+vips_add_init( VipsAdd *add )
+{
+}
 
 /**
- * VipsAdd:
- * @in1: input image 
- * @in2: input image 
+ * vips_add:
+ * @left: input image 
+ * @right: input image 
  * @out: output image
+ * @...: %NULL-terminated list of optional named arguments
  *
  * This operation calculates @in1 + @in2 and writes the result to @out. 
  *
@@ -108,7 +254,7 @@
  * following table is used to determine the output type:
  *
  * <table>
- *   <title>im_add() type promotion</title>
+ *   <title>VipsAdd type promotion</title>
  *   <tgroup cols='2' align='left' colsep='1' rowsep='1'>
  *     <thead>
  *       <row>
@@ -167,171 +313,19 @@
  * Operations on integer images are performed using the processor's vector unit,
  * if possible. Disable this with --vips-novector or IM_NOVECTOR.
  *
- * See also: im_subtract(), im_lintra().
+ * See also: vips_subtract(), vips_linear().
+ *
+ * Returns: 0 on success, -1 on error
  */
-
-typedef VipsBinary VipsAdd;
-typedef VipsBinaryClass VipsAddClass;
-
-G_DEFINE_TYPE( VipsAdd, vips_add, VIPS_TYPE_BINARY );
-
-#define LOOP( IN, OUT ) { \
-	IN *p1 = (IN *) left; \
-	IN *p2 = (IN *) right; \
-	OUT *q = (OUT *) out; \
-	\
-	for( x = 0; x < sz; x++ ) \
-		q[x] = p1[x] + p2[x]; \
-}
-
-static void
-add_buffer( VipsBinary *binary, PEL *out, PEL *left, PEL *right, int width )
-{
-	VipsArithmeticClass *class = VIPS_ARITHMETIC_GET_CLASS( binary );
-	VipsImage *im = binary->left_processed;
-
-	/* Complex just doubles the size.
-	 */
-	const int sz = width * im->Bands * 
-		(vips_band_format_iscomplex( im->BandFmt ) ? 2 : 1);
-
-	VipsVector *v;
-
-	if( (v = vips_arithmetic_get_vector( class, im->BandFmt )) ) {
-		VipsExecutor ex;
-
-		vips_executor_set_program( &ex, v, sz );
-		vips_executor_set_array( &ex, v->s[0], left );
-		vips_executor_set_array( &ex, v->s[1], right );
-		vips_executor_set_destination( &ex, out );
-
-		vips_executor_run( &ex );
-	}
-	else {
-		int x;
-
-		/* Add all input types. Keep types here in sync with 
-		 * bandfmt_add[] below.
-		 */
-		switch( im->BandFmt ) {
-		case VIPS_FORMAT_UCHAR: 	
-			LOOP( unsigned char, unsigned short ); break; 
-		case VIPS_FORMAT_CHAR: 	
-			LOOP( signed char, signed short ); break; 
-		case VIPS_FORMAT_USHORT: 
-			LOOP( unsigned short, unsigned int ); break; 
-		case VIPS_FORMAT_SHORT: 	
-			LOOP( signed short, signed int ); break; 
-		case VIPS_FORMAT_UINT: 	
-			LOOP( unsigned int, unsigned int ); break; 
-		case VIPS_FORMAT_INT: 	
-			LOOP( signed int, signed int ); break; 
-
-		case VIPS_FORMAT_FLOAT: 		
-		case VIPS_FORMAT_COMPLEX: 
-			LOOP( float, float ); break; 
-
-		case VIPS_FORMAT_DOUBLE:	
-		case VIPS_FORMAT_DPCOMPLEX: 
-			LOOP( double, double ); break;
-
-		default:
-			g_assert( 0 );
-		}
-	}
-}
-
-/* Save a bit of typing.
- */
-#define UC IM_BANDFMT_UCHAR
-#define C IM_BANDFMT_CHAR
-#define US IM_BANDFMT_USHORT
-#define S IM_BANDFMT_SHORT
-#define UI IM_BANDFMT_UINT
-#define I IM_BANDFMT_INT
-#define F IM_BANDFMT_FLOAT
-#define X IM_BANDFMT_COMPLEX
-#define D IM_BANDFMT_DOUBLE
-#define DX IM_BANDFMT_DPCOMPLEX
-
-/* Type promotion for addition. Sign and value preserving. Make sure these
- * match the case statement in add_buffer() above.
- */
-static int bandfmt_add[10] = {
-/* UC  C   US  S   UI  I  F  X  D  DX */
-   US, S,  UI, I,  UI, I, F, X, D, DX
-};
-
-static void
-vips_add_class_init( VipsAddClass *class )
-{
-	VipsObjectClass *object_class = (VipsObjectClass *) class;
-	VipsArithmeticClass *aclass = VIPS_ARITHMETIC_CLASS( class );
-	VipsBinaryClass *bclass = VIPS_BINARY_CLASS( class );
-	VipsVector *v;
-
-	object_class->nickname = "add";
-	object_class->description = _( "add two images" );
-
-	vips_arithmetic_set_format_table( aclass, bandfmt_add );
-
-	v = vips_arithmetic_get_program( aclass, VIPS_FORMAT_UCHAR );
-	vips_vector_asm2( v, "convubw", "t1", "s1" );
-	vips_vector_asm2( v, "convubw", "t2", "s2" );
-	vips_vector_asm3( v, "addw", "d1", "t1", "t2" ); 
-
-	v = vips_arithmetic_get_program( aclass, VIPS_FORMAT_CHAR );
-	vips_vector_asm2( v, "convsbw", "t1", "s1" );
-	vips_vector_asm2( v, "convsbw", "t2", "s2" );
-	vips_vector_asm3( v, "addw", "d1", "t1", "t2" ); 
-
-	v = vips_arithmetic_get_program( aclass, VIPS_FORMAT_USHORT );
-	vips_vector_asm2( v, "convuwl", "t1", "s1" );
-	vips_vector_asm2( v, "convuwl", "t2", "s2" );
-	vips_vector_asm3( v, "addl", "d1", "t1", "t2" );
-
-	v = vips_arithmetic_get_program( aclass, VIPS_FORMAT_SHORT );
-	vips_vector_asm2( v, "convswl", "t1", "s1" );
-	vips_vector_asm2( v, "convswl", "t2", "s2" );
-	vips_vector_asm3( v, "addl", "d1", "t1", "t2" );
-
-	/*
-
-	   uint/int are a little slower than C, on a c2d anyway
-
-	   float/double/complex are not handled well
-
-	v = vips_arithmetic_get_vector( aclass, VIPS_FORMAT_UINT );
-	vips_vector_asm3( v, "addl", "d1", "s1", "s2" );
-
-	v = vips_arithmetic_get_vector( aclass, VIPS_FORMAT_INT );
-	vips_vector_asm3( v, "addl", "d1", "s1", "s2" );
-
-	 */
-
-	vips_arithmetic_compile( aclass );
-
-	bclass->process_line = add_buffer;
-}
-
-VipsImage *
-vips_add( VipsImage *in1, VipsImage *in2, ... )
+int
+vips_add( VipsImage *left, VipsImage *right, VipsImage **out, ... )
 {
 	va_list ap;
 	int result;
-	VipsImage *out;
 
-	va_start( ap, in2 );
-	result = vips_call_split( "add", ap, in1, in2, &out );
+	va_start( ap, out );
+	result = vips_call_split( "add", ap, left, right, out );
 	va_end( ap );
 
-	if( result )
-		return( NULL );
-
-	return( out );
-}
-
-static void
-vips_add_init( VipsAdd *add )
-{
+	return( result );
 }
