@@ -50,10 +50,6 @@
 #include <math.h>
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC */
-
 
 /** TYPES **/
 
