@@ -53,19 +53,15 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* imb_LabQ2Lab: CONVERT n pels from packed 32bit Lab to float values
  * in a buffer
- * ARGS:   PEL *inp       pointer to first byte of Lab32 buffer
+ * ARGS:   VipsPel *inp       pointer to first byte of Lab32 buffer
  * float *outbuf   destination buffer
  *	int n           number of pels to process
  * (C) K.Martinez 2/5/93
  */
 void
-imb_LabQ2Lab( PEL *inp, float *outbuf, int n )        
+imb_LabQ2Lab( VipsPel *inp, float *outbuf, int n )        
 {
 	signed char *b;		/* to read input bytes */
 	int l;
