@@ -62,14 +62,10 @@
 #include <vips/vips.h>
 #include <vips/internal.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Process a buffer of data.
  */
 void
-imb_XYZ2disp( float *p, PEL *q, int n, struct im_col_display *d )
+imb_XYZ2disp( float *p, VipsPel *q, int n, struct im_col_display *d )
 {
 	struct im_col_tab_disp *table = im_col_display_get_table( d );
 	float rstep = (d->d_YCR - d->d_Y0R) / 1500.0;
