@@ -68,10 +68,6 @@
 #include <vips/region.h>
 #include <vips/util.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /**
  * VipsIntent:
  * @IM_INTENT_PERCEPTUAL:
@@ -303,7 +299,7 @@ decode_lab( WORD *fixed, float *lab, int n )
 }
 
 static void
-transform_buf( PEL *in, PEL *out, int n, Icc *icc )
+transform_buf( VipsPel *in, VipsPel *out, int n, Icc *icc )
 {
 	g_mutex_lock( icc->lock );
 	cmsDoTransform( icc->trans, in, out, n );
@@ -319,11 +315,8 @@ attach_profile( IMAGE *im, const char *filename )
 	if( !(data = im__file_read_name( filename, VIPS_ICC_DIR, 
 		&data_length )) ) 
 		return( -1 );
-	if( im_meta_set_blob( im, IM_META_ICC_NAME, 
-		(im_callback_fn) im_free, data, data_length ) ) {
-		im_free( data );
-		return( -1 );
-	}
+	im_meta_set_blob( im, IM_META_ICC_NAME, 
+		(im_callback_fn) im_free, data, data_length );
 
 	return( 0 );
 }
@@ -469,7 +462,7 @@ im_icc_transform( IMAGE *in, IMAGE *out,
 }
 
 static void
-import_buf( PEL *in, float *out, int n, Icc *icc )
+import_buf( VipsPel *in, float *out, int n, Icc *icc )
 {
 	/* Buffer of encoded 16-bit pixels we write to.
 	 */
@@ -651,7 +644,7 @@ im_icc_import_embedded( IMAGE *in, IMAGE *out, VipsIntent intent )
 }
 
 static void
-export_buf( float *in, PEL *out, int n, Icc *icc )
+export_buf( float *in, VipsPel *out, int n, Icc *icc )
 {
 	/* Buffer of encoded 16-bit pixels we transform.
 	 */
@@ -878,10 +871,6 @@ im_icc_ac2rc( IMAGE *in, IMAGE *out, const char *profile_filename )
 #include <vips/region.h>
 #include <vips/util.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /**
  * VipsIntent:
  * @IM_INTENT_PERCEPTUAL: perceptual rendering intent
@@ -1114,7 +1103,7 @@ decode_lab( guint16 *fixed, float *lab, int n )
 }
 
 static void
-transform_buf( PEL *in, PEL *out, int n, Icc *icc )
+transform_buf( VipsPel *in, VipsPel *out, int n, Icc *icc )
 {
 	g_mutex_lock( icc->lock );
 	cmsDoTransform( icc->trans, in, out, n );
@@ -1284,7 +1273,7 @@ im_icc_transform( IMAGE *in, IMAGE *out,
 }
 
 static void
-import_buf( PEL *in, float *out, int n, Icc *icc )
+import_buf( VipsPel *in, float *out, int n, Icc *icc )
 {
 	/* Buffer of encoded 16-bit pixels we write to.
 	 */
@@ -1448,7 +1437,7 @@ im_icc_import_embedded( IMAGE *in, IMAGE *out, VipsIntent intent )
 }
 
 static void
-export_buf( float *in, PEL *out, int n, Icc *icc )
+export_buf( float *in, VipsPel *out, int n, Icc *icc )
 {
 	/* Buffer of encoded 16-bit pixels we transform.
 	 */
