@@ -45,10 +45,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 typedef struct _Grid {
 	IMAGE *in;
 	IMAGE *out;
