@@ -47,10 +47,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Maximum width of input we ask for.
  */
 #define IM_MAX_WIDTH (100)
@@ -89,8 +85,8 @@ line_shrink_gen( REGION *or, void *seq, void *a, void *b )
 	/* Loop down the region.
 	 */
 	for( y = to; y < bo; y++ ) {
-		char *q = IM_REGION_ADDR( or, le, y );
-		char *p;
+		VipsPel *q = IM_REGION_ADDR( or, le, y );
+		VipsPel *p;
 
 		/* Loop across the region, in owidth sized pieces.
 		 */
@@ -153,8 +149,8 @@ point_shrink_gen( REGION *or, void *seq, void *a, void *b )
 	/* Loop down the region.
 	 */
 	for( y = to; y < bo; y++ ) {
-		char *q = IM_REGION_ADDR( or, le, y );
-		char *p;
+		VipsPel *q = IM_REGION_ADDR( or, le, y );
+		VipsPel *p;
 
 		/* Loop across the region, in owidth sized pieces.
 		 */
