@@ -49,10 +49,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 #define IM_MAX_STRSIZE (4096)
 
 static int
@@ -130,7 +126,7 @@ system_image( IMAGE *im,
  *
  * See also: im_system().
  *
- * Returns: an image on success, NULL on error
+ * Returns: (transfer full): an image on success, NULL on error
  */
 IMAGE *
 im_system_image( IMAGE *im, 
