@@ -53,10 +53,6 @@
 #include <pango/pangoft2.h>
 #endif /*HAVE_PANGOFT2*/
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 #ifdef HAVE_PANGOFT2
 
 static PangoLayout *
@@ -98,7 +94,7 @@ text_ft_to_vips( FT_Bitmap *bitmap, IMAGE *out )
 
 	for( y = 0; y < bitmap->rows; y++ ) 
 		if( im_writeline( y, out, 
-			(PEL *) bitmap->buffer + y * bitmap->pitch ) )
+			(VipsPel *) bitmap->buffer + y * bitmap->pitch ) )
 			return( -1 );
 
 	return( 0 );
