@@ -72,9 +72,9 @@ $ vips im_max abs.v
  */
 
 /*
- */
 #define DEBUG
 #define VIPS_DEBUG
+ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -91,10 +91,6 @@ $ vips im_max abs.v
 #include <vips/vector.h>
 #include <vips/debug.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Maximum number of boxes we can break the mask into.
  */
 #define MAX_LINES (10000)
@@ -676,8 +672,7 @@ boxes_new( IMAGE *in, IMAGE *out, DOUBLEMASK *mask, int n_layers, int cluster )
 		vips_check_dmask( "im_aconv", mask ) ) 
 		return( NULL );
 
-	if( !(boxes = VIPS_NEW( out, Boxes )) )
-		return( NULL );
+	boxes = VIPS_NEW( out, Boxes );
 	boxes->in = in;
 	boxes->out = out;
 	if( !(boxes->mask = (DOUBLEMASK *) im_local( out, 
