@@ -77,10 +77,6 @@
 #include <vips/vector.h>
 #include <vips/debug.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Maximum number of lines we can break the mask into.
  */
 #define MAX_LINES (1000)
@@ -162,8 +158,7 @@ lines_new( IMAGE *in, IMAGE *out, DOUBLEMASK *mask, int n_layers )
 		vips_check_dmask_1d( "im_aconvsep", mask ) ) 
 		return( NULL );
 
-	if( !(lines = VIPS_NEW( out, Lines )) )
-		return( NULL );
+	lines = VIPS_NEW( out, Lines );
 	lines->in = in;
 	lines->out = out;
 	if( !(lines->mask = (DOUBLEMASK *) im_local( out, 
