@@ -41,6 +41,8 @@
  * 29/10/10
  * 	- get rid of im_convsep_f(), just call this twice, no longer worth
  * 	  keeping two versions
+ * 15/10/11 Nicolas
+ * 	- handle offset correctly in seperable convolutions
  */
 
 /*
@@ -80,10 +82,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Our parameters ... we take a copy of the mask argument, plus we make a
  * smaller version with the zeros squeezed out. 
  */
@@ -156,7 +154,7 @@ typedef struct {
 	REGION *ir;		/* Input region */
 
 	int *offsets;		/* Offsets for each non-zero matrix element */
-	PEL **pts;		/* Per-non-zero mask element image pointers */
+	VipsPel **pts;		/* Per-non-zero mask element image pointers */
 
 	int last_bpl;		/* Avoid recalcing offsets, if we can */
 } ConvSequence;
@@ -196,7 +194,7 @@ conv_start( IMAGE *out, void *a, void *b )
 	 */
 	seq->ir = im_region_create( in );
 	seq->offsets = IM_ARRAY( out, conv->nnz, int );
-	seq->pts = IM_ARRAY( out, conv->nnz, PEL * );
+	seq->pts = IM_ARRAY( out, conv->nnz, VipsPel * );
 	if( !seq->ir || !seq->offsets || !seq->pts ) {
 		conv_stop( seq, in, conv );
 		return( NULL );
@@ -280,8 +278,8 @@ conv_gen( REGION *or, void *vseq, void *a, void *b )
 		/* Init pts for this line of PELs.
 		 */
                 for( z = 0; z < conv->nnz; z++ ) 
-                        seq->pts[z] = seq->offsets[z] +  
-                                (PEL *) IM_REGION_ADDR( ir, le, y ); 
+                        seq->pts[z] = seq->offsets[z] + 
+				IM_REGION_ADDR( ir, le, y ); 
 
 		switch( in->BandFmt ) {
 		case IM_BANDFMT_UCHAR: 	
@@ -414,9 +412,10 @@ im_convsep_f_raw( IMAGE *in, IMAGE *out, DOUBLEMASK *mask )
 
 	rmask->xsize = mask->ysize;
 	rmask->ysize = mask->xsize;
+        rmask->offset = 0.;
 
-	if( im_conv_f_raw( in, t, mask ) ||
-		im_conv_f_raw( t, out, rmask ) )
+	if( im_conv_f_raw( in, t, rmask ) ||
+		im_conv_f_raw( t, out, mask ) )
 		return( -1 );
 
 	return( 0 );
