@@ -82,10 +82,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* A lut --- we need indexes in the range [-x3,x2], so add x3 to indexes
  * before starting to index table.
  */
@@ -287,7 +283,7 @@ im_sharpen( IMAGE *in, IMAGE *out,
 	int mask_size, 
 	double x1, double y2, double y3, 
 	double m1, double m2 )
-{ 
+{
 	IMAGE *arry[3];
 	IMAGE *t[4];
 	INTMASK *mask;
