@@ -74,10 +74,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Hold per-call state here.
  */
 typedef struct {
@@ -103,7 +99,7 @@ typedef struct {
 			sum1 += a1[i]; \
 		a1 += in_lsk;  \
 	} \
-	imean = sum1 / (ref->Xsize * ref->Ysize); \
+	imean = sum1 / VIPS_IMAGE_N_PELS( ref ); \
  	\
 	/* Loop over ir again, this time calculating  \
 	 * sum-of-squares-of-differences for this window on \
@@ -175,7 +171,7 @@ spcor_gen( REGION *or, void *vseq, void *a, void *b )
 		float *q = (float *) IM_REGION_ADDR( or, le, y );
 
 		for( x = le; x < ri; x++ ) {
-			PEL *p = (PEL *) IM_REGION_ADDR( ir, x, y );
+			VipsPel *p = IM_REGION_ADDR( ir, x, y );
 
 			/* Find sums for this position.
 			 */
@@ -206,8 +202,8 @@ static Spcor *
 spcor_new( IMAGE *out, IMAGE *ref )
 {
 	Spcor *spcor;
-	size_t sz = ref->Xsize * ref->Ysize;
-	PEL *p = (PEL *) ref->data;
+	guint64 sz = VIPS_IMAGE_N_PELS( ref );
+	VipsPel *p = ref->data;
 	double s;
 	size_t i;
 
