@@ -46,10 +46,7 @@
 #include <string.h>
 
 #include <vips/vips.h>
-
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
+#include <vips/internal.h>
 
 /* Max str we parse.
  */
@@ -113,7 +110,7 @@ input_image_init( im_object *obj, char *str )
 {
 	IMAGE **im = (IMAGE **) obj;
 
-	return( !(*im = im_open( str, "rd" )) );
+	return( !(*im = vips__deprecated_open_read( str )) );
 }
 
 /* Input image type.
@@ -133,7 +130,7 @@ output_image_init( im_object *obj, char *str )
 {
 	IMAGE **im = (IMAGE **) obj;
 
-	return( !(*im = im_open( str, "w" )) );
+	return( !(*im = vips__deprecated_open_write( str )) );
 }
 
 /* Output image type.
@@ -182,7 +179,7 @@ imagevec_dest( im_object obj )
 				iv->vec[i] = NULL;
 			}
 
-		vips_free( iv->vec );
+		g_free( iv->vec );
 		iv->vec = NULL;
 		iv->n = 0;
 	}
@@ -244,7 +241,7 @@ mask_init( im_object *obj, char *str )
 
 	/* Install string, clear mask.
 	 */
-	if( str && !(mo->name = vips_strdup( NULL, str )) ) 
+	if( str && !(mo->name = im_strdup( NULL, str )) ) 
 		return( -1 );
 	mo->mask = NULL;
 
@@ -288,14 +285,8 @@ dmask_dest( im_object obj )
 {
 	im_mask_object *mo = obj;
 
-	if( mo->name ) {
-		vips_free( mo->name );
-		mo->name = NULL;
-	}
-	if( mo->mask ) {
-		im_free_dmask( (DOUBLEMASK *) mo->mask );
-		mo->mask = NULL;
-	}
+	VIPS_FREE( mo->name );
+	VIPS_FREEF( im_free_dmask, mo->mask );
 
 	return( 0 );
 }
@@ -307,14 +298,8 @@ imask_dest( im_object obj )
 {
 	im_mask_object *mo = obj;
 
-	if( mo->name ) {
-		vips_free( mo->name );
-		mo->name = NULL;
-	}
-	if( mo->mask ) {
-		im_free_imask( (INTMASK *) mo->mask );
-		mo->mask = NULL;
-	}
+	VIPS_FREE( mo->name );
+	VIPS_FREEF( im_free_imask, mo->mask );
 
 	return( 0 );
 }
@@ -424,7 +409,7 @@ doublevec_dest( im_object obj )
 	im_doublevec_object *dv = obj;
 
 	if( dv->vec ) {
-		vips_free( dv->vec );
+		g_free( dv->vec );
 		dv->vec = NULL;
 		dv->n = 0;
 	}
@@ -509,7 +494,7 @@ intvec_dest( im_object obj )
 	im_intvec_object *iv = obj;
 
 	if( iv->vec ) {
-		vips_free( iv->vec );
+		g_free( iv->vec );
 		iv->vec = NULL;
 		iv->n = 0;
 	}
@@ -622,7 +607,7 @@ im_type_desc im__input_int = {
 static int
 input_string_init( im_object *obj, char *str )
 {
-	if( !(*obj = (im_object) vips_strdup( NULL, str )) ) 
+	if( !(*obj = (im_object) im_strdup( NULL, str )) ) 
 		return( -1 );
 
 	return( 0 );
@@ -737,11 +722,10 @@ im__dmsprint( im_object obj )
 
 	/* Print statistics band stats eg: 2 bands:b 0,1 
 	 */
-	printf( "\
-band    minimum     maximum         sum       sum^2        mean   deviation\
-\n" );
+	printf( "band    minimum     maximum         sum       "
+		"sum^2        mean   deviation\n" );
 	for( j = 0; j < mask->ysize; j++ ) {
-		row = mask->coeff + j * 6;
+		row = mask->coeff + j * mask->xsize;
 		if( j == 0 )
 			printf( "all" );
 		else
@@ -833,7 +817,11 @@ gvalue_free( im_object obj )
 {
 	GValue *value = obj;
 
-	g_value_unset( value );
+	/* If it's just zeros (built but not used) we'll get an error if we
+	 * unset().
+	 */
+	if( G_IS_VALUE( value ) )
+		g_value_unset( value );
 
 	return( 0 );
 }
@@ -886,12 +874,18 @@ im_type_desc im__output_gvalue = {
 static int
 input_interpolate_init( im_object *obj, char *str )
 {
-	VipsObjectClass *interpolate_class;
+	GType type = g_type_from_name( "VipsInterpolate" );
+	VipsObjectClass *class = VIPS_OBJECT_CLASS( g_type_class_ref( type ) );
 	VipsObject *object;
 
-	interpolate_class = vips_class_find( "VipsInterpolate", "interpolate" );
-	if( !(object = vips_object_new_from_string( interpolate_class, str )) )
+	g_assert( class );
+
+	if( !(object = vips_object_new_from_string( class, str )) )
 		return( -1 );
+	if( vips_object_build( object ) ) {
+		g_object_unref( object );
+		return( -1 );
+	}
 	*obj = object;
 
 	return( 0 );
