@@ -61,10 +61,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /*               uchar char ushort short uint   int float double */
 static int array[8][8] = {
 /* uchar */     {  0,    0,    0,    0,    0,    0,    0,    1 },
