@@ -56,15 +56,10 @@
 #include <vips/internal.h>
 #include <vips/debug.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Standard VIPS packages.
  */
 extern im_package im__arithmetic;
 extern im_package im__cimg;
-extern im_package im__boolean;
 extern im_package im__colour;
 extern im_package im__conversion;
 extern im_package im__convolution;
@@ -77,7 +72,6 @@ extern im_package im__mask;
 extern im_package im__morphology;
 extern im_package im__mosaicing;
 extern im_package im__other;
-extern im_package im__relational;
 extern im_package im__resample;
 extern im_package im__video;
 
@@ -101,7 +95,7 @@ guess_prefix_vec( im_object *argv )
 		return( -1 );
 	}
 
-	argv[2] = vips_strdup( NULL, prefix );
+	argv[2] = im_strdup( NULL, prefix );
 
 	return( 0 );
 }
@@ -137,7 +131,7 @@ guess_libdir_vec( im_object *argv )
 		return( -1 );
 	}
 
-	argv[2] = vips_strdup( NULL, libdir );
+	argv[2] = im_strdup( NULL, libdir );
 
 	return( 0 );
 }
@@ -293,7 +287,7 @@ history_get_vec( im_object *argv )
 	const char *str;
 
 	if( !(str = im_history_get( (IMAGE *) argv[0] )) ||
-		!(*out = vips_strdup( NULL, str )) )
+		!(*out = im_strdup( NULL, str )) )
 		return( -1 );
 
 	return( 0 );
@@ -355,7 +349,7 @@ static im_arg_desc printdesc_args[] = {
 static int
 printdesc_vec( im_object *argv )
 {
-	vips_object_print( VIPS_OBJECT( argv[0] ) );
+	vips_object_print_dump( VIPS_OBJECT( argv[0] ) );
 
 	return( 0 );
 }
@@ -382,7 +376,7 @@ static im_arg_desc version_string_args[] = {
 static int
 version_string_vec( im_object *argv )
 {
-	if( !(argv[0] = vips_strdup( NULL, vips_version_string() )) )
+	if( !(argv[0] = im_strdup( NULL, vips_version_string() )) )
 		return( -1 );
 
 	return( 0 );
@@ -573,7 +567,6 @@ static im_package im__iofuncs = {
  */
 static im_package *built_in[] = {
 	&im__arithmetic,
-	&im__boolean,
 #ifdef ENABLE_CXX
 	&im__cimg,
 #endif /*ENABLE_CXX*/
@@ -590,7 +583,6 @@ static im_package *built_in[] = {
 	&im__morphology,
 	&im__mosaicing,
 	&im__other,
-	&im__relational,
 	&im__resample,
 	&im__video
 };
@@ -625,7 +617,7 @@ plugin_free( Plugin *plug )
 	}
 	VIPS_FREE( plug->name );
 	plug->pack = NULL;
-	vips_free( plug );
+	g_free( plug );
 
 	plugin_list = g_slist_remove( plugin_list, plug );
 
@@ -647,24 +639,17 @@ im_load_plugin( const char *name )
 
 	/* Build a new plugin.
 	 */
-	if( !(plug = VIPS_NEW( NULL, Plugin )) ) 
-		return( NULL );
+	plug = VIPS_NEW( NULL, Plugin );
 	plug->module = NULL;
-	plug->name = NULL;
+	plug->name = g_strdup( name );
 	plug->pack = NULL;
 	plugin_list = g_slist_prepend( plugin_list, plug );
 
-	/* Attach name.
-	 */
-	if( !(plug->name = vips_strdup( NULL, name )) ) {
-		plugin_free( plug );
-		return( NULL );
-	}
-
 	/* Open library.
 	 */
 	if( !(plug->module = g_module_open( name, 0 )) ) {
-		vips_error( "plugin", _( "unable to open plugin \"%s\"" ), name );
+		vips_error( "plugin", 
+			_( "unable to open plugin \"%s\"" ), name );
 		vips_error( "plugin", "%s", g_module_error() );
 		plugin_free( plug );
 
