@@ -39,10 +39,6 @@
 #include <vips/vips.h>
 #include <vips/internal.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 int 
 im_remainderconst_vec( IMAGE *in, IMAGE *out, int n, double *c )
 {
@@ -405,7 +401,7 @@ im_iterate( IMAGE *im,
 	im_start_fn start, im_generate_fn generate, im_stop_fn stop,
 	void *b, void *c )
 {
-	return( vips_sink( im, start, generate, stop, b, c ) );
+	return( vips_sink( im, start, (VipsGenerateFn) generate, stop, b, c ) );
 }
 
 int
