@@ -34,7 +34,6 @@
  
    	TODO:
 
-	- works for iamges, but not tested for much else
 	- masks would be hard, vips8 won't really have these
 	- keep for testing, mostly ... iofuncs/init.c has a commeted-out
 	  call to init this thing
@@ -61,10 +60,6 @@
 #include <vips/debug.h>
 #include <vips/vector.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 #define VIPS_WRAP7_PREFIX "VipsWrap7_"
 
 static GHashTable *vips_wrap7_subclass_types = NULL;
@@ -127,6 +122,8 @@ typedef enum {
 	VIPS_WRAP7_INTERPOLATE
 } VipsWrap7Type;
 
+/* Order must match VipsWrap7Type enums.
+ */
 static char *vips_wrap7_supported[] = {
 	IM_TYPE_DOUBLE,
 	IM_TYPE_INT,
@@ -355,7 +352,7 @@ vips_wrap7_object_set_property( GObject *gobject,
 	 */
 	if( argument_class->flags & VIPS_ARGUMENT_CONSTRUCT &&
 		object->constructed &&
-		!vips_pspec_value_is_null( pspec, value ) ) {
+		!vips_value_is_null( pspec, value ) ) {
 		g_warning( "%s: %s can't assign '%s' after construct",
 			G_STRLOC,
 			G_OBJECT_TYPE_NAME( gobject ),
@@ -367,7 +364,7 @@ vips_wrap7_object_set_property( GObject *gobject,
 	 */
 	if( argument_class->flags & VIPS_ARGUMENT_SET_ONCE &&
 		argument_instance->assigned &&
-		!vips_pspec_value_is_null( pspec, value ) ) {
+		!vips_value_is_null( pspec, value ) ) {
 		g_warning( "%s: %s can only assign '%s' once",
 			G_STRLOC,
 			G_OBJECT_TYPE_NAME( gobject ),
@@ -385,7 +382,7 @@ vips_wrap7_object_set_property( GObject *gobject,
 		break;
 
 	case VIPS_WRAP7_STRING:
-		VIPS_SETSTR( wrap7->vargv[i], g_value_get_string( value ) );
+		IM_SETSTR( wrap7->vargv[i], g_value_get_string( value ) );
 		break;
 
 	case VIPS_WRAP7_GVALUE:
@@ -483,10 +480,11 @@ vips_wrap7_build_output( VipsObject *object,
 {
 	VipsWrap7 *wrap7 = VIPS_WRAP7( object );  
 	VipsWrap7Class *class = VIPS_WRAP7_GET_CLASS( wrap7 );
-	int i = argument_class->offset;
-	im_arg_desc *arg = &class->fn->argv[i];
-	im_type_desc *type = arg->desc;
-	im_arg_type vt = type->type;
+
+	int i;
+	im_arg_desc *arg;
+	im_type_desc *type;
+	im_arg_type vt;
 
 	/* We want required, construct-time, unassigned output args.
 	 */
@@ -496,6 +494,14 @@ vips_wrap7_build_output( VipsObject *object,
 		!(argument_class->flags & VIPS_ARGUMENT_OUTPUT) )
 		return( NULL ); 
 
+	/* argument_class->offset is only the argv slot for true params. We
+	 * can't do this before we've tested the arg class.
+	 */
+	i = argument_class->offset;
+	arg = &class->fn->argv[i];
+	type = arg->desc;
+	vt = type->type;
+
 	/* Provide output objects for the operation to write to.
 	 */
 	switch( vips_wrap7_lookup_type( vt ) ) {
@@ -532,6 +538,33 @@ vips_wrap7_build_output( VipsObject *object,
 	return( NULL );
 }
 
+/* Mark an output as assigned.
+ */
+static void *
+vips_wrap7_note_assigned( VipsObject *object,
+	GParamSpec *pspec,
+	VipsArgumentClass *argument_class,
+	VipsArgumentInstance *argument_instance,
+	void *a, void *b )
+{
+	/* We want required, construct-time, output args.
+	 */
+	if( !(argument_class->flags & VIPS_ARGUMENT_REQUIRED) ||
+		!(argument_class->flags & VIPS_ARGUMENT_CONSTRUCT) ||
+		!(argument_class->flags & VIPS_ARGUMENT_OUTPUT) )
+		return( NULL ); 
+
+	if( !argument_instance->assigned ) {
+		argument_instance->assigned = TRUE;
+
+		VIPS_DEBUG_MSG( "vips_wrap7_note_assigned: "
+			"marking %s as assigned\n", 
+			g_param_spec_get_name( pspec ) );
+	}
+
+	return( NULL );
+}
+
 static int
 vips_wrap7_build( VipsObject *object )
 {
@@ -547,8 +580,9 @@ vips_wrap7_build( VipsObject *object )
 	}
 
 	if( class->not_supported ) {
-		vips_error( "wrap7", _( "unable to call vips7 operation "
-			"%s from vips8" ), oclass->nickname );
+		vips_error( "wrap7", 
+			_( "vips7 operation %s not supported by vips8" ), 
+			oclass->nickname );
 		return( -1 );
 	}
 
@@ -567,14 +601,26 @@ vips_wrap7_build( VipsObject *object )
 	if( VIPS_OBJECT_CLASS( vips_wrap7_parent_class )->build( object ) )
 		return( -1 );
 
+	/* Run the operation.
+	 */
 	if( class->fn->disp( wrap7->vargv ) )
 		return( -1 );
 
+	/* The vips7 operation will have written any outputs (such as the
+	 * output double for im_max(), for example) to varg. g_object_get()
+	 * fetches directly from vargv (see vips_wrap7_object_get_property())
+	 * so we don't need to do any copying, we just need to tell vips that
+	 * all outputs now have values. 
+	 */
+	(void) vips_argument_map( VIPS_OBJECT( wrap7 ),
+		vips_wrap7_note_assigned, 
+		NULL, NULL ); 
+
 	return( 0 );
 }
 
 static void
-vips_wrap7_print_class( VipsObjectClass *oclass, VipsBuf *buf )
+vips_wrap7_summary_class( VipsObjectClass *oclass, VipsBuf *buf )
 {
 	VipsWrap7Class *class = VIPS_WRAP7_CLASS( oclass );
 	im_function *fn = class->fn;
@@ -595,13 +641,13 @@ vips_wrap7_print_class( VipsObjectClass *oclass, VipsBuf *buf )
 }
 
 static void
-vips_wrap7_print( VipsObject *object, VipsBuf *buf )
+vips_wrap7_dump( VipsObject *object, VipsBuf *buf )
 {
 	VipsWrap7Class *class = VIPS_WRAP7_GET_CLASS( object );
 	im_function *fn = class->fn;
 	im_package *pack = im_package_of_function( fn->name );
 
-	VIPS_OBJECT_CLASS( vips_wrap7_parent_class )->print( object, buf );
+	VIPS_OBJECT_CLASS( vips_wrap7_parent_class )->dump( object, buf );
 
 	if( pack )
 		vips_buf_appendf( buf, "from package \"%s\"", pack->name );
@@ -642,8 +688,8 @@ vips_wrap7_class_init( VipsWrap7Class *class )
 	vobject_class->nickname = "wrap7";
 	vobject_class->description = _( "vips7 operations as vips8 classes" );
 	vobject_class->build = vips_wrap7_build;
-	vobject_class->print_class = vips_wrap7_print_class;
-	vobject_class->print = vips_wrap7_print;
+	vobject_class->summary_class = vips_wrap7_summary_class;
+	vobject_class->dump = vips_wrap7_dump;
 }
 
 static void
@@ -654,65 +700,6 @@ vips_wrap7_init( VipsWrap7 *wrap7 )
 /* Build a subclass of vips7 for every vips7 operation.
  */
 
-static gboolean
-drop_postfix( char *str, const char *postfix )
-{
-	if( vips_ispostfix( str, postfix ) ) {
-		str[strlen( str ) - strlen( postfix )] = '\0';
-
-		return( TRUE );
-	}
-
-	return( FALSE );
-}
-
-/* Turn a vips7 name into a nickname. Eg. im_lintra_vec becomes lin.
- */
-static void
-vips_wrap7_nickname( const char *in, char *out )
-{
-	static const char *dont_drop[] = {
-		"_set",
-	};
-	static const char *drop[] = {
-		"_vec",
-		"const",
-		"tra",
-		"set",
-		"_f"
-	};
-
-	int i;
-	gboolean changed;
-
-	/* Copy, chopping off "im_" prefix.
-	 */
-	if( vips_isprefix( "im_", in ) )
-		strcpy( out, in + 3 );
-	else
-		strcpy( out, in );
-
-	/* Repeatedly drop postfixes while we can. Stop if we see a dont_drop
-	 * postfix.
-	 */
-	do {
-		gboolean found;
-
-		found = FALSE;
-		for( i = 0; i < IM_NUMBER( dont_drop ); i++ )
-			if( vips_ispostfix( out, dont_drop[i] ) ) {
-				found = TRUE;
-				break;
-			}
-		if( found )
-			break;
-
-		changed = FALSE;
-		for( i = 0; i < IM_NUMBER( drop ); i++ )
-			changed |= drop_postfix( out, drop[i] );
-	} while( changed );
-}
-
 static void
 vips_wrap7_subclass_class_init( VipsWrap7Class *class )
 {
@@ -725,7 +712,6 @@ vips_wrap7_subclass_class_init( VipsWrap7Class *class )
 		strlen( VIPS_WRAP7_PREFIX );
 	im_function *fn = im_find_function( name );
 
-	char nickname[4096];
 	int i;
 
 	g_assert( !class->fn );
@@ -734,8 +720,7 @@ vips_wrap7_subclass_class_init( VipsWrap7Class *class )
 	gobject_class->set_property = vips_wrap7_object_set_property;
 	gobject_class->get_property = vips_wrap7_object_get_property;
 
-	vips_wrap7_nickname( name, nickname );
-	vobject_class->nickname = im_strdup( NULL, nickname );
+	vobject_class->nickname = im_strdup( NULL, name );
 	vobject_class->description = fn->desc;
 
 	class->fn = fn;
@@ -755,8 +740,6 @@ vips_wrap7_subclass_class_init( VipsWrap7Class *class )
 		case VIPS_WRAP7_INTVEC:
 		case VIPS_WRAP7_GVALUE:
 		case VIPS_WRAP7_INTERPOLATE:
-		case VIPS_WRAP7_DOUBLE:
-		case VIPS_WRAP7_INT:
 		case VIPS_WRAP7_COMPLEX:
 		case VIPS_WRAP7_STRING:
 		case VIPS_WRAP7_NONE:
@@ -764,6 +747,23 @@ vips_wrap7_subclass_class_init( VipsWrap7Class *class )
 			 * set a flag to block _build().
 			 */
 			class->not_supported = TRUE;
+			pspec = NULL;
+			break;
+
+		case VIPS_WRAP7_INT:
+			pspec = g_param_spec_int( arg->name, 
+				arg->name, 
+				arg->name,
+				-G_MAXINT, G_MAXINT, 0,
+				G_PARAM_READWRITE );
+			break;
+
+		case VIPS_WRAP7_DOUBLE:
+			pspec = g_param_spec_double( arg->name, 
+				arg->name, 
+				arg->name,
+				-G_MAXDOUBLE, G_MAXDOUBLE, 0.0,
+				G_PARAM_READWRITE );
 			break;
 
 		case VIPS_WRAP7_IMAGE:
@@ -772,6 +772,13 @@ vips_wrap7_subclass_class_init( VipsWrap7Class *class )
 				arg->name,
 				VIPS_TYPE_IMAGE,
 				G_PARAM_READWRITE );
+			break;
+
+		default:
+			g_assert( 0 );
+		}
+
+		if( pspec ) {
 			g_object_class_install_property( gobject_class, 
 				i + 1, pspec );
 			vips_object_class_install_argument( vobject_class, 
@@ -779,11 +786,7 @@ vips_wrap7_subclass_class_init( VipsWrap7Class *class )
 				(type->flags & IM_TYPE_OUTPUT) ?
 					VIPS_ARGUMENT_REQUIRED_OUTPUT : 
 					VIPS_ARGUMENT_REQUIRED_INPUT,
-				i );
-			break;
-
-		default:
-			g_assert( 0 );
+				i, i );
 		}
 	}
 }
@@ -852,3 +855,4 @@ vips__init_wrap7_classes( void )
 {
 	(void) im_map_packages( (VSListMap2Fn) vips_wrap7_build_package, NULL );
 }
+
