@@ -75,10 +75,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /************************************************************************/
 /* malloc space and create normalised coefficients accross		*/
 /* the x (horizontal) and y (vertical) direction.			*/
