@@ -47,10 +47,6 @@
 #include <vips/vips.h>
 #include <vips/internal.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Create the final mask by copying the 1/4 of the mask held by coeff
  *  The final mask is written onto image on a line by line basis
  *  The buffer coeff should hold (xsize/2+1)*(ysize/2+1) elms
@@ -77,7 +73,7 @@ copy_quarter( IMAGE *out, float *coeff_s )
                         *cpline++ = *cpcoeff++;
                 for( x = out->Xsize/2; x < out->Xsize; x++ )
                         *cpline++ = *cpcoeff--;
-                if( im_writeline( y, out, (PEL *) line ) )
+                if( im_writeline( y, out, (VipsPel *) line ) )
                         return( -1 );
 	}
 
@@ -89,7 +85,7 @@ copy_quarter( IMAGE *out, float *coeff_s )
                         *cpline++ = *cpcoeff++;
                 for( x = out->Xsize/2; x < out->Xsize; x++ )
                         *cpline++ = *cpcoeff--;
-                if( im_writeline( y, out, (PEL *) line ) )
+                if( im_writeline( y, out, (VipsPel *) line ) )
                         return( -1 );
 	}
 
