@@ -57,10 +57,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 #define ACCUMULATE( ITYPE, OTYPE ) { \
 	for( b = 0; b < nb; b++ ) { \
 		ITYPE *p = (ITYPE *) in->data; \
@@ -89,13 +85,13 @@
 int 
 im_histcum( IMAGE *in, IMAGE *out )
 {
-	const int px = in->Xsize * in->Ysize;
+	const guint64 px = VIPS_IMAGE_N_PELS( in );
 	const int nb = vips_bandfmt_iscomplex( in->BandFmt ) ? 
 		in->Bands * 2 : in->Bands;
-	const int mx = px * nb;
+	const guint64 mx = px * nb;
 
-	PEL *outbuf;		
-	int b, x;
+	VipsPel *outbuf;		
+	guint64 b, x;
 
 	if( im_check_uncoded( "im_histcum", in ) ||
 		im_check_hist( "im_histcum", in ) ||
@@ -163,10 +159,10 @@ im_histcum( IMAGE *in, IMAGE *out )
 int 
 im_histnorm( IMAGE *in, IMAGE *out )
 {
-	const int px = in->Xsize * in->Ysize;
+	const guint64 px = VIPS_IMAGE_N_PELS( in );
 	DOUBLEMASK *stats;
 	double *a, *b;
-	int i;
+	int y;
 	IMAGE *t1;
 	int fmt;
 
@@ -179,9 +175,9 @@ im_histnorm( IMAGE *in, IMAGE *out )
 
 	/* Scale each channel by px / channel max
 	 */
-	for( i = 0; i < in->Bands; i++ ) {
-		a[i] = px / stats->coeff[6 + 1 + 6*i];
-		b[i] = 0;
+	for( y = 0; y < in->Bands; y++ ) {
+		a[y] = px / VIPS_MASK( stats, 1, y + 1 );
+		b[y] = 0;
 	}
 
 	im_free_dmask( stats );
