@@ -19,7 +19,9 @@
  * 	- number bands from zero
  * 24/3/10
  * 	- gtkdoc
- * 	- small celanups
+ * 	- small cleanups
+ * 25/1/12
+ * 	- cast @in to u8/u16.
  */
 
 /*
@@ -59,10 +61,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Accumulate a histogram in one of these.
  */
 typedef struct {
@@ -138,7 +136,7 @@ merge_subhist( void *seq, void *a, void *b )
 /* Histogram of all bands of a uchar image.
  */
 static int
-find_uchar_hist( REGION *reg, void *seq, void *a, void *b )
+find_uchar_hist( REGION *reg, void *seq, void *a, void *b, gboolean *stop )
 {
 	Histogram *hist = (Histogram *) seq;
 	Rect *r = &reg->valid;
@@ -152,7 +150,7 @@ find_uchar_hist( REGION *reg, void *seq, void *a, void *b )
 	/* Accumulate!
 	 */
 	for( y = to; y < bo; y++ ) {
-		PEL *p = (PEL *) IM_REGION_ADDR( reg, le, y );
+		VipsPel *p = IM_REGION_ADDR( reg, le, y );
 		int i;
 
 		for( i = 0, x = 0; x < r->width; x++ )
@@ -170,7 +168,8 @@ find_uchar_hist( REGION *reg, void *seq, void *a, void *b )
 /* Histogram of a selected band of a uchar image.
  */
 static int
-find_uchar_hist_extract( REGION *reg, void *seq, void *a, void *b )
+find_uchar_hist_extract( REGION *reg, 
+	void *seq, void *a, void *b, gboolean *stop )
 {
 	Histogram *hist = (Histogram *) seq;
 	Rect *r = &reg->valid;
@@ -186,7 +185,7 @@ find_uchar_hist_extract( REGION *reg, void *seq, void *a, void *b )
 	/* Accumulate!
 	 */
 	for( y = to; y < bo; y++ ) {
-		PEL *p = (PEL *) IM_REGION_ADDR( reg, le, y );
+		VipsPel *p = IM_REGION_ADDR( reg, le, y );
 
 		for( x = hist->which; x < max; x += nb ) 
 			bins[p[x]]++;
@@ -202,7 +201,7 @@ find_uchar_hist_extract( REGION *reg, void *seq, void *a, void *b )
 /* Histogram of all bands of a ushort image.
  */
 static int
-find_ushort_hist( REGION *reg, void *seq, void *a, void *b )
+find_ushort_hist( REGION *reg, void *seq, void *a, void *b, gboolean *stop )
 {
 	Histogram *hist = (Histogram *) seq;
 	Rect *r = &reg->valid;
@@ -244,7 +243,8 @@ find_ushort_hist( REGION *reg, void *seq, void *a, void *b )
 /* Histogram of all bands of a ushort image.
  */
 static int
-find_ushort_hist_extract( REGION *reg, void *seq, void *a, void *b )
+find_ushort_hist_extract( REGION *reg, 
+	void *seq, void *a, void *b, gboolean *stop )
 {
 	Histogram *hist = (Histogram *) seq;
 	Rect *r = &reg->valid;
@@ -283,6 +283,19 @@ find_ushort_hist_extract( REGION *reg, void *seq, void *a, void *b )
 	return( 0 );
 }
 
+/* Save a bit of typing.
+ */
+#define UC IM_BANDFMT_UCHAR
+#define US IM_BANDFMT_USHORT
+#define UI IM_BANDFMT_UINT
+
+/* Type mapping: go to uchar or ushort.
+ */
+static int bandfmt_histgr[10] = {
+/* UC   C  US   S  UI   I   F   X  D   DX */
+   UC, UC, US, US, US, US, US, US, US, US
+};
+
 /**
  * im_histgr:
  * @in: input image
@@ -293,7 +306,7 @@ find_ushort_hist_extract( REGION *reg, void *seq, void *a, void *b )
  * one-band histogram), or for all bands (producing an n-band histogram) if 
  * @bandno is -1. 
  *
- * @in must be u8 or u16. @out is always u32.
+ * @in is cast to u8 or u16. @out is always u32.
  *
  * See also: im_hist_indexed(), im_histeq().
  *
@@ -302,22 +315,29 @@ find_ushort_hist_extract( REGION *reg, void *seq, void *a, void *b )
 int 
 im_histgr( IMAGE *in, IMAGE *out, int bandno )
 {
+	IMAGE *t;
 	int size;		/* Length of hist */
 	int bands;		/* Number of bands in output */
 	Histogram *mhist;
-	im_generate_fn scanfn;
+	VipsGenerateFn scanfn;
 	int i, j;
 	unsigned int *obuffer, *q;
 
 	/* Check images. PIO from in, WIO to out.
 	 */
 	if( im_check_uncoded( "im_histgr", in ) || 
-		im_check_u8or16( "im_histgr", in ) ||
 		im_check_bandno( "im_histgr", in, bandno ) ||
 		im_pincheck( in ) || 
 		im_outcheck( out ) )
 		return( -1 );
 
+	/* Cast in to u8/u16.
+	 */
+	if( !(t = im_open_local( out, "im_histgr", "p" )) ||
+		im_clip2fmt( in, t, bandfmt_histgr[in->BandFmt] ) )
+		return( -1 );
+	in = t;
+
 	/* Find the range of pixel values we must handle.
 	 */
 	size = in->BandFmt == IM_BANDFMT_UCHAR ? 256 : 65536;
@@ -372,7 +392,7 @@ im_histgr( IMAGE *in, IMAGE *out, int bandno )
 
 	/* Write interleaved buffer into hist.
 	 */
-	if( im_writeline( 0, out, (PEL *) obuffer ) )
+	if( im_writeline( 0, out, (VipsPel *) obuffer ) )
 		return( -1 );
 
 	return( 0 );
