@@ -43,10 +43,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Accumulate a histogram in one of these.
  */
 typedef struct {
@@ -155,7 +151,7 @@ hist_stop( void *seq, void *a, void *b )
 /* A uchar index image.
  */
 static int
-hist_scan_uchar( REGION *reg, void *seq, void *a, void *b )
+hist_scan_uchar( REGION *reg, void *seq, void *a, void *b, gboolean *stop )
 {
 	Histogram *hist = (Histogram *) seq;
 	Rect *r = &reg->valid;
@@ -173,8 +169,8 @@ hist_scan_uchar( REGION *reg, void *seq, void *a, void *b )
 	/* Accumulate!
 	 */
 	for( y = 0; y < r->height; y++ ) {
-		PEL *i = (PEL *) IM_REGION_ADDR( reg, r->left, r->top + y );
-		PEL *v = (PEL *) IM_REGION_ADDR( hist->vreg, 
+		VipsPel *i = IM_REGION_ADDR( reg, r->left, r->top + y );
+		VipsPel *v = IM_REGION_ADDR( hist->vreg, 
 			r->left, r->top + y );
 
 		switch( value->BandFmt ) {
@@ -230,7 +226,7 @@ hist_scan_uchar( REGION *reg, void *seq, void *a, void *b )
 /* A ushort index image.
  */
 static int
-hist_scan_ushort( REGION *reg, void *seq, void *a, void *b )
+hist_scan_ushort( REGION *reg, void *seq, void *a, void *b, gboolean *stop )
 {
 	Histogram *hist = (Histogram *) seq;
 	Rect *r = &reg->valid;
@@ -251,7 +247,7 @@ hist_scan_ushort( REGION *reg, void *seq, void *a, void *b )
 	for( y = 0; y < r->height; y++ ) {
 		unsigned short *i = (unsigned short *) IM_REGION_ADDR( reg, 
 				r->left, r->top + y );
-		PEL *v = (PEL *) IM_REGION_ADDR( hist->vreg, 
+		VipsPel *v = IM_REGION_ADDR( hist->vreg, 
 			r->left, r->top + y );
 
 		switch( value->BandFmt ) {
@@ -296,7 +292,7 @@ hist_write( IMAGE *out, Histogram *hist )
 	if( im_setupout( out ) )
 		return( -1 );
 
-	if( im_writeline( 0, out, (PEL *) hist->bins ) )
+	if( im_writeline( 0, out, (VipsPel *) hist->bins ) )
 		return( -1 );
 
 	return( 0 );
@@ -327,7 +323,7 @@ im_hist_indexed( IMAGE *index, IMAGE *value, IMAGE *out )
 {
 	int size;		/* Length of hist */
 	Histogram *mhist;
-	im_generate_fn scanfn;
+	VipsGenerateFn scanfn;
 
 	/* Check images. PIO from in, WIO to out.
 	 */
