@@ -65,10 +65,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Normalise an image using the rules noted above.
  */
 static int
@@ -109,8 +105,8 @@ normalise( IMAGE *in, IMAGE *out )
 
 		if( !(stats = im_stats( in )) )
 			return( -1 );
-		min = stats->coeff[0];
-		max = stats->coeff[1];
+		min = VIPS_MASK( stats, 0, 0 );
+		max = VIPS_MASK( stats, 1, 0 );
 		im_free_dmask( stats );
 
 		if( !(t1 = im_open_local( out, "im_histplot", "p" )) ||
@@ -149,8 +145,8 @@ make_vert_gen( REGION *or, void *seq, void *a, void *b )
 	int x, y, z;
 
 	for( y = to; y < bo; y++ ) {
-		PEL *q = (PEL *) IM_REGION_ADDR( or, le, y );
-		PEL *p = (PEL *) IM_IMAGE_ADDR( in, 0, y );
+		VipsPel *q = IM_REGION_ADDR( or, le, y );
+		VipsPel *p = IM_IMAGE_ADDR( in, 0, y );
 
 		switch( in->BandFmt ) {
 		case IM_BANDFMT_UCHAR: 	VERT( unsigned char ); break;
@@ -199,8 +195,8 @@ make_horz_gen( REGION *or, void *seq, void *a, void *b )
 	int x, y, z;
 
 	for( x = le; x < ri; x++ ) {
-		PEL *q = (PEL *) IM_REGION_ADDR( or, x, to );
-		PEL *p = (PEL *) IM_IMAGE_ADDR( in, x, 0 );
+		VipsPel *q = IM_REGION_ADDR( or, x, to );
+		VipsPel *p = IM_IMAGE_ADDR( in, x, 0 );
 
 		switch( in->BandFmt ) {
 		case IM_BANDFMT_UCHAR: 	HORZ( unsigned char ); break;
