@@ -51,10 +51,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /*
 #define DEBUG
  */
@@ -68,18 +64,18 @@
 static int
 match( IMAGE *in, IMAGE *ref, IMAGE *out )
 {
-	const int inpx = in->Xsize * in->Ysize;
-	const int refpx = ref->Xsize * ref->Ysize;
+	const guint64 inpx = VIPS_IMAGE_N_PELS( in );
+	const guint64 refpx = VIPS_IMAGE_N_PELS( ref );
 	const int bands = in->Bands;	
 
 	unsigned int *inbuf;		/* in and ref, padded to same size */
 	unsigned int *refbuf;
 	unsigned int *outbuf;		/* Always output as uint */
 
-	int px;				/* Number of pixels */
-	int max;			/* px * bands */
+	guint64 px;			/* Number of pixels */
+	guint64 max;			/* px * bands */
 
-	int i, j;
+	guint64 i, j;
 
 	if( im_iocheck( in, out ) || 
 		im_iocheck( ref, out ) ||
@@ -98,6 +94,8 @@ match( IMAGE *in, IMAGE *ref, IMAGE *out )
 		px = 256;
 	else if( inpx <= 65536 && refpx <= 65536 )
 		px = 65536;
+	else 
+		px = IM_MAX( inpx, refpx );
 	max = px * bands;
 
 	/* Unpack to equal-sized buffers.
@@ -153,7 +151,7 @@ match( IMAGE *in, IMAGE *ref, IMAGE *out )
         if( im_setupout( out ) )
 		return( -1 );
 
-        if( im_writeline( 0, out, (PEL *) outbuf ) )
+        if( im_writeline( 0, out, (VipsPel *) outbuf ) )
 		return( -1 );
 
 	return( 0 );
@@ -161,9 +159,9 @@ match( IMAGE *in, IMAGE *ref, IMAGE *out )
 
 /**
  * im_histspec:
- * @in: input image
+ * @in: input histogram
  * @ref: reference histogram 
- * @out: output image
+ * @out: output histogram
  *
  * Creates a lut which, when applied to the image from which histogram @in was
  * formed, will produce an image whose PDF matches that of the image from 
@@ -177,7 +175,7 @@ int
 im_histspec( IMAGE *in, IMAGE *ref, IMAGE *out )
 {
 	IMAGE *t[5];
-	int px;
+	guint64 px;
 	int fmt;
 
 	if( im_check_uint( "im_histspec", in ) ||
@@ -196,7 +194,7 @@ im_histspec( IMAGE *in, IMAGE *ref, IMAGE *out )
 
 	/* Clip type down.
 	 */
-	px = t[4]->Xsize * t[4]->Ysize;
+	px = VIPS_IMAGE_N_PELS( t[4] );
 	if( px <= 256 ) 
 		fmt = IM_BANDFMT_UCHAR;
 	else if( px <= 65536 ) 
