@@ -44,10 +44,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Accumulate a projection in one of these.
  */
 typedef struct {
@@ -189,7 +185,7 @@ project_merge( void *seq, void *a, void *b )
 /* Add a region to a project.
  */
 static int
-project_scan( REGION *reg, void *seq, void *a, void *b )
+project_scan( REGION *reg, void *seq, void *a, void *b, gboolean *stop )
 {
 	Project *project = (Project *) seq;
 	Rect *r = &reg->valid;
@@ -296,10 +292,10 @@ im_project( IMAGE *in, IMAGE *hout, IMAGE *vout )
 		im_setupout( vout ) )
 		return( -1 );
 
-	if( im_writeline( 0, vout, (PEL *) mproject->columns ) )
+	if( im_writeline( 0, vout, (VipsPel *) mproject->columns ) )
 		return( -1 );
 	for( y = 0; y < in->Ysize; y++ )
-		if( im_writeline( y, hout, (PEL *) mproject->rows + 
+		if( im_writeline( y, hout, (VipsPel *) mproject->rows + 
 			y * IM_IMAGE_SIZEOF_PEL( hout ) ) )
 			return( -1 );
 
