@@ -42,6 +42,33 @@
 extern "C" {
 #endif /*__cplusplus*/
 
+struct im_col_display;
+
+int vips_LabQ2disp( VipsImage *in, VipsImage **out, 
+	struct im_col_display *disp, ... )
+	__attribute__((sentinel));
+int vips_rad2float( VipsImage *in, VipsImage **out, ... )
+	__attribute__((sentinel));
+int vips_float2rad( VipsImage *in, VipsImage **out, ... )
+	__attribute__((sentinel));
+int vips_LabS2LabQ( VipsImage *in, VipsImage **out, ... )
+	__attribute__((sentinel));
+int vips_LabQ2Lab( VipsImage *in, VipsImage **out, ... )
+	__attribute__((sentinel));
+int vips_Lab2LabQ( VipsImage *in, VipsImage **out, ... )
+	__attribute__((sentinel));
+int vips_LCh2Lab( VipsImage *in, VipsImage **out, ... )
+	__attribute__((sentinel));
+int vips_Yxy2Lab( VipsImage *in, VipsImage **out, ... )
+	__attribute__((sentinel));
+int vips_UCS2XYZ( VipsImage *in, VipsImage **out, ... )
+	__attribute__((sentinel));
+int vips_Lab2XYZ( VipsImage *in, VipsImage **out, ... )
+	__attribute__((sentinel));
+int vips_XYZ2disp( VipsImage *in, VipsImage **out, 
+	struct im_col_display *disp, ... )
+	__attribute__((sentinel));
+
 /* Areas under curves for Dxx. 2 degree observer.
  */
 #define IM_D93_X0 (89.7400)
@@ -154,8 +181,6 @@ int im_lab_morph( VipsImage *in, VipsImage *out,
 	double L_offset, double L_scale,
 	double a_scale, double b_scale );
 
-void im_col_make_tables_UCS( void );
-
 /* Render intents for icc wrappers.
  */
 typedef enum {
