@@ -37,30 +37,34 @@
 extern "C" {
 #endif /*__cplusplus*/
 
-int im_conv( IMAGE *in, IMAGE *out, INTMASK *mask );
-int im_conv_f( IMAGE *in, IMAGE *out, DOUBLEMASK *mask );
-int im_convsep( IMAGE *in, IMAGE *out, INTMASK *mask );
-int im_convsep_f( IMAGE *in, IMAGE *out, DOUBLEMASK *mask );
+int im_aconvsep( VipsImage *in, VipsImage *out, 
+	DOUBLEMASK *mask, int n_layers );
+int im_aconv( VipsImage *in, VipsImage *out, 
+	DOUBLEMASK *mask, int n_layers, int cluster );
+int im_conv( VipsImage *in, VipsImage *out, INTMASK *mask );
+int im_conv_f( VipsImage *in, VipsImage *out, DOUBLEMASK *mask );
+int im_convsep( VipsImage *in, VipsImage *out, INTMASK *mask );
+int im_convsep_f( VipsImage *in, VipsImage *out, DOUBLEMASK *mask );
 
-int im_compass( IMAGE *in, IMAGE *out, INTMASK *mask );
-int im_gradient( IMAGE *in, IMAGE *out, INTMASK *mask );
-int im_lindetect( IMAGE *in, IMAGE *out, INTMASK *mask );
+int im_compass( VipsImage *in, VipsImage *out, INTMASK *mask );
+int im_gradient( VipsImage *in, VipsImage *out, INTMASK *mask );
+int im_lindetect( VipsImage *in, VipsImage *out, INTMASK *mask );
 
-int im_sharpen( IMAGE *in, IMAGE *out, 
+int im_sharpen( VipsImage *in, VipsImage *out, 
 	int mask_size, 
 	double x1, double y2, double y3, 
 	double m1, double m2 );
 
-int im_grad_x( IMAGE *in, IMAGE *out );
-int im_grad_y( IMAGE *in, IMAGE *out );
+int im_grad_x( VipsImage *in, VipsImage *out );
+int im_grad_y( VipsImage *in, VipsImage *out );
 
-int im_fastcor( IMAGE *in, IMAGE *ref, IMAGE *out );
-int im_spcor( IMAGE *in, IMAGE *ref, IMAGE *out );
-int im_gradcor( IMAGE *in, IMAGE *ref, IMAGE *out );
-int im_contrast_surface( IMAGE *in, IMAGE *out, 
+int im_fastcor( VipsImage *in, VipsImage *ref, VipsImage *out );
+int im_spcor( VipsImage *in, VipsImage *ref, VipsImage *out );
+int im_gradcor( VipsImage *in, VipsImage *ref, VipsImage *out );
+int im_contrast_surface( VipsImage *in, VipsImage *out, 
 	int half_win_size, int spacing );
 
-int im_addgnoise( IMAGE *in, IMAGE *out, double sigma );
+int im_addgnoise( VipsImage *in, VipsImage *out, double sigma );
 
 #ifdef __cplusplus
 }
