@@ -27,8 +27,8 @@
 
  */
 
-#ifndef IM_DEBUG_H
-#define IM_DEBUG_H
+#ifndef VIPS_DEBUG_H
+#define VIPS_DEBUG_H
 
 #ifdef __cplusplus
 extern "C" {
@@ -66,35 +66,8 @@ extern "C" {
 	G_STMT_START { ; } G_STMT_END
 #endif /*VIPS_DEBUG_GREEN*/
 
-/* All open image descriptors ... see im_init() and im_close().
- */
-extern GSList *im__open_images;
-
-/* Print one line for each descriptor, complete dump for one descriptor.
- */
-void im__print_one( int n );
-void im__print_all( void );
-
-const char *im_Type2char( VipsType type );
-const char *im_BandFmt2char( VipsBandFmt fmt );
-const char *im_Coding2char( VipsCoding coding );
-const char *im_Compression2char( int n );
-const char *im_dtype2char( im_desc_type n );
-const char *im_dhint2char( VipsDemandStyle style );
-
-int im_char2Type( const char *str );
-int im_char2BandFmt( const char *str );
-int im_char2Coding( const char *str );
-int im_char2Compression( const char *str );
-im_desc_type im_char2dtype( const char *str );
-im_demand_type im_char2dhint( const char *str );
-
-void im_printdesc( IMAGE *image );
-int im_image_sanity( IMAGE *im );
-void im_image_sanity_all( void );
-
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
 
-#endif /* IM_DEBUG_H */
+#endif /* VIPS_DEBUG_H */
