@@ -1,4 +1,4 @@
-/* Definitions for partial image regions.
+/* Generate pixels.
  *
  * J.Cupitt, 8/4/93
  */
@@ -36,9 +36,29 @@
 extern "C" {
 #endif /*__cplusplus*/
 
+typedef int (*VipsRegionWrite)( VipsRegion *region, VipsRect *area, void *a );
+int vips_sink_disc( VipsImage *im, VipsRegionWrite write_fn, void *a );
+
 typedef void *(*VipsStartFn)( VipsImage *out, void *a, void *b );
-typedef int (*VipsGenerateFn)( VipsRegion *out, void *seq, void *a, void *b );
+typedef int (*VipsGenerateFn)( VipsRegion *out, 
+	void *seq, void *a, void *b, gboolean *stop );
 typedef int (*VipsStopFn)( void *seq, void *a, void *b );
+int vips_sink( VipsImage *im, 
+	VipsStartFn start_fn, VipsGenerateFn generate_fn, VipsStopFn stop_fn,
+	void *a, void *b );
+int vips_sink_tile( VipsImage *im, 
+	int tile_width, int tile_height,
+	VipsStartFn start_fn, VipsGenerateFn generate_fn, VipsStopFn stop_fn,
+	void *a, void *b );
+
+typedef void (*VipsSinkNotify)( VipsImage *im, VipsRect *rect, void *a );
+int vips_sink_screen( VipsImage *in, VipsImage *out, VipsImage *mask,
+	int tile_width, int tile_height, int max_tiles,
+	int priority,
+	VipsSinkNotify notify, void *a );
+
+int vips_sink_memory( VipsImage *im );
+int vips_sink_memory2( VipsImage *image );
 
 void *vips_start_one( VipsImage *out, void *a, void *b );
 int vips_stop_one( void *seq, void *a, void *b );
@@ -48,13 +68,13 @@ VipsImage **vips_allocate_input_array( VipsImage *out, ... )
 	__attribute__((sentinel));
 
 int vips_image_generate( VipsImage *im,
-	VipsStartFn start, VipsGenerateFn generate, VipsStopFn stop,
+	VipsStartFn start_fn, VipsGenerateFn generate_fn, VipsStopFn stop_fn,
 	void *a, void *b
 );
 
-int vips_demand_hint_array( VipsImage *image, 
+void vips_demand_hint_array( VipsImage *image, 
 	VipsDemandStyle hint, VipsImage **in );
-int vips_demand_hint( VipsImage *image, VipsDemandStyle hint, ... )
+void vips_demand_hint( VipsImage *image, VipsDemandStyle hint, ... )
 	__attribute__((sentinel));
 
 #ifdef __cplusplus
