@@ -77,108 +77,57 @@ extern "C" {
  */
 #define VIPS_META_RESOLUTION_UNIT "resolution-unit"
 
-int vips_format_sizeof( VipsBandFormat format );
+guint64 vips_format_sizeof( VipsBandFormat format );
 
-int vips_image_get_width( VipsImage *image );
-int vips_image_get_height( VipsImage *image );
-int vips_image_get_bands( VipsImage *image );
-VipsBandFormat vips_image_get_format( VipsImage *image );
-VipsCoding vips_image_get_coding( VipsImage *image );
-VipsInterpretation vips_image_get_interpretation( VipsImage *image );
-double vips_image_get_xres( VipsImage *image );
-double vips_image_get_yres( VipsImage *image );
-int vips_image_get_xoffset( VipsImage *image );
-int vips_image_get_yoffset( VipsImage *image );
-const char *vips_image_get_filename( VipsImage *image );
-const char *vips_image_get_mode( VipsImage *image );
-void *vips_image_get_data( VipsImage *image );;
+int vips_image_get_width( const VipsImage *image );
+int vips_image_get_height( const VipsImage *image );
+int vips_image_get_bands( const VipsImage *image );
+VipsBandFormat vips_image_get_format( const VipsImage *image );
+VipsCoding vips_image_get_coding( const VipsImage *image );
+VipsInterpretation vips_image_get_interpretation( const VipsImage *image );
+double vips_image_get_xres( const VipsImage *image );
+double vips_image_get_yres( const VipsImage *image );
+int vips_image_get_xoffset( const VipsImage *image );
+int vips_image_get_yoffset( const VipsImage *image );
+const char *vips_image_get_filename( const VipsImage *image );
+const char *vips_image_get_mode( const VipsImage *image );
+void *vips_image_get_data( VipsImage *image );
 
 void vips_image_init_fields( VipsImage *image, 
 	int xsize, int ysize, int bands, 
 	VipsBandFormat format, VipsCoding coding, 
 	VipsInterpretation interpretation, 
-	float xres, float yres );
+	double xres, double yres );
 
 int vips_image_copy_fields_array( VipsImage *out, VipsImage *in[] );
 int vips_image_copy_fieldsv( VipsImage *out, VipsImage *in1, ... )
 	__attribute__((sentinel));
 int vips_image_copy_fields( VipsImage *out, VipsImage *in );
 
-int vips_image_set( VipsImage *image, const char *field, GValue *value );
+void vips_image_set( VipsImage *image, const char *field, GValue *value );
 int vips_image_get( VipsImage *image, const char *field, GValue *value_copy );
+int vips_image_get_as_string( VipsImage *image, const char *field, char **out );
 GType vips_image_get_typeof( VipsImage *image, const char *field );
 gboolean vips_image_remove( VipsImage *image, const char *field );
 typedef void *(*VipsImageMapFn)( VipsImage *image, 
 	const char *field, GValue *value, void *a );
-void *vips_image_map( VipsImage *im, VipsImageMapFn fn, void *a );
+void *vips_image_map( VipsImage *image, VipsImageMapFn fn, void *a );
 
-/**
- * VIPS_TYPE_SAVE_STRING:
- *
- * The #GType for an "vips_save_string".
- */
-#define VIPS_TYPE_SAVE_STRING (vips_save_string_get_type())
-GType vips_save_string_get_type( void );
-const char *vips_save_string_get( const GValue *value );
-void vips_save_string_set( GValue *value, const char *str );
-void vips_save_string_setf( GValue *value, const char *fmt, ... )
-	__attribute__((format(printf, 2, 3)));
-
-/**
- * VIPS_TYPE_AREA:
- *
- * The #GType for an #vips_area.
- */
-#define VIPS_TYPE_AREA (vips_area_get_type())
-GType vips_area_get_type( void );
-
-/**
- * VIPS_TYPE_REF_STRING:
- *
- * The #GType for an #vips_refstring.
- */
-#define VIPS_TYPE_REF_STRING (vips_ref_string_get_type())
-GType vips_ref_string_get_type( void );
-int vips_ref_string_set( GValue *value, const char *str );
-const char *vips_ref_string_get( const GValue *value );
-size_t vips_ref_string_get_length( const GValue *value );
-
-/**
- * VIPS_TYPE_BLOB:
- *
- * The #GType for an #vips_blob.
- */
-
-/* Also used for eg. vips_local() and friends.
- */
-typedef int (*VipsCallbackFn)( void *a, void *b );
-
-#define VIPS_TYPE_BLOB (vips_blob_get_type())
-GType vips_blob_get_type( void );
-void *vips_blob_get( const GValue *value, size_t *length );
-int vips_blob_set( GValue *value, VipsCallbackFn free_fn, 
-	void *data, size_t length ); 
-
-int vips_image_set_area( VipsImage *image, 
+void vips_image_set_area( VipsImage *image, 
 	const char *field, VipsCallbackFn free_fn, void *data );
 int vips_image_get_area( VipsImage *image, const char *field, void **data );
-int vips_image_set_string( VipsImage *image, 
-	const char *field, const char *str );
-int vips_image_get_string( VipsImage *image, const char *field, char **str );
-int vips_image_set_blob( VipsImage *image, const char *field, 
+void vips_image_set_blob( VipsImage *image, const char *field, 
 	VipsCallbackFn free_fn, void *data, size_t length );
 int vips_image_get_blob( VipsImage *image, const char *field, 
 	void **data, size_t *length );
 
 int vips_image_get_int( VipsImage *image, const char *field, int *out );
-int vips_image_set_int( VipsImage *image, const char *field, int i );
+void vips_image_set_int( VipsImage *image, const char *field, int i );
 int vips_image_get_double( VipsImage *image, const char *field, double *out );
-int vips_image_set_double( VipsImage *image, const char *field, double d );
+void vips_image_set_double( VipsImage *image, const char *field, double d );
 int vips_image_get_string( VipsImage *image, const char *field, char **out );
-int vips_image_set_string( VipsImage *image, 
+void vips_image_set_string( VipsImage *image, 
 	const char *field, const char *str );
-int vips_image_get_as_string( VipsImage *image, const char *field, char **out );
-GType vips_image_get_typeof( VipsImage *image, const char *field );
 
 int vips_image_history_printf( VipsImage *image, const char *format, ... )
 	__attribute__((format(printf, 2, 3)));
