@@ -62,29 +62,30 @@ extern "C" {
  * will use the most restrictive of the styles requested by the operations 
  * in the pipeline.
  *
- * VIPS_DEMAND_STYLE_THINSTRIP --- This operation would like to output strips 
+ * #VIPS_DEMAND_STYLE_THINSTRIP --- This operation would like to output strips 
  * the width of the image and a few pels high. This is option suitable for 
  * point-to-point operations, such as those in the arithmetic package.
  *
  * This option is only efficient for cases where each output pel depends 
  * upon the pel in the corresponding position in the input image.
  *
- * VIPS_DEMAND_STYLE_FATSTRIP --- This operation would like to output strips 
+ * #VIPS_DEMAND_STYLE_FATSTRIP --- This operation would like to output strips 
  * the width of the image and as high as possible. This option is suitable 
  * for area operations which do not violently transform coordinates, such 
  * as im_conv(). 
  *
- * VIPS_DEMAND_STYLE_SMALLTILE --- This is the most general demand format.
+ * #VIPS_DEMAND_STYLE_SMALLTILE --- This is the most general demand format.
  * Output is demanded in small (around 100x100 pel) sections. This style works 
  * reasonably efficiently, even for bizzare operations like 45 degree rotate.
  *
- * VIPS_DEMAND_STYLE_ANY --- This image is not being demand-read from a disc 
+ * #VIPS_DEMAND_STYLE_ANY --- This image is not being demand-read from a disc 
  * file (even indirectly) so any demand style is OK. It's used for things like
  * im_black() where the pixels are calculated.
  *
  * See also: vips_demand_hint().
  */
 typedef enum {
+	VIPS_DEMAND_STYLE_ERROR = -1,	
 	VIPS_DEMAND_STYLE_SMALLTILE,	
 	VIPS_DEMAND_STYLE_FATSTRIP,
 	VIPS_DEMAND_STYLE_THINSTRIP,
@@ -95,6 +96,7 @@ typedef enum {
  * does not imply that any fields in IMAGE have valid data.
  */
 typedef enum {
+	VIPS_IMAGE_ERROR = -1,	
 	VIPS_IMAGE_NONE,		/* no type set */
 	VIPS_IMAGE_SETBUF,		/* malloced memory array */
 	VIPS_IMAGE_SETBUF_FOREIGN,	/* memory array, don't free on close */
@@ -107,25 +109,26 @@ typedef enum {
 
 /**
  * VipsInterpretation: 
- * @VIPS_TYPE_MULTIBAND: generic many-band image
- * @VIPS_TYPE_B_W: some kind of single-band image
- * @VIPS_TYPE_HISTOGRAM: a 1D image such as a histogram or lookup table
- * @VIPS_TYPE_FOURIER: image is in fourier space
- * @VIPS_TYPE_XYZ: the first three bands are colours in CIE XYZ colourspace
- * @VIPS_TYPE_LAB: pixels are in CIE Lab space
- * @VIPS_TYPE_CMYK: the first four bands are in CMYK space
- * @VIPS_TYPE_LABQ: implies #VIPS_CODING_LABQ
- * @VIPS_TYPE_RGB: generic RGB space
- * @VIPS_TYPE_UCS: a uniform colourspace based on CMC
- * @VIPS_TYPE_LCH: pixels are in CIE LCh space
- * @VIPS_TYPE_LABS: pixels are CIE LAB coded as three signed 16-bit values
- * @VIPS_TYPE_sRGB: pixels are sRGB
- * @VIPS_TYPE_YXY: pixels are CIE Yxy
- * @VIPS_TYPE_RGB16: generic 16-bit RGB
- * @VIPS_TYPE_GREY16: generic 16-bit mono
+ * @VIPS_INTERPREATION_MULTIBAND: generic many-band image
+ * @VIPS_INTERPREATION_B_W: some kind of single-band image
+ * @VIPS_INTERPREATION_HISTOGRAM: a 1D image such as a histogram or lookup table
+ * @VIPS_INTERPREATION_FOURIER: image is in fourier space
+ * @VIPS_INTERPREATION_XYZ: the first three bands are CIE XYZ 
+ * @VIPS_INTERPREATION_LAB: pixels are in CIE Lab space
+ * @VIPS_INTERPREATION_CMYK: the first four bands are in CMYK space
+ * @VIPS_INTERPREATION_LABQ: implies #VIPS_CODING_LABQ
+ * @VIPS_INTERPREATION_RGB: generic RGB space
+ * @VIPS_INTERPREATION_UCS: a uniform colourspace based on CMC
+ * @VIPS_INTERPREATION_LCH: pixels are in CIE LCh space
+ * @VIPS_INTERPREATION_LABS: CIE LAB coded as three signed 16-bit values
+ * @VIPS_INTERPREATION_sRGB: pixels are sRGB
+ * @VIPS_INTERPREATION_YXY: pixels are CIE Yxy
+ * @VIPS_INTERPREATION_RGB16: generic 16-bit RGB
+ * @VIPS_INTERPREATION_GREY16: generic 16-bit mono
+ * @VIPS_INTERPREATION_ARRAY: an array
  *
  * How the values in an image should be interpreted. For example, a
- * three-band float image of type #VIPS_TYPE_LAB should have its pixels
+ * three-band float image of type #VIPS_INTERPREATION_LAB should have its pixels
  * interpreted as coordinates in CIE Lab space.
  *
  * These values are set by operations as hints to user-interfaces built on top 
@@ -136,6 +139,7 @@ typedef enum {
  * new numbers from the end.
  */
 typedef enum {
+	VIPS_INTERPRETATION_ERROR = -1,
 	VIPS_INTERPRETATION_MULTIBAND = 0,
 	VIPS_INTERPRETATION_B_W = 1,
 	VIPS_INTERPRETATION_HISTOGRAM = 10,
@@ -151,7 +155,8 @@ typedef enum {
 	VIPS_INTERPRETATION_sRGB = 22,
 	VIPS_INTERPRETATION_YXY = 23,
 	VIPS_INTERPRETATION_RGB16 = 25,
-	VIPS_INTERPRETATION_GREY16 = 26
+	VIPS_INTERPRETATION_GREY16 = 26,
+	VIPS_INTERPRETATION_ARRAY = 27
 } VipsInterpretation;
 
 /**
@@ -204,15 +209,17 @@ typedef enum {
  * new numbers from the end.
  */
 typedef enum {
+	VIPS_CODING_ERROR = -1,
 	VIPS_CODING_NONE = 0,
 	VIPS_CODING_LABQ = 2,
-	VIPS_CODING_RAD = 6
+	VIPS_CODING_RAD = 6,
+	VIPS_CODING_LAST = 7
 } VipsCoding;
 
 /* Struct we keep a record of execution time in. Passed to eval signal so
  * it can assess progress.
  */
-typedef struct {
+typedef struct _VipsProgress {
 	/*< private >*/
 	struct _VipsImage *im;	/* Image we are part of */
 
@@ -256,8 +263,8 @@ typedef struct _VipsImage {
 	VipsBandFormat BandFmt;	/* pixel format */
 	VipsCoding Coding;	/* pixel coding */
 	VipsInterpretation Type;/* pixel interpretation */
-	float Xres;		/* horizontal pixels per millimetre */
-	float Yres;		/* vertical pixels per millimetre */
+	double Xres;		/* horizontal pixels per millimetre */
+	double Yres;		/* vertical pixels per millimetre */
 
 	int Xoffset;		/* image origin hint */
 	int Yoffset;		/* image origin hint */
@@ -280,26 +287,33 @@ typedef struct _VipsImage {
 	 */
 	char *Hist;		/* don't use, see vips_image_get_history() */
 	char *filename;		/* pointer to copy of filename */
-	char *data;		/* start of image data for WIO */
+	VipsPel *data;		/* start of image data for WIO */
 	int kill;		/* set to non-zero to block eval */
 
 	/* Everything below this private and only used internally by
 	 * VipsImage.
 	 */
 
+	/* During vips image read and write we need temporary float-sized
+	 * fields in the struct for staging xres/yres. Don't use these any
+	 * other time.
+	 */
+	float Xres_float;
+	float Yres_float;
+
 	char *mode;		/* mode string passed to _new() */
 	VipsImageType dtype;	/* descriptor type */
 	int fd;         	/* file descriptor */
-	char *baseaddr;     	/* pointer to the start of an mmap file */
+	void *baseaddr;     	/* pointer to the start of an mmap file */
 	size_t length;		/* size of mmap area */
 	guint32 magic;		/* magic from header, endian-ness of image */
 
 	/* Partial image stuff. All these fields are initialised 
 	 * to NULL and ignored unless set by vips_image_generate() etc.
 	 */
-	void *(*start)();	/* user-supplied start function */
-	int (*generate)();	/* user-supplied generate function */
-	int (*stop)();		/* user-supplied stop function */
+	void *(*start_fn)();	/* user-supplied start function */
+	int (*generate_fn)();	/* user-supplied generate function */
+	int (*stop_fn)();	/* user-supplied stop function */
 	void *client1;		/* user arguments */
 	void *client2;
 	GMutex *sslock;		/* start-stop lock */
@@ -314,8 +328,11 @@ typedef struct _VipsImage {
 	/* Part of mmap() read ... the sizeof() the header we skip from the
 	 * file start. Usually VIPS_SIZEOF_HEADER, but can be something else
 	 * for binary file read.
+	 *
+	 * gint64 so that we can guarantee to work even on systems with
+	 * strange ideas about large files.
 	 */
-	int sizeof_header;
+	guint64 sizeof_header;
 
 	/* If this is a large disc image, don't map the whole thing, instead
 	 * have a set of windows shared between the regions active on the
@@ -370,7 +387,6 @@ typedef struct _VipsImage {
 	 */
 	gboolean delete_on_close;
 	char *delete_on_close_filename;
-
 } VipsImage;
 
 typedef struct _VipsImageClass {
@@ -407,6 +423,9 @@ typedef struct _VipsImageClass {
 
 GType vips_image_get_type( void );
 
+/* Has to be guint64 and not size_t/off_t since we have to be able to address
+ * huge images on platforms with 32-bit files.
+ */
 extern const guint64 vips__image_sizeof_bandformat[];
 
 /* Pixel address calculation macros.
@@ -421,6 +440,8 @@ extern const guint64 vips__image_sizeof_bandformat[];
 	(VIPS_IMAGE_SIZEOF_LINE( I ) * (I)->Ysize)
 #define VIPS_IMAGE_N_ELEMENTS( I ) \
 	((I)->Bands * (I)->Xsize)
+#define VIPS_IMAGE_N_PELS( I ) \
+	((guint64) (I)->Xsize * (I)->Ysize)
 
 /* If VIPS_DEBUG is defined, add bounds checking.
  */
@@ -441,7 +462,7 @@ extern const guint64 vips__image_sizeof_bandformat[];
 		(X), (Y), \
 		0, 0, \
 		(I)->Xsize, \
-		(I)->Ysize ), abort(), (char *) NULL) \
+		(I)->Ysize ), abort(), (VipsPel *) NULL) \
 	)
 #else /*!VIPS_DEBUG*/
 #define VIPS_IMAGE_ADDR( I, X, Y ) \
@@ -455,7 +476,7 @@ int vips_image_written( VipsImage *image );
 void vips_image_invalidate_all( VipsImage *image );
 
 void vips_image_preeval( VipsImage *image );
-void vips_image_eval( VipsImage *image, int w, int h );
+void vips_image_eval( VipsImage *image, guint64 processed );
 void vips_image_posteval( VipsImage *image );
 void vips_image_set_progress( VipsImage *image, gboolean progress );
 
@@ -463,29 +484,30 @@ gboolean vips_image_get_kill( VipsImage *image );
 void vips_image_set_kill( VipsImage *image, gboolean kill );
 
 VipsImage *vips_image_new( void );
-int vips_image_new_array( VipsObject *parent, VipsImage **images, int n );
 VipsImage *vips_image_new_mode( const char *filename, const char *mode );
+VipsImage *vips_image_new_buffer( void );
 VipsImage *vips_image_new_from_file( const char *filename );
 VipsImage *vips_image_new_from_file_raw( const char *filename, 
-	int xsize, int ysize, int bands, int offset );
+	int xsize, int ysize, int bands, guint64 offset );
 VipsImage *vips_image_new_from_memory( void *buffer, 
 	int xsize, int ysize, int bands, VipsBandFormat bandfmt );
+VipsImage *vips_image_new_array( int xsize, int ysize );
 void vips_image_set_delete_on_close( VipsImage *image, 
 	gboolean delete_on_close );
 VipsImage *vips_image_new_disc_temp( const char *format );
-int vips_image_write( VipsImage *image, const char *filename );
+int vips_image_write( VipsImage *image, VipsImage *out );
+int vips_image_write_to_file( VipsImage *image, const char *filename );
 
 gboolean vips_image_isMSBfirst( VipsImage *image );
 gboolean vips_image_isfile( VipsImage *image );
 gboolean vips_image_ispartial( VipsImage *image );
 
-int vips_image_write_line( VipsImage *image, int ypos, PEL *linebuffer );
-
 int vips_image_wio_input( VipsImage *image );
-int vips_image_wio_output( VipsImage *image );
-int vips_image_inplace( VipsImage *image );
 int vips_image_pio_input( VipsImage *image );
-int vips_image_pio_output( VipsImage *image );
+int vips_image_inplace( VipsImage *image );
+int vips_image_write_prepare( VipsImage *image );
+
+int vips_image_write_line( VipsImage *image, int ypos, VipsPel *linebuffer );
 
 gboolean vips_band_format_isint( VipsBandFormat format );
 gboolean vips_band_format_isuint( VipsBandFormat format );
