@@ -75,11 +75,20 @@ extern int vips__progress;
  */
 extern char *vips__disc_threshold;
 
+/* Cache size settings.
+ */
+extern char *vips__cache_max;
+extern char *vips__cache_max_mem;
+extern char *vips__cache_max_files;
+extern gboolean vips__cache_dump;
+extern gboolean vips__cache_trace;
+
 typedef int (*im__fftproc_fn)( VipsImage *, VipsImage *, VipsImage * );
 
 /* iofuncs
  */
 int vips__open_image_read( const char *filename );
+int vips__open_image_write( const char *filename, gboolean temp );
 int vips_image_open_input( VipsImage *image );
 int vips_image_open_output( VipsImage *image );
 
@@ -97,10 +106,10 @@ int vips_remapfilerw( VipsImage * );
 
 void vips__buffer_init( void );
 
-void vips__read_4byte( int msb_first, unsigned char *to, unsigned char **from );
-void vips__read_2byte( int msb_first, unsigned char *to, unsigned char **from );
-void vips__write_4byte( unsigned char **to, unsigned char *from );
-void vips__write_2byte( unsigned char **to, unsigned char *from );
+void vips__copy_4byte( int swap, unsigned char *to, unsigned char *from );
+void vips__copy_2byte( gboolean swap, unsigned char *to, unsigned char *from );
+
+guint32 vips__file_magic( const char *filename );
 int vips__has_extension_block( VipsImage *im );
 void *vips__read_extension_block( VipsImage *im, int *size );
 int vips__write_extension_block( VipsImage *im, void *buf, int size );
@@ -110,6 +119,24 @@ int vips__write_header_bytes( VipsImage *im, unsigned char *to );
 
 extern GMutex *vips__global_lock;
 
+int vips__formatalike_vec( VipsImage **in, VipsImage **out, int n );
+int vips__sizealike_vec( VipsImage **in, VipsImage **out, int n );
+int vips__bandup( const char *domain, VipsImage *in, VipsImage **out, int n );
+int vips__bandalike_vec( const char *domain, 
+	VipsImage **in, VipsImage **out, int n, int base_bands );
+
+int vips__formatalike( VipsImage *in1, VipsImage *in2, 
+	VipsImage **out1, VipsImage **out2 );
+int vips__sizealike( VipsImage *in1, VipsImage *in2, 
+	VipsImage **out1, VipsImage **out2 );
+int vips__bandalike( const char *domain, 
+	VipsImage *in1, VipsImage *in2, VipsImage **out1, VipsImage **out2 );
+
+int vips_foreign_tilecache( VipsImage *in, VipsImage **out, int strip_height );
+
+
+
+
 void im__format_init( void );
 
 void im__tiff_register( void );
@@ -121,16 +148,6 @@ void im__analyze_register( void );
 void im__exr_register( void );
 void im__magick_register( void );
 
-typedef enum {
-	IM__RGB,	/* 1 or 3 bands (eg. PPM) */
-	IM__RGBA,	/* 1, 2, 3 or 4 bands (eg. PNG) */
-	IM__RGB_CMYK,	/* 1, 3 or 4 bands (eg. JPEG) */
-	IM__ANY		/* any number of bands (eg. TIFF) */
-} im__saveable_t;
-
-VipsImage *im__convert_saveable( VipsImage *in, 
-	im__saveable_t saveable, int format_table[10] );
-
 int im__bandup( const char *domain, VipsImage *in, VipsImage *out, int n );
 int im__bandalike_vec( const char *domain, VipsImage **in, VipsImage **out, int n );
 int im__bandalike( const char *domain, 
@@ -185,8 +202,8 @@ struct im_col_tab_disp *im_col_make_tables_RGB( VipsImage *im,
 	struct im_col_display *d );
 struct im_col_tab_disp *im_col_display_get_table( struct im_col_display *d );
 
-int im__fft_sp( float *rvec, float *ivec, int logrows, int logcols );
-int im__fftproc( VipsImage *dummy, VipsImage *in, VipsImage *out, im__fftproc_fn fn );
+int im__fftproc( VipsImage *dummy, 
+	VipsImage *in, VipsImage *out, im__fftproc_fn fn );
 
 int im__find_lroverlap( VipsImage *ref_in, VipsImage *sec_in, VipsImage *out,
         int bandno_in,
@@ -218,8 +235,8 @@ typedef struct {
 
 void imb_XYZ2Lab( float *, float *, int, im_colour_temperature * );
 void imb_Lab2XYZ( float *, float *, int, im_colour_temperature * );
-void imb_LabQ2Lab( PEL *, float *, int );
-void imb_Lab2LabQ( float *, PEL *, int );
+void imb_LabQ2Lab( VipsPel *, float *, int );
+void imb_Lab2LabQ( float *, VipsPel *, int );
 void imb_LabS2Lab( signed short *, float *, int );
 void imb_Lab2LabS( float *, signed short *, int n );
 
@@ -253,7 +270,7 @@ int im__fmaskcir( VipsImage *out, VipsMaskType flag, va_list ap );
 /* inplace
  */
 
-PEL *im__vector_to_ink( const char *domain, VipsImage *im, int n, double *vec );
+VipsPel *im__vector_to_ink( const char *domain, VipsImage *im, int n, double *vec );
 VipsImage *im__inplace_base( const char *domain, 
 	VipsImage *main, VipsImage *sub, VipsImage *out );
 
@@ -265,6 +282,16 @@ void vips__interpolate_init( void );
  */
 void vips__init_wrap7_classes( void );
 
+/* Start up various packages.
+ */
+void vips_arithmetic_operation_init( void );
+void vips_conversion_operation_init( void );
+
+guint64 vips__parse_size( const char *size_string );
+
+IMAGE *vips__deprecated_open_read( const char *filename );
+IMAGE *vips__deprecated_open_write( const char *filename );
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
