@@ -50,6 +50,7 @@ typedef struct _VipsObjectClass VipsObjectClass;
  * @VIPS_ARGUMENT_REQUIRED: must be set in the constructor
  * @VIPS_ARGUMENT_CONSTRUCT: can only be set in the constructor
  * @VIPS_ARGUMENT_SET_ONCE: can only be set once
+ * @VIPS_ARGUMENT_SET_ALWAYS: don't do use-before-set checks
  * @VIPS_ARGUMENT_INPUT: is an input argument (one we depend on)
  * @VIPS_ARGUMENT_OUTPUT: is an output argument (depends on us)
  *
@@ -59,14 +60,20 @@ typedef struct _VipsObjectClass VipsObjectClass;
  *
  * Input gobjects are automatically reffed, output gobjects automatically ref
  * us. We also automatically watch for "destroy" and unlink.
+ *
+ * @VIPS_ARGUMENT_SET_ALWAYS is handy for arguments which are set from C. For
+ * example, VipsImage::width is a property that gives access to the Xsize
+ * member of struct _VipsImage. We default its 'assigned' to TRUE
+ * since the field is always set directly by C.
  */
-typedef enum {
+typedef enum /*< flags >*/ {
 	VIPS_ARGUMENT_NONE = 0,
 	VIPS_ARGUMENT_REQUIRED = 1,
 	VIPS_ARGUMENT_CONSTRUCT = 2,
 	VIPS_ARGUMENT_SET_ONCE = 4,
-	VIPS_ARGUMENT_INPUT = 8,
-	VIPS_ARGUMENT_OUTPUT = 16
+	VIPS_ARGUMENT_SET_ALWAYS = 8,
+	VIPS_ARGUMENT_INPUT = 16,
+	VIPS_ARGUMENT_OUTPUT = 32
 } VipsArgumentFlags;
 
 /* Useful flag combinations. User-visible ones are:
@@ -75,7 +82,9 @@ VIPS_ARGUMENT_REQUIRED_INPUT 	Eg. the "left" argument for an add operation
 
 VIPS_ARGUMENT_OPTIONAL_INPUT 	Eg. the "caption" for an object
 
-VIPS_ARGUMENT_OUTPUT 	        Eg. the "result" of an add operation
+VIPS_ARGUMENT_REQUIRED_OUTPUT  	Eg. the "result" of an add operation
+
+VIPS_ARGUMENT_OPTIONAL_OUTPUT   Eg. the x pos of the image minimum
 
    Other combinations are used internally, eg. supplying the cast-table for an 
    arithmetic operation
@@ -85,19 +94,149 @@ VIPS_ARGUMENT_OUTPUT 	        Eg. the "result" of an add operation
 #define VIPS_ARGUMENT_REQUIRED_INPUT \
 	(VIPS_ARGUMENT_INPUT | \
 	 VIPS_ARGUMENT_REQUIRED | \
-	 VIPS_ARGUMENT_CONSTRUCT | \
-	 VIPS_ARGUMENT_SET_ONCE)
+	 VIPS_ARGUMENT_CONSTRUCT) 
 
 #define VIPS_ARGUMENT_OPTIONAL_INPUT \
 	(VIPS_ARGUMENT_INPUT | \
-	 VIPS_ARGUMENT_CONSTRUCT | \
-	 VIPS_ARGUMENT_SET_ONCE)
+	 VIPS_ARGUMENT_CONSTRUCT)
 
 #define VIPS_ARGUMENT_REQUIRED_OUTPUT \
 	(VIPS_ARGUMENT_OUTPUT | \
 	 VIPS_ARGUMENT_REQUIRED | \
-	 VIPS_ARGUMENT_CONSTRUCT | \
-	 VIPS_ARGUMENT_SET_ONCE)
+	 VIPS_ARGUMENT_CONSTRUCT)
+
+#define VIPS_ARGUMENT_OPTIONAL_OUTPUT \
+	(VIPS_ARGUMENT_OUTPUT | \
+	 VIPS_ARGUMENT_CONSTRUCT)
+
+extern int _vips__argument_id;
+
+#define VIPS_ARG_IMAGE( CLASS, NAME, PRIORITY, LONG, DESC, FLAGS, OFFSET ) { \
+	GParamSpec *pspec; \
+	\
+	pspec = g_param_spec_object( (NAME), (LONG), (DESC),  \
+		VIPS_TYPE_IMAGE, \
+		G_PARAM_READWRITE ); \
+	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
+		_vips__argument_id++, pspec ); \
+	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
+		pspec, (FLAGS), (PRIORITY), (OFFSET) ); \
+}
+
+#define VIPS_ARG_BOOL( CLASS, NAME, PRIORITY, LONG, DESC, \
+	FLAGS, OFFSET, VALUE ) { \
+	GParamSpec *pspec; \
+	\
+	pspec = g_param_spec_boolean( (NAME), (LONG), (DESC), \
+		(VALUE), \
+		G_PARAM_READWRITE ); \
+	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
+		_vips__argument_id++, pspec ); \
+	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
+		pspec, (FLAGS), (PRIORITY), (OFFSET) ); \
+}
+
+#define VIPS_ARG_DOUBLE( CLASS, NAME, PRIORITY, LONG, DESC, \
+	FLAGS, OFFSET, MIN, MAX, VALUE ) { \
+	GParamSpec *pspec; \
+	\
+	pspec = g_param_spec_double( (NAME), (LONG), (DESC), \
+		(MIN), (MAX), (VALUE), \
+		G_PARAM_READWRITE );\
+	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
+		_vips__argument_id++, pspec ); \
+	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
+		pspec, (FLAGS), (PRIORITY), (OFFSET) ); \
+}
+
+#define VIPS_ARG_BOXED( CLASS, NAME, PRIORITY, LONG, DESC, \
+	FLAGS, OFFSET, TYPE ) { \
+	GParamSpec *pspec; \
+	\
+	pspec = g_param_spec_boxed( (NAME), (LONG), (DESC), \
+		(TYPE), \
+		G_PARAM_READWRITE );\
+	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
+		_vips__argument_id++, pspec ); \
+	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
+		pspec, (FLAGS), (PRIORITY), (OFFSET) ); \
+}
+
+#define VIPS_ARG_INT( CLASS, NAME, PRIORITY, LONG, DESC, \
+	FLAGS, OFFSET, MIN, MAX, VALUE ) { \
+	GParamSpec *pspec; \
+	\
+	pspec = g_param_spec_int( (NAME), (LONG), (DESC), \
+		(MIN), (MAX), (VALUE), \
+		G_PARAM_READWRITE );\
+	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
+		_vips__argument_id++, pspec ); \
+	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
+		pspec, (FLAGS), (PRIORITY), (OFFSET) ); \
+}
+
+#define VIPS_ARG_UINT64( CLASS, NAME, PRIORITY, LONG, DESC, \
+	FLAGS, OFFSET, MIN, MAX, VALUE ) { \
+	GParamSpec *pspec; \
+	\
+	pspec = g_param_spec_uint64( (NAME), (LONG), (DESC), \
+		(MIN), (MAX), (VALUE), \
+		G_PARAM_READWRITE );\
+	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
+		_vips__argument_id++, pspec ); \
+	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
+		pspec, (FLAGS), (PRIORITY), (OFFSET) ); \
+}
+
+#define VIPS_ARG_ENUM( CLASS, NAME, PRIORITY, LONG, DESC, \
+	FLAGS, OFFSET, TYPE, VALUE ) { \
+	GParamSpec *pspec; \
+	\
+	pspec = g_param_spec_enum( (NAME), (LONG), (DESC), \
+		(TYPE), (VALUE), \
+		G_PARAM_READWRITE );\
+	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
+		_vips__argument_id++, pspec ); \
+	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
+		pspec, (FLAGS), (PRIORITY), (OFFSET) ); \
+}
+
+#define VIPS_ARG_FLAGS( CLASS, NAME, PRIORITY, LONG, DESC, \
+	FLAGS, OFFSET, TYPE, VALUE ) { \
+	GParamSpec *pspec; \
+	\
+	pspec = g_param_spec_flags( (NAME), (LONG), (DESC), \
+		(TYPE), (VALUE), \
+		G_PARAM_READWRITE );\
+	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
+		_vips__argument_id++, pspec ); \
+	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
+		pspec, (FLAGS), (PRIORITY), (OFFSET) ); \
+}
+
+#define VIPS_ARG_STRING( CLASS, NAME, PRIORITY, LONG, DESC, FLAGS, OFFSET, \
+	VALUE ) { \
+	GParamSpec *pspec; \
+	\
+	pspec = g_param_spec_string( (NAME), (LONG), (DESC), \
+		(VALUE), \
+		G_PARAM_READWRITE ); \
+	g_object_class_install_property( G_OBJECT_CLASS( CLASS ), \
+		_vips__argument_id++, pspec ); \
+	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
+		pspec, (FLAGS), (PRIORITY), (OFFSET) ); \
+}
+
+#define VIPS_ARG_POINTER( CLASS, NAME, PRIORITY, LONG, DESC, FLAGS, OFFSET ) { \
+	GParamSpec *pspec; \
+	\
+	pspec = g_param_spec_pointer( (NAME), (LONG), (DESC), \
+		G_PARAM_READWRITE ); \
+	g_object_class_install_property( gobject_class,  \
+		_vips__argument_id++, pspec ); \
+	vips_object_class_install_argument( VIPS_OBJECT_CLASS( CLASS ), \
+		pspec, (FLAGS), (PRIORITY), (OFFSET) ); \
+}
 
 /* Keep one of these for every argument.
  */
@@ -117,6 +256,7 @@ typedef struct _VipsArgumentClass {
 	VipsObjectClass *object_class;
 
 	VipsArgumentFlags flags;
+	int priority;		/* Order args by this */
 	guint offset;		/* G_STRUCT_OFFSET of member in object */
 } VipsArgumentClass;
 
@@ -125,6 +265,10 @@ typedef struct _VipsArgumentClass {
 typedef struct _VipsArgumentInstance {
 	VipsArgument parent;
 
+	/* The class we are part of.
+	 */
+	VipsArgumentClass *argument_class;
+
 	/* The object we are attached to.
 	 */
 	VipsObject *object;
@@ -145,16 +289,28 @@ typedef struct _VipsArgumentInstance {
 typedef GHashTable VipsArgumentTable;
 
 VipsArgumentInstance *vips__argument_get_instance( VipsArgumentClass *,
-	VipsObject *);
+	VipsObject * );
 VipsArgument *vips__argument_table_lookup( VipsArgumentTable *, 
-	GParamSpec *);
+	GParamSpec * );
 void vips__object_set_member( VipsObject *object, GParamSpec *pspec,
 	GObject **member, GObject *argument );
 typedef void *(*VipsArgumentMapFn)( VipsObject *, GParamSpec *,
 	VipsArgumentClass *, VipsArgumentInstance *, void *a, void *b );
 void *vips_argument_map( VipsObject *object, 
 	VipsArgumentMapFn fn, void *a, void *b );
-void vips_argument_dispose_all( VipsObject *object );
+typedef void *(*VipsArgumentClassMapFn)( VipsObjectClass *, GParamSpec *,
+	VipsArgumentClass *, void *a, void *b );
+void *vips_argument_class_map( VipsObjectClass *object_class, 
+	VipsArgumentClassMapFn fn, void *a, void *b );
+int vips_object_get_argument( VipsObject *object, const char *name,
+	GParamSpec **pspec,
+	VipsArgumentClass **argument_class,
+	VipsArgumentInstance **argument_instance );
+gboolean vips_object_get_argument_assigned( VipsObject *object, 
+	const char *name );
+VipsArgumentFlags vips_object_get_argument_flags( VipsObject *object, 
+	const char *name );
+int vips_object_get_argument_priority( VipsObject *object, const char *name );
 
 /* We have to loop over an objects args in several places, and we can't always
  * use vips_argument_map(), the preferred looper. Have the loop code as a
@@ -182,6 +338,40 @@ void vips_argument_dispose_all( VipsObject *object );
 
 #define VIPS_ARGUMENT_FOR_ALL_END } } }
 
+/* And some macros to collect args from a va list. 
+ */
+#define VIPS_ARGUMENT_COLLECT_SET( PSPEC, ARG_CLASS, AP ) \
+	if( (ARG_CLASS->flags & VIPS_ARGUMENT_INPUT) ) { \
+		GValue value = { 0, }; \
+		gchar *error = NULL; \
+ 		\
+		/* Input args are given inline, eg. ("factor", 12.0)  \
+		 * and must be collected. \
+		 */ \
+		g_value_init( &value, G_PARAM_SPEC_VALUE_TYPE( PSPEC ) ); \
+		G_VALUE_COLLECT( &value, AP, 0, &error ); \
+		\
+		/* Don't bother with the error message. \
+		 */ \
+		if( error ) { \
+			VIPS_DEBUG_MSG( "VIPS_OBJECT_COLLECT_SET: err\n" ); \
+			g_free( error ); \
+		}
+
+#define VIPS_ARGUMENT_COLLECT_GET( PSPEC, ARG_CLASS, AP ) \
+		g_value_unset( &value ); \
+	} \
+	else if( (ARG_CLASS->flags & VIPS_ARGUMENT_OUTPUT) ) { \
+		void **arg __attribute__ ((unused)); \
+ 		\
+		/* Output args are a pointer to where to send the \
+		 * result. \
+		 */ \
+		arg = va_arg( AP, void ** ); 
+
+#define VIPS_ARGUMENT_COLLECT_END \
+	} 
+
 #define VIPS_TYPE_OBJECT (vips_object_get_type())
 #define VIPS_OBJECT( obj ) \
 	(G_TYPE_CHECK_INSTANCE_CAST( (obj), VIPS_TYPE_OBJECT, VipsObject ))
@@ -197,7 +387,15 @@ void vips_argument_dispose_all( VipsObject *object );
 struct _VipsObject {
 	GObject parent_object;
 
-	gboolean constructed;		/* Construct done and checked */
+	/* Set after ->build() has run succesfully: construct is fully done
+	 * and checked.
+	 */
+	gboolean constructed;
+
+	/* Set for static objects which are allocated at startup and never
+	 * freed. These objects are ommitted from leak reports.
+	 */
+	gboolean static_object;
 
 	/* Table of argument instances for this class and any derived classes.
 	 */
@@ -225,12 +423,17 @@ struct _VipsObjectClass {
 	int (*build)( VipsObject *object );
 
 	/* Try to print something about the class, handy for help displays.
+	 * Keep to one line.
 	 */
-	void (*print_class)( struct _VipsObjectClass *, VipsBuf * );
+	void (*summary_class)( struct _VipsObjectClass *, VipsBuf * );
 
-	/* Try to print something about the object, handy for debugging.
+	/* Try to print a one-line summary for the object, handy for debugging.
 	 */
-	void (*print)( VipsObject *, VipsBuf * );
+	void (*summary)( VipsObject *, VipsBuf * );
+
+	/* Try to print everything about the object, handy for debugging.
+	 */
+	void (*dump)( VipsObject *, VipsBuf * );
 
 	/* Sanity-check the object. Print messages and stuff. 
 	 * Handy for debugging.
@@ -300,7 +503,7 @@ struct _VipsObjectClass {
 	GSList *argument_table_traverse;
 };
 
-gboolean vips_pspec_value_is_null( GParamSpec *pspec, const GValue *value );
+gboolean vips_value_is_null( GParamSpec *psoec, const GValue *value );
 void vips_object_set_property( GObject *gobject, 
 	guint property_id, const GValue *value, GParamSpec *pspec );
 void vips_object_get_property( GObject *gobject, 
@@ -308,15 +511,22 @@ void vips_object_get_property( GObject *gobject,
 
 void vips_object_preclose( VipsObject *object );
 int vips_object_build( VipsObject *object );
-void vips_object_print_class( VipsObjectClass *klass );
-void vips_object_print( VipsObject *object );
+
+void vips_object_summary_class( VipsObjectClass *klass, VipsBuf *buf );
+void vips_object_summary( VipsObject *object, VipsBuf *buf );
+void vips_object_dump( VipsObject *object, VipsBuf *buf );
+
+void vips_object_print_summary_class( VipsObjectClass *klass );
+void vips_object_print_summary( VipsObject *object );
+void vips_object_print_dump( VipsObject *object );
 void vips_object_print_name( VipsObject *object );
+
 gboolean vips_object_sanity( VipsObject *object );
 
 GType vips_object_get_type( void );
 
-void vips_object_class_install_argument( VipsObjectClass *,
-	GParamSpec *pspec, VipsArgumentFlags flags, guint offset );
+void vips_object_class_install_argument( VipsObjectClass *, GParamSpec *pspec, 
+	VipsArgumentFlags flags, int priority, guint offset );
 int vips_object_set_argument_from_string( VipsObject *object, 
 	const char *name, const char *value );
 gboolean vips_object_get_argument_needs_string( VipsObject *object, 
@@ -335,26 +545,32 @@ void vips_object_to_string( VipsObject *object, VipsBuf *buf );
 
 void *vips_object_map( VipsSListMap2Fn fn, void *a, void *b );
 
-typedef void *(*VipsTypeMap)( GType, void * );
-typedef void *(*VipsTypeMap2)( GType, void *, void * );
-typedef void *(*VipsClassMap)( VipsObjectClass *, void * );
-void *vips_type_map( GType base, VipsTypeMap2 fn, void *a, void *b );
-void *vips_type_map_all( GType base, VipsTypeMap fn, void *a );
-void *vips_class_map_all( GType base, VipsClassMap fn, void *a );
-int vips_class_depth( VipsObjectClass *klass );
-VipsObjectClass *vips_class_find( const char *basename, const char *nickname );
+typedef void *(*VipsTypeMapFn)( GType, void * );
+typedef void *(*VipsTypeMap2Fn)( GType, void *, void * );
+typedef void *(*VipsClassMapFn)( VipsObjectClass *, void * );
+void *vips_type_map( GType base, VipsTypeMap2Fn fn, void *a, void *b );
+void *vips_type_map_all( GType base, VipsTypeMapFn fn, void *a );
+int vips_type_depth( GType type );
 GType vips_type_find( const char *basename, const char *nickname );
 
+void *vips_class_map_all( GType base, VipsClassMapFn fn, void *a );
+VipsObjectClass *vips_class_find( const char *basename, const char *nickname );
+
+VipsObject **vips_object_local_array( VipsObject *parent, int n );
+
 void vips_object_local_cb( VipsObject *vobject, GObject *gobject );
 #define vips_object_local( V, G ) \
 	(g_signal_connect( V, "close", \
 		G_CALLBACK( vips_object_local_cb ), G ), 0)
 
+void vips_object_set_static( VipsObject *object, gboolean static_object );
 void vips_object_print_all( void );
 void vips_object_sanity_all( void );
 
 void vips_object_rewind( VipsObject *object );
 
+void vips_object_unref_outputs( VipsObject *object );
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
