@@ -34,6 +34,8 @@
 extern "C" {
 #endif /*__cplusplus*/
 
+#include <vips/vips.h>
+
 #define VIPS_TYPE_OPERATION (vips_operation_get_type())
 #define VIPS_OPERATION( obj ) \
 	(G_TYPE_CHECK_INSTANCE_CAST( (obj), \
@@ -54,15 +56,30 @@ typedef gboolean (*VipsOperationBuildFn)( VipsObject * );
 typedef struct _VipsOperation {
 	VipsObject parent_instance;
 
+	/* When we added this operation to cache .. used to find LRU for
+	 * flush.
+	 */
+	int time;
+
+	/* Keep the hash here.
+	 */
+	guint hash;
+	gboolean found_hash;
+
 } VipsOperation;
 
 typedef struct _VipsOperationClass {
 	VipsObjectClass parent_class;
 
+	/* Print the usage message.
+	 */
+	void (*usage)( struct _VipsOperationClass *, VipsBuf * );
 } VipsOperationClass;
 
 GType vips_operation_get_type( void );
 
+void vips_operation_class_print_usage( VipsOperationClass *operation_class );
+
 int vips_operation_call_valist( VipsOperation *operation, va_list ap );
 VipsOperation *vips_operation_new( const char *name ); 
 int vips_call( const char *operation_name, ... );
@@ -71,6 +88,19 @@ int vips_call_split( const char *operation_name, va_list optional, ... );
 void vips_call_options( GOptionGroup *group, VipsOperation *operation );
 int vips_call_argv( VipsOperation *operation, int argc, char **argv );
 
+void vips_cache_drop_all( void );
+int vips_cache_operation_buildp( VipsOperation **operation );
+VipsOperation *vips_cache_operation_build( VipsOperation *operation );
+void vips_cache_set_max( int max );
+void vips_cache_set_max_mem( size_t max_mem );
+int vips_cache_get_max( void );
+int vips_cache_get_size( void );
+size_t vips_cache_get_max_mem( void );
+int vips_cache_get_max_files( void );
+void vips_cache_set_max_files( int max_files );
+void vips_cache_set_dump( gboolean dump );
+void vips_cache_set_trace( gboolean trace );
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
