@@ -55,8 +55,6 @@ extern "C" {
  */
 #define VIPS_SIZEOF_HEADER (64)
 
-typedef unsigned char PEL;			/* useful datum		*/
-
 /* What we track for each mmap window. Have a list of these on an openin
  * VipsImage.
  */
@@ -66,9 +64,9 @@ typedef struct {
 
 	int top; 		/* Area of image we have mapped, in pixels */
 	int height;
-	char *data;		/* First pixel of line 'top' */
+	VipsPel *data;		/* First pixel of line 'top' */
 
-	PEL *baseaddr;		/* Base of window */
+	void *baseaddr;		/* Base of window */
 	size_t length;		/* Size of window */
 } VipsWindow;
 
@@ -90,7 +88,7 @@ typedef struct {
  * update operation and we'd need to _remove() and _insert() on every list
  * operation.
  */
-typedef struct {
+typedef struct _VipsBufferCacheList {
 	GSList *buffers;	/* GSList of VipsBuffer* */
 	GThread *thread;	/* Just for sanity checking */
 	struct _VipsImage *im;
@@ -99,14 +97,14 @@ typedef struct {
 
 /* What we track for each pixel buffer. 
  */
-typedef struct {
+typedef struct _VipsBuffer {
 	int ref_count;		/* # of regions referencing us */
 	struct _VipsImage *im;	/* VipsImage we are attached to */
 
 	VipsRect area;		/* Area this pixel buffer covers */
 	gboolean done;		/* Calculated and in cache */
 	VipsBufferCache *cache;
-	char *buf;		/* Private malloc() area */
+	VipsPel *buf;		/* Private malloc() area */
 	size_t bsize;		/* Size of private malloc() */
 } VipsBuffer;
 
@@ -124,20 +122,19 @@ void vips_buffer_print( VipsBuffer *buffer );
 
 /* Region types.
  */
-typedef enum region_type {
+typedef enum _RegionType {
 	VIPS_REGION_NONE,
-	VIPS_REGION_BUFFER,	/* a pixel buffer */
-	VIPS_REGION_OTHER_REGION, /* memory on another region */
-	VIPS_REGION_OTHER_IMAGE,/* memory on another image */
-	VIPS_REGION_WINDOW	/* mmap() buffer on fd on another image */
+	VIPS_REGION_BUFFER,		/* A VipsBuffer */
+	VIPS_REGION_OTHER_REGION, 	/* Memory on another region */
+	VIPS_REGION_OTHER_IMAGE,	/* Memory on another image */
+	VIPS_REGION_WINDOW		/* A VipsWindow on fd */
 } RegionType;
 
 /* Private to iofuncs: the size of the `tiles' requested by 
- * vips_image_generate()
- * when acting as a data sink.
+ * vips_image_generate() when acting as a data sink.
  */
-#define VIPS__TILE_WIDTH (512)
-#define VIPS__TILE_HEIGHT (512)
+#define VIPS__TILE_WIDTH (128)
+#define VIPS__TILE_HEIGHT (128)
 
 /* The height of the strips for the other two request styles.
  */
@@ -158,13 +155,14 @@ typedef int (*VipsRegionFillFn)( struct _VipsRegion *, void * );
 int vips_region_fill( struct _VipsRegion *reg, 
 	VipsRect *r, VipsRegionFillFn fn, void *a );
 
-/* The new name for im_setupout(), still used by some old code.
- */
-int vips__image_write_prepare( struct _VipsImage *image );
+int vips__image_wio_output( struct _VipsImage *image );
+int vips__image_pio_output( struct _VipsImage *image );
 
-/* Start up various packages.
- */
-void vips_arithmetic_operation_init( void );
+VipsArgumentInstance *vips__argument_get_instance( 
+	VipsArgumentClass *argument_class,
+	VipsObject *object);
+VipsArgument *vips__argument_table_lookup( VipsArgumentTable *table, 
+	GParamSpec *pspec);
 
 #ifdef __cplusplus
 }
