@@ -94,10 +94,12 @@ extern "C" {
 #  endif
 #endif /*SWIG*/
 
+#include <vips/basic.h>
+
 #include <vips/buf.h>
 #include <vips/util.h>
 #include <vips/object.h>
-#include <vips/pool.h>
+#include <vips/type.h>
 
 #include <vips/version.h>
 #include <vips/rect.h>
@@ -116,11 +118,11 @@ extern "C" {
 #include <vips/threadpool.h>
 #include <vips/header.h>
 #include <vips/operation.h>
+#include <vips/foreign.h>
 
 #include <vips/enumtypes.h>
 
 #include <vips/arithmetic.h>
-#include <vips/boolean.h>
 #include <vips/relational.h>
 #include <vips/conversion.h>
 #include <vips/convolution.h>
@@ -150,6 +152,7 @@ extern "C" {
 const char *vips_get_argv0( void );
 int vips_init( const char *argv0 );
 void vips_check_init( void );
+void vips_shutdown( void );
 GOptionGroup *vips_get_option_group( void );
 
 const char *vips_version_string( void );
