@@ -40,6 +40,8 @@ extern "C" {
 /* Renamed types.
  */
 
+#define PEL VipsPel
+
 #define IM_BANDFMT_NOTSET VIPS_FORMAT_NOTSET
 #define IM_BANDFMT_UCHAR VIPS_FORMAT_UCHAR
 #define IM_BANDFMT_CHAR VIPS_FORMAT_CHAR
@@ -93,12 +95,7 @@ extern "C" {
 #define IM_DEG VIPS_DEG
 #define IM_PI VIPS_PI
 #define IM_RINT VIPS_RINT
-#define IM_NEW VIPS_NEW
-#define IM_ARRAY VIPS_ARRAY
-#define IM_SETSTR VIPS_SETSTR
 #define IM_ABS VIPS_ABS
-#define IM_FREE VIPS_FREE
-#define IM_FREEF VIPS_FREEF
 #define IM_NUMBER VIPS_NUMBER
 #define IM_CLIP VIPS_CLIP
 #define IM_CLIP_UCHAR VIPS_CLIP_UCHAR
@@ -163,7 +160,7 @@ extern "C" {
 #define im__end_eval vips_image_posteval
 #define im_invalidate vips_image_invalidate_all
 #define im_isfile vips_image_isfile
-#define im_printdesc( I ) vips_object_print( VIPS_OBJECT( I ) )
+#define im_printdesc( I ) vips_object_print_dump( VIPS_OBJECT( I ) )
 #define im_openout( F ) vips_image_new_mode( F, "w" )
 #define im_setbuf( F ) vips_image_new( "t" )
 
@@ -175,7 +172,7 @@ extern "C" {
 		type, xres, yres )
 
 #define im__open_image_file vips__open_image_read
-#define im_setupout( I ) vips__image_write_prepare( I )
+#define im_setupout vips_image_write_prepare
 #define im_writeline( Y, IM, P ) vips_image_write_line( IM, Y, P )
 
 #define im_prepare vips_region_prepare
@@ -266,8 +263,8 @@ VipsDemandStyle im_char2dhint( const char *str );
 #define im_rect_dup vips_rect_dup
 #define im_rect_normalise vips_rect_normalise
 
-#define im_demand_hint vips_demand_hint
-#define im_demand_hint_array vips_demand_hint_array
+int im_demand_hint (IMAGE * im, VipsDemandStyle hint, ...);
+#define im_demand_hint_array( A, B, C ) (vips_demand_hint_array( A, B, C ), 0)
 
 #define im_start_one vips_start_one
 #define im_stop_one vips_stop_one
@@ -275,9 +272,11 @@ VipsDemandStyle im_char2dhint( const char *str );
 #define im_stop_many vips_stop_many
 #define im_allocate_input_array vips_allocate_input_array
 #define im_start_fn VipsStartFn
-#define im_generate_fn VipsGenerateFn
+typedef int (*im_generate_fn)( VipsRegion *out, void *seq, void *a, void *b );
 #define im_stop_fn VipsStopFn
-#define im_generate vips_image_generate
+int im_generate( VipsImage *im,
+	im_start_fn start, im_generate_fn generate, im_stop_fn stop,
+	void *a, void *b );
 
 #define im__mmap vips__mmap
 #define im__munmap vips__munmap
@@ -286,20 +285,57 @@ VipsDemandStyle im_char2dhint( const char *str );
 #define im_remapfilerw vips_remapfilerw
 
 #define im__print_renders vips__print_renders
-#define im_cache vips_image_cache
 
-#define im_malloc vips_malloc
+int im_cache( IMAGE *in, IMAGE *out, int width, int height, int max );
+
+#define IM_FREEF( F, S ) \
+G_STMT_START { \
+        if( S ) { \
+                (void) F( (S) ); \
+                (S) = 0; \
+        } \
+} G_STMT_END
+
+/* Can't just use VIPS_FREEF(), we want the extra cast to void on the argument
+ * to vips_free() to make sure we can work for "const char *" variables.
+ */
+#define IM_FREE( S ) \
+G_STMT_START { \
+        if( S ) { \
+                (void) im_free( (void *) (S) ); \
+                (S) = 0; \
+        } \
+} G_STMT_END
+
+#define IM_SETSTR( S, V ) \
+G_STMT_START { \
+        const char *sst = (V); \
+	\
+        if( (S) != sst ) { \
+                if( !(S) || !sst || strcmp( (S), sst ) != 0 ) { \
+                        IM_FREE( S ); \
+                        if( sst ) \
+                                (S) = im_strdup( NULL, sst ); \
+                } \
+        } \
+} G_STMT_END
+
+#define im_malloc( IM, SZ ) \
+	(vips_malloc( VIPS_OBJECT( IM ), (SZ) ))
 #define im_free vips_free
-#define im_strdup vips_strdup
+#define im_strdup( IM, STR ) \
+	(vips_strdup( VIPS_OBJECT( IM ), (STR) ))
+#define IM_NEW( IM, T ) ((T *) im_malloc( (IM), sizeof( T )))
+#define IM_ARRAY( IM, N, T ) ((T *) im_malloc( (IM), (N) * sizeof( T )))
 
 #define im_incheck vips_image_wio_input
-#define im_outcheck vips_image_wio_output
+#define im_outcheck( I ) (0)
 #define im_rwcheck vips_image_inplace
 #define im_pincheck vips_image_pio_input
-#define im_poutcheck vips_image_pio_output
+#define im_poutcheck( I ) (0)
 
-#define im_iocheck( I, O ) (im_incheck( I ) || im_outcheck( O ))
-#define im_piocheck( I, O ) (im_pincheck( I ) || im_poutcheck( O ))
+#define im_iocheck( I, O ) im_incheck( I )
+#define im_piocheck( I, O ) im_pincheck( I )
 
 #define im_check_uncoded vips_check_uncoded 
 #define im_check_coding_known vips_check_coding_known 
@@ -377,31 +413,32 @@ int im_wrapmany( VipsImage **in, VipsImage *out,
 #define im_updatehist vips_image_history_args
 #define im_history_get vips_image_get_history
 
-#define im_save_string_get vips_save_string_get
-#define im_save_string_set vips_save_string_set
-#define im_save_string_setf vips_save_string_setf
+#define im_save_string_get vips_value_get_save_string
+#define im_save_string_set vips_value_set_save_string
+#define im_save_string_setf vips_value_set_save_stringf
 
-#define im_ref_string_set vips_ref_string_set
-#define im_ref_string_get vips_ref_string_get
-#define im_ref_string_get_length vips_ref_string_get_length
+#define im_ref_string_set vips_value_set_ref_string
+#define im_ref_string_get( V ) vips_value_get_ref_string( V, NULL )
+size_t im_ref_string_get_length( const GValue *value );
 
-#define im_blob_get vips_blob_get
-#define im_blob_set vips_blob_set
+#define im_blob_get vips_value_get_blob
+#define im_blob_set vips_value_set_blob
 
-#define im_meta_set vips_image_set
+#define im_meta_set( A, B, C ) (vips_image_set( A, B, C ), 0)
 #define im_meta_remove vips_image_remove
 #define im_meta_get vips_image_get
 #define im_meta_get_typeof vips_image_get_typeof
 
-#define im_meta_set_int vips_image_set_int
+#define im_meta_set_int( A, B, C ) (vips_image_set_int( A, B, C ), 0)
 #define im_meta_get_int vips_image_get_int
-#define im_meta_set_double vips_image_set_double
+#define im_meta_set_double( A, B, C ) (vips_image_set_double( A, B, C ), 0)
 #define im_meta_get_double vips_image_get_double
-#define im_meta_set_area vips_image_set_area
+#define im_meta_set_area( A, B, C, D ) (vips_image_set_area( A, B, C, D ), 0)
 #define im_meta_get_area vips_image_get_area
-#define im_meta_set_string vips_image_set_string
+#define im_meta_set_string( A, B, C ) (vips_image_set_string( A, B, C ), 0)
 #define im_meta_get_string vips_image_get_string
-#define im_meta_set_blob vips_image_set_blob
+#define im_meta_set_blob( A, B, C, D, E ) \
+	(vips_image_set_blob( A, B, C, D, E ), 0)
 #define im_meta_get_blob vips_image_get_blob
 
 #define im_semaphore_t VipsSemaphore
@@ -416,10 +453,6 @@ int im_wrapmany( VipsImage **in, VipsImage *out,
 #define im__open_image_read vips__open_image_read
 #define im_image_open_input vips_image_open_input
 #define im_image_open_output vips_image_open_output
-#define im__read_4byte vips__read_4byte
-#define im__read_2byte vips__read_2byte
-#define im__write_4byte vips__write_4byte
-#define im__write_2byte vips__write_2byte
 #define im__has_extension_block vips__has_extension_block
 #define im__read_extension_block vips__read_extension_block
 #define im__write_extension_block vips__write_extension_block
@@ -486,6 +519,137 @@ int im_wrapmany( VipsImage **in, VipsImage *out,
 #define im_concurrency_set vips_concurrency_set
 #define im_concurrency_get vips_concurrency_get
 
+int im_add( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_subtract( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_multiply( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_divide( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_min( VipsImage *in, double *out );
+int im_minpos( VipsImage *in, int *xpos, int *ypos, double *out );
+int im_max( VipsImage *in, double *out );
+int im_maxpos( VipsImage *in, int *xpos, int *ypos, double *out );
+int im_avg( VipsImage *in, double *out );
+int im_deviate( VipsImage *in, double *out );
+int im_invert( VipsImage *in, VipsImage *out );
+int im_lintra( double a, VipsImage *in, double b, VipsImage *out );
+int im_lintra_vec( int n, double *a, VipsImage *in, double *b, VipsImage *out );
+int im_abs( VipsImage *in, VipsImage *out );
+int im_sign( VipsImage *in, VipsImage *out );
+DOUBLEMASK *im_stats( VipsImage *in );
+DOUBLEMASK *im_measure_area( VipsImage *im, 
+	int left, int top, int width, int height, 
+	int h, int v, 
+	int *sel, int nsel, const char *name );
+
+int im_sintra( VipsImage *in, VipsImage *out );
+int im_costra( VipsImage *in, VipsImage *out );
+int im_tantra( VipsImage *in, VipsImage *out );
+int im_asintra( VipsImage *in, VipsImage *out );
+int im_acostra( VipsImage *in, VipsImage *out );
+int im_atantra( VipsImage *in, VipsImage *out );
+int im_logtra( VipsImage *in, VipsImage *out );
+int im_log10tra( VipsImage *in, VipsImage *out );
+int im_exptra( VipsImage *in, VipsImage *out );
+int im_exp10tra( VipsImage *in, VipsImage *out );
+
+int im_floor( VipsImage *in, VipsImage *out );
+int im_rint( VipsImage *in, VipsImage *out );
+int im_ceil( VipsImage *in, VipsImage *out );
+
+int im_equal( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_notequal( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_less( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_lesseq( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_more( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_moreeq( VipsImage *in1, VipsImage *in2, VipsImage *out );
+
+int im_andimage( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_orimage( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_eorimage( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_andimage_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_orimage_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_eorimage_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_andimageconst( VipsImage *in, VipsImage *out, double c );
+int im_orimageconst( VipsImage *in, VipsImage *out, double c );
+int im_eorimageconst( VipsImage *in, VipsImage *out, double c );
+
+int im_shiftleft_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_shiftleft( VipsImage *in, VipsImage *out, int n );
+int im_shiftright_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_shiftright( VipsImage *in, VipsImage *out, int n );
+
+int im_remainder( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_remainder_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_remainderconst( VipsImage *in, VipsImage *out, double c );
+
+int im_powtra( VipsImage *in, VipsImage *out, double e );
+int im_powtra_vec( VipsImage *in, VipsImage *out, int n, double *e );
+int im_expntra( VipsImage *in, VipsImage *out, double e );
+int im_expntra_vec( VipsImage *in, VipsImage *out, int n, double *e );
+
+int im_equal_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_notequal_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_less_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_lesseq_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_more_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_moreeq_vec( VipsImage *in, VipsImage *out, int n, double *c );
+int im_equalconst( VipsImage *in, VipsImage *out, double c );
+int im_notequalconst( VipsImage *in, VipsImage *out, double c );
+int im_lessconst( VipsImage *in, VipsImage *out, double c );
+int im_lesseqconst( VipsImage *in, VipsImage *out, double c );
+int im_moreconst( VipsImage *in, VipsImage *out, double c );
+int im_moreeqconst( VipsImage *in, VipsImage *out, double c );
+
+int im_copy( VipsImage *in, VipsImage *out );
+int im_copy_set( VipsImage *in, VipsImage *out, 
+	VipsInterpretation interpretation, 
+	float xres, float yres, int xoffset, int yoffset );
+int im_copy_set_meta( VipsImage *in, VipsImage *out, 
+	const char *field, GValue *value );
+int im_copy_morph( VipsImage *in, VipsImage *out, 
+	int bands, VipsBandFormat format, VipsCoding coding );
+int im_copy_swap( VipsImage *in, VipsImage *out );
+int im_copy_native( VipsImage *in, VipsImage *out, gboolean is_msb_first );
+int im_embed( VipsImage *in, VipsImage *out, 
+	int type, int x, int y, int width, int height );
+int im_fliphor( VipsImage *in, VipsImage *out );
+int im_flipver( VipsImage *in, VipsImage *out );
+int im_insert( VipsImage *main, VipsImage *sub, VipsImage *out, int x, int y );
+int im_insert_noexpand( VipsImage *main, VipsImage *sub, VipsImage *out, int x, int y );
+int im_lrjoin( VipsImage *left, VipsImage *right, VipsImage *out );
+int im_tbjoin( VipsImage *top, VipsImage *bottom, VipsImage *out );
+int im_extract_area( VipsImage *in, VipsImage *out, 
+	int left, int top, int width, int height );
+int im_extract_band( VipsImage *in, VipsImage *out, int band );
+int im_extract_bands( VipsImage *in, VipsImage *out, int band, int nbands );
+int im_extract_areabands( VipsImage *in, VipsImage *out,
+	int left, int top, int width, int height, int band, int nbands );
+int im_replicate( VipsImage *in, VipsImage *out, int across, int down );
+int im_clip2fmt( VipsImage *in, VipsImage *out, VipsBandFormat fmt );
+int im_bandjoin( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_gbandjoin( VipsImage **in, VipsImage *out, int n );
+int im_black( VipsImage *out, int x, int y, int bands );
+
+int im_c2amph( VipsImage *in, VipsImage *out );
+int im_c2rect( VipsImage *in, VipsImage *out );
+int im_c2imag( VipsImage *in, VipsImage *out );
+int im_c2real( VipsImage *in, VipsImage *out );
+int im_ri2c( VipsImage *in1, VipsImage *in2, VipsImage *out );
+
+int im_rot90( VipsImage *in, VipsImage *out );
+int im_rot180( VipsImage *in, VipsImage *out );
+int im_rot270( VipsImage *in, VipsImage *out );
+
+int im_ifthenelse( VipsImage *c, VipsImage *a, VipsImage *b, VipsImage *out );
+int im_blend( VipsImage *c, VipsImage *a, VipsImage *b, VipsImage *out );
+
+DOUBLEMASK *im_vips2mask( VipsImage *in, const char *filename );
+int im_mask2vips( DOUBLEMASK *in, VipsImage *out );
+
+int im_bandmean( VipsImage *in, VipsImage *out );
+int im_recomb( VipsImage *in, VipsImage *out, DOUBLEMASK *recomb );
+
+int im_argb2rgba( VipsImage *in, VipsImage *out );
+
 /* ruby-vips uses this
  */
 #define vips_class_map_concrete_all vips_class_map_all
