@@ -44,10 +44,6 @@
 #include <vips/vips.h>
 #include <vips/buf.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /**
  * SECTION: buf
  * @short_description: a string you can append to
@@ -78,7 +74,7 @@
 
 /* Largest string we can append in one operation.
  */
-#define MAX_STRSIZE (16000)
+#define MAX_STRSIZE (100000)
 
 /** 
  * VIPS_BUF_STATIC:
@@ -505,6 +501,55 @@ vips_buf_appendgv( VipsBuf *buf, GValue *value )
 	return( result );
 }
 
+/**
+ * vips_buf_append_size:
+ * @buf: the buffer
+ * @n: the number of bytes
+ *
+ * Turn a number of bytes into a sensible string ... eg "12", "12KB", "12MB",
+ * "12GB" etc.
+ * 
+ * Returns: %FALSE on overflow, %TRUE otherwise.
+ */
+gboolean
+vips_buf_append_size( VipsBuf *buf, size_t n )
+{
+	const static char *names[] = { 
+		/* File length unit.
+		 */
+		N_( "bytes" ), 
+
+		/* Kilo byte unit.
+		 */
+		N_( "KB" ), 
+
+		/* Mega byte unit.
+		 */
+		N_( "MB" ), 
+
+		/* Giga byte unit.
+		 */
+		N_( "GB" ), 
+
+		/* Tera byte unit.
+		 */
+		N_( "TB" ) 
+	};
+
+	double sz = n;
+	int i;
+
+	for( i = 0; sz > 1024 && i < VIPS_NUMBER( names ); sz /= 1024, i++ )
+		;
+
+	if( i == 0 )
+		/* No decimal places for bytes.
+		 */
+		return( vips_buf_appendf( buf, "%g %s", sz, _( names[i] ) ) );
+	else
+		return( vips_buf_appendf( buf, "%.2f %s", sz, _( names[i] ) ) );
+}
+
 /**
  * vips_buf_all:
  * @buf: the buffer
