@@ -67,10 +67,6 @@
 #include <lmerr.h>
 #endif /*OS_WIN32*/
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /**
  * SECTION: error
  * @short_description: error messages and error handling
@@ -392,7 +388,8 @@ vips_warn( const char *domain, const char *fmt, ... )
  * @Varargs: arguments to the format string
  *
  * Sends a formatted error message to stderr, then sends the contents of the
- * error buffer, if any, then terminates the program with an error code.
+ * error buffer, if any, then shuts down vips and terminates the program with 
+ * an error code.
  *
  * @fmt may be %NULL, in which case only the error buffer is printed before
  * exiting.
@@ -416,6 +413,8 @@ vips_error_exit( const char *fmt, ... )
 
 	fprintf( stderr, "%s", vips_error_buffer() );
 
+	vips_shutdown();
+
 	exit( 1 );
 }
 
@@ -748,7 +747,7 @@ vips_check_format( const char *domain, VipsImage *im, VipsBandFormat fmt )
 	if( im->BandFmt != fmt ) {
 		vips_error( domain, 
 			_( "image must be %s" ), 
-			VIPS_ENUM_STRING( VIPS_TYPE_BAND_FORMAT, fmt ) );
+			vips_enum_string( VIPS_TYPE_BAND_FORMAT, fmt ) );
 		return( -1 );
 	}
 
@@ -1090,7 +1089,7 @@ vips_check_hist( const char *domain, VipsImage *im )
 			_( "histograms must have width or height 1" ) );
 		return( -1 );
 	}
-	if( im->Xsize * im->Ysize > 65536 ) {
+	if( VIPS_IMAGE_N_PELS( im ) > 65536 ) {
 		vips_error( domain, "%s", 
 			_( "histograms must have not have more than "
 				"65536 elements" ) );
@@ -1101,7 +1100,7 @@ vips_check_hist( const char *domain, VipsImage *im )
 }
 
 /**
- * vips_check_imask:
+ * vips_check_imask: (skip)
  * @domain: the originating domain for the error message
  * @mask: mask to check
  *
@@ -1129,7 +1128,7 @@ vips_check_imask( const char *domain, INTMASK *mask )
 }
 
 /**
- * vips_check_dmask:
+ * vips_check_dmask: (skip)
  * @domain: the originating domain for the error message
  * @mask: mask to check
  *
@@ -1157,7 +1156,7 @@ vips_check_dmask( const char *domain, DOUBLEMASK *mask )
 }
 
 /**
- * vips_check_dmask_1d:
+ * vips_check_dmask_1d: (skip)
  * @domain: the originating domain for the error message
  * @mask: mask to check
  *
