@@ -103,10 +103,6 @@
 #include <vips/thread.h>
 #include <vips/debug.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /**
  * SECTION: generate
  * @short_description: calculate pixels and pixel buffers
@@ -261,10 +257,8 @@ vips__link_map( VipsImage *image, VipsSListMap2Fn fn, void *a, void *b )
  * varargs friend vips_demand_hint().
  *
  * See also: vips_demand_hint(), vips_image_generate().
- *
- * Returns: 0 on success, or -1 on error.
  */
-int 
+void 
 vips_demand_hint_array( VipsImage *image, VipsDemandStyle hint, VipsImage **in )
 {
 	int i, len, nany;
@@ -301,13 +295,13 @@ vips_demand_hint_array( VipsImage *image, VipsDemandStyle hint, VipsImage **in )
 #ifdef DEBUG
         printf( "vips_demand_hint_array: set dhint for \"%s\" to %s\n",
 		im->filename, 
-		VIPS_ENUM_NICK( VIPS_TYPE_DEMAND_STYLE, image->dhint ) );
+		vips_enum_nick( VIPS_TYPE_DEMAND_STYLE, image->dhint ) );
 	printf( "\toperation requested %s\n", 
-		VIPS_ENUM_NICK( VIPS_TYPE_DEMAND_STYLE, hint ) );
+		vips_enum_nick( VIPS_TYPE_DEMAND_STYLE, hint ) );
 	printf( "\tinputs were:\n" );
 	printf( "\t" );
 	for( i = 0; in[i]; i++ )
-		printf( "%s ", VIPS_ENUM_NICK( VIPS_TYPE_DEMAND_STYLE, 
+		printf( "%s ", vips_enum_nick( VIPS_TYPE_DEMAND_STYLE, 
 			in[i]->dhint ) );
 	printf( "\n" );
 #endif /*DEBUG*/
@@ -321,8 +315,6 @@ vips_demand_hint_array( VipsImage *image, VipsDemandStyle hint, VipsImage **in )
 	 * vips_image_generate() and friends check this.
 	 */
 	image->hint_set = TRUE;
-
-	return( 0 );
 }
 
 /**
@@ -334,10 +326,8 @@ vips_demand_hint_array( VipsImage *image, VipsDemandStyle hint, VipsImage **in )
  * Build an array and call vips_demand_hint_array().
  *
  * See also: vips_demand_hint(), vips_image_generate().
- *
- * Returns: 0 on success, or -1 on error.
  */
-int 
+void 
 vips_demand_hint( VipsImage *image, VipsDemandStyle hint, ... )
 {
 	va_list ap;
@@ -346,15 +336,18 @@ vips_demand_hint( VipsImage *image, VipsDemandStyle hint, ... )
 
 	va_start( ap, hint );
 	for( i = 0; i < MAX_IMAGES && 
-		(ar[i] = va_arg( ap, VipsImage * )); i++ ) 
+		(ar[i] = va_arg( ap, VipsImage * )); i++ )
 		;
 	va_end( ap );
 	if( i == MAX_IMAGES ) {
-		vips_error( "vips_demand_hint", "%s", _( "too many images" ) );
-		return( -1 );
+		vips_warn( "vips_demand_hint", "%s", _( "too many images" ) );
+
+		/* Make sure we have a sentinel there.
+		 */
+		ar[i - 1] = NULL;
 	}
 
-	return( vips_demand_hint_array( image, hint, ar ) );
+	vips_demand_hint_array( image, hint, ar );
 }
 
 /**
@@ -574,9 +567,9 @@ write_vips( VipsRegion *region, VipsRect *area, void *a, void *b )
 /**
  * vips_image_generate:
  * @image: generate this image
- * @start: start sequences with this function
- * @generate: generate pixels with this function
- * @stop: stop sequences with this function
+ * @start_fn: start sequences with this function
+ * @generate_fn: generate pixels with this function
+ * @stop_fn: stop sequences with this function
  * @a: user data
  * @b: user data
  *
@@ -598,14 +591,14 @@ write_vips( VipsRegion *region, VipsRect *area, void *a, void *b )
  */
 int
 vips_image_generate( VipsImage *image,
-	VipsStartFn start, VipsGenerateFn generate, VipsStopFn stop,
+	VipsStartFn start_fn, VipsGenerateFn generate_fn, VipsStopFn stop_fn,
         void *a, void *b )
 {
         int res;
 
 	VIPS_DEBUG_MSG( "vips_image_generate: %p\n", image ); 
 
-	g_assert( generate );
+	g_assert( generate_fn );
 	g_assert( vips_object_sanity( VIPS_OBJECT( image ) ) );
 
 	if( !image->hint_set ) {
@@ -625,17 +618,17 @@ vips_image_generate( VipsImage *image,
         case VIPS_IMAGE_PARTIAL:
                 /* Output to partial image. Just attach functions and return.
                  */
-                if( image->generate || 
-			image->start || 
-			image->stop ) {
+                if( image->generate_fn || 
+			image->start_fn || 
+			image->stop_fn ) {
                         vips_error( "VipsImage", 
 				"%s", _( "generate() called twice" ) );
                         return( -1 );
                 }
 
-                image->start = start;
-                image->generate = generate;
-                image->stop = stop;
+                image->start_fn = start_fn;
+                image->generate_fn = generate_fn;
+                image->stop_fn = stop_fn;
                 image->client1 = a;
                 image->client2 = b;
  
@@ -650,9 +643,9 @@ vips_image_generate( VipsImage *image,
         case VIPS_IMAGE_OPENOUT:
                 /* Eval now .. sanity check.
                  */
-                if( image->generate || 
-			image->start || 
-			image->stop ) {
+                if( image->generate_fn || 
+			image->start_fn || 
+			image->stop_fn ) {
                         vips_error( "VipsImage", 
 				"%s", _( "generate() called twice" ) );
                         return( -1 );
@@ -660,15 +653,13 @@ vips_image_generate( VipsImage *image,
 
                 /* Attach callbacks.
                  */
-                image->start = start;
-                image->generate = generate;
-                image->stop = stop;
+                image->start_fn = start_fn;
+                image->generate_fn = generate_fn;
+                image->stop_fn = stop_fn;
                 image->client1 = a;
                 image->client2 = b;
 
-                /* Get output ready.
-                 */
-                if( vips__image_write_prepare( image ) )
+                if( vips_image_write_prepare( image ) )
                         return( -1 );
 
                 if( image->dtype == VIPS_IMAGE_OPENOUT ) 
@@ -689,7 +680,7 @@ vips_image_generate( VipsImage *image,
                  */
 		vips_error( "VipsImage", 
 			_( "unable to output to a %s image" ),
-			VIPS_ENUM_NICK( VIPS_TYPE_DEMAND_STYLE, 
+			vips_enum_nick( VIPS_TYPE_IMAGE_TYPE, 
 				image->dtype ) );
                 return( -1 );
         }
