@@ -50,6 +50,7 @@
 
 /*
 #define VIPS_DEBUG
+#define DEBUG
  */
 
 #ifdef HAVE_CONFIG_H
@@ -65,10 +66,6 @@
 #include <vips/internal.h>
 #include <vips/debug.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /**
  * SECTION: header
  * @short_description: get, set and walk image headers
@@ -156,9 +153,6 @@ static HeaderField int_field[] = {
 	{ "yoffset", G_STRUCT_OFFSET( VipsImage, Yoffset ) }
 };
 
-/* These are actually floats :-( how annoying. We report them as doubles for
- * consistency with the vips_image_*() functions.
- */
 static HeaderField double_field[] = {
 	{ "xres", G_STRUCT_OFFSET( VipsImage, Xres ) },
 	{ "yres", G_STRUCT_OFFSET( VipsImage, Yres ) }
@@ -190,9 +184,8 @@ static HeaderField old_double_field[] = {
 /* This is used by (eg.) VIPS_IMAGE_SIZEOF_ELEMENT() to calculate object
  * size.
  *
- * We can't use size_t or off_t because we have to work on some platforms
- * which have these as 32-bit and we need to calculate addresses in large
- * files. 
+ * It needs to be guint64 and not size_t since we use this as the basis for 
+ * image address calcs and they have to be 64-bit, even on 32-bit machines. 
  */
 const guint64 vips__image_sizeof_bandformat[] = {
 	sizeof( unsigned char ), 	/* VIPS_FORMAT_UCHAR */
@@ -209,7 +202,7 @@ const guint64 vips__image_sizeof_bandformat[] = {
 
 /* Return number of bytes for a band format, or -1 on error.
  */
-int 
+guint64 
 vips_format_sizeof( VipsBandFormat format )
 {
 	return( (format < 0 || format > VIPS_FORMAT_DPCOMPLEX) ?
@@ -287,8 +280,8 @@ meta_free( VipsMeta *meta )
 			g_slist_remove( meta->im->meta_traverse, meta );
 
 	g_value_unset( &meta->value );
-	VIPS_FREE( meta->field );
-	vips_free( meta );
+	g_free( meta->field );
+	g_free( meta );
 }
 
 static VipsMeta *
@@ -296,16 +289,11 @@ meta_new( VipsImage *image, const char *field, GValue *value )
 {
 	VipsMeta *meta;
 
-	if( !(meta = VIPS_NEW( NULL, VipsMeta )) )
-		return( NULL );
+	meta = g_new( VipsMeta, 1 );
 	meta->im = image;
 	meta->field = NULL;
 	memset( &meta->value, 0, sizeof( GValue ) );
-
-	if( !(meta->field = vips_strdup( NULL, field )) ) {
-		meta_free( meta );
-		return( NULL );
-	}
+	meta->field = g_strdup( field );
 
 	g_value_init( &meta->value, G_VALUE_TYPE( value ) );
 	g_value_copy( value, &meta->value );
@@ -347,79 +335,79 @@ meta_init( VipsImage *im )
 }
 
 int
-vips_image_get_width( VipsImage *image )
+vips_image_get_width( const VipsImage *image )
 {
 	return( image->Xsize );
 }
 
 int
-vips_image_get_height( VipsImage *image )
+vips_image_get_height( const VipsImage *image )
 {
 	return( image->Ysize );
 }
 
 int
-vips_image_get_bands( VipsImage *image )
+vips_image_get_bands( const VipsImage *image )
 {
 	return( image->Bands );
 }
 
 VipsBandFormat
-vips_image_get_format( VipsImage *image )
+vips_image_get_format( const VipsImage *image )
 {
 	return( image->BandFmt );
 }
 
 VipsCoding
-vips_image_get_coding( VipsImage *image )
+vips_image_get_coding( const VipsImage *image )
 {
 	return( image->Coding );
 }
 
 VipsInterpretation
-vips_image_get_interpretation( VipsImage *image )
+vips_image_get_interpretation( const VipsImage *image )
 {
 	return( image->Type );
 }
 
 double
-vips_image_get_xres( VipsImage *image )
+vips_image_get_xres( const VipsImage *image )
 {
 	return( image->Xres );
 }
 
 double
-vips_image_get_yres( VipsImage *image )
+vips_image_get_yres( const VipsImage *image )
 {
 	return( image->Yres );
 }
 
 int
-vips_image_get_xoffset( VipsImage *image )
+vips_image_get_xoffset( const VipsImage *image )
 {
 	return( image->Xoffset );
 }
 
 int
-vips_image_get_yoffset( VipsImage *image )
+vips_image_get_yoffset( const VipsImage *image )
 {
 	return( image->Yoffset );
 }
 
 const char *
-vips_image_get_filename( VipsImage *image )
+vips_image_get_filename( const VipsImage *image )
 {
 	return( image->filename );
 }
 
 const char *
-vips_image_get_mode( VipsImage *image )
+vips_image_get_mode( const VipsImage *image )
 {
 	return( image->mode );
 }
 
 /**
- * vips_image_get_data:
+ * vips_image_get_data: (skip)
  * @image: image to get data for
  *
  * Return a pointer to the image's pixel data, if possible. This can involve
@@ -445,9 +433,9 @@ vips_image_get_data( VipsImage *image )
  * @xsize: image width
  * @ysize: image height
  * @bands: image bands
- * @bandfmt: band format
+ * @format: band format
  * @coding: image coding
- * @type: image type
+ * @interpretation: image type
  * @xres: horizontal resolution, pixels per millimetre
  * @yres: vertical resolution, pixels per millimetre
  *
@@ -465,7 +453,7 @@ vips_image_init_fields( VipsImage *image,
 	int xsize, int ysize, int bands, 
 	VipsBandFormat format, VipsCoding coding, 
 	VipsInterpretation interpretation, 
-	float xres, float yres )
+	double xres, double yres )
 {
 	g_object_set( image,
 		"width", xsize,
@@ -494,8 +482,6 @@ meta_cp_field( VipsMeta *meta, VipsImage *dst )
 }
 #endif /*DEBUG*/
 
-	/* No way to return error here, sadly.
-	 */
 	(void) meta_new( dst, meta->field, &meta->value );
 
 #ifdef DEBUG
@@ -552,6 +538,10 @@ vips_image_copy_fields_array( VipsImage *out, VipsImage *in[] )
 
 	g_assert( in[0] );
 
+	/* Copy magic too, handy for knowing the original image's byte order.
+	 */
+	out->magic = in[0]->magic;
+
 	out->Xsize = in[0]->Xsize;
 	out->Ysize = in[0]->Ysize;
 	out->Bands = in[0]->Bands;
@@ -561,8 +551,8 @@ vips_image_copy_fields_array( VipsImage *out, VipsImage *in[] )
 	out->Coding = in[0]->Coding;
 	out->Xres = in[0]->Xres;
 	out->Yres = in[0]->Yres;
-	out->Xoffset = 0;
-	out->Yoffset = 0;
+	out->Xoffset = in[0]->Xoffset;
+	out->Yoffset = in[0]->Yoffset;
 
 	/* Count number of images.
 	 */
@@ -661,37 +651,26 @@ vips_image_copy_fields( VipsImage *out, VipsImage *in )
  *
  * g_value_init( &value, G_TYPE_INT );
  * g_value_set_int( &value, 42 );
- *
- * if( vips_image_set( image, field, &value ) ) {
- *   g_value_unset( &value );
- *   return( -1 );
- * }
+ * vips_image_set( image, field, &value );
  * g_value_unset( &value );
  *
  * return( 0 );
  * ]|
  *
  * See also: vips_image_get().
- *
- * Returns: 0 on success, -1 otherwise.
  */
-int
+void
 vips_image_set( VipsImage *image, const char *field, GValue *value )
 {
-	VipsMeta *meta;
-
 	g_assert( field );
 	g_assert( value );
 
 	meta_init( image );
-	if( !(meta = meta_new( image, field, value )) )
-		return( -1 );
+	(void) meta_new( image, field, value );
 
 #ifdef DEBUG
 	meta_sanity( image );
 #endif /*DEBUG*/
-
-	return( 0 );
 }
 
 /**
@@ -768,7 +747,7 @@ vips_image_get( VipsImage *image, const char *field, GValue *value_copy )
 		if( strcmp( field, double_field[i].field ) == 0 ) {
 			g_value_init( value_copy, G_TYPE_DOUBLE );
 			g_value_set_double( value_copy, 
-				G_STRUCT_MEMBER( float, image, 
+				G_STRUCT_MEMBER( double, image, 
 					double_field[i].offset ) );
 			return( 0 );
 		}
@@ -777,7 +756,7 @@ vips_image_get( VipsImage *image, const char *field, GValue *value_copy )
 		if( strcmp( field, old_double_field[i].field ) == 0 ) {
 			g_value_init( value_copy, G_TYPE_DOUBLE );
 			g_value_set_double( value_copy, 
-				G_STRUCT_MEMBER( float, image, 
+				G_STRUCT_MEMBER( double, image, 
 					old_double_field[i].offset ) );
 			return( 0 );
 		}
@@ -879,7 +858,7 @@ vips_image_map_fn( VipsMeta *meta, VipsImageMapFn fn, void *a )
 }
 
 /**
- * vips_image_map:
+ * vips_image_map: (skip)
  * @image: image to map over
  * @fn: function to call for each header field
  * @a: user data for function
@@ -936,339 +915,61 @@ vips_image_map( VipsImage *image, VipsImageMapFn fn, void *a )
 	return( NULL );
 }
 
-/* Save meta fields to the header. We have a new string type for header fields
- * to save to XML and define transform functions to go from our meta types to
- * this string type.
- */
-GType
-vips_save_string_get_type( void )
-{
-	static GType type = 0;
-
-	if( !type ) {
-		type = g_boxed_type_register_static( "vips_save_string",
-			(GBoxedCopyFunc) g_strdup, 
-			(GBoxedFreeFunc) g_free );
-	}
-
-	return( type );
-}
-
-/** 
- * vips_save_string_get:
- * @value: GValue to get from
+/**
+ * vips_image_set_area:
+ * @image: image to attach the metadata to
+ * @field: metadata name
+ * @free_fn: (scope async): free function for @data
+ * @data: pointer to area of memory
  *
- * Get the C string held internally by the GValue.
+ * Attaches @data as a metadata item on @image under the name @field. When
+ * VIPS no longer needs the metadata, it will be freed with @free_fn.
  *
- * Returns: The C string held by @value. This must not be freed.
- */
-const char *
-vips_save_string_get( const GValue *value )
-{
-	return( (char *) g_value_get_boxed( value ) );
-}
-
-/** 
- * vips_save_string_set:
- * @value: GValue to set
- * @str: C string to copy into the GValue
- *
- * Copies the C string into @value.
+ * See also: vips_image_get_double(), vips_image_set()
  */
 void
-vips_save_string_set( GValue *value, const char *str )
+vips_image_set_area( VipsImage *image, const char *field,
+		VipsCallbackFn free_fn, void *data )
 {
-	g_assert( G_VALUE_TYPE( value ) == VIPS_TYPE_SAVE_STRING );
+	GValue value = { 0 };
 
-	g_value_set_boxed( value, str );
-}
-
-/** 
- * vips_save_string_setf:
- * @value: GValue to set
- * @fmt: printf()-style format string
- * @Varargs: arguments to printf()-formatted @fmt
- *
- * Generates a string and copies it into @value.
- */
-void
-vips_save_string_setf( GValue *value, const char *fmt, ... )
-{
-	va_list ap;
-	char *str;
-
-	g_assert( G_VALUE_TYPE( value ) == VIPS_TYPE_SAVE_STRING );
-
-	va_start( ap, fmt );
-	str = g_strdup_vprintf( fmt, ap );
-	va_end( ap );
-	vips_save_string_set( value, str );
-	g_free( str );
-}
-
-/* Transform funcs for builtin types to SAVE_STRING.
- */
-static void
-transform_int_save_string( const GValue *src_value, GValue *dest_value )
-{
-	vips_save_string_setf( dest_value, "%d", g_value_get_int( src_value ) );
-}
-
-static void
-transform_save_string_int( const GValue *src_value, GValue *dest_value )
-{
-	g_value_set_int( dest_value, atoi( vips_save_string_get( src_value ) ) );
-}
-
-static void
-transform_double_save_string( const GValue *src_value, GValue *dest_value )
-{
-	char buf[G_ASCII_DTOSTR_BUF_SIZE];
-
-	/* Need to be locale independent.
-	 */
-	g_ascii_dtostr( buf, G_ASCII_DTOSTR_BUF_SIZE, 
-		g_value_get_double( src_value ) );
-	vips_save_string_set( dest_value, buf );
-}
-
-static void
-transform_save_string_double( const GValue *src_value, GValue *dest_value )
-{
-	g_value_set_double( dest_value, 
-		g_ascii_strtod( vips_save_string_get( src_value ), NULL ) );
-}
-
-/* A GType for a ref-counted area of memory.
- */
-typedef struct _Area {
-	int count;
-	size_t length;		/* 0 if not known */
-	void *data;
-	VipsCallbackFn free_fn;
-} Area;
-
-#ifdef DEBUG
-static int area_number = 0;
-#endif /*DEBUG*/
-
-/* An area of mem with a free func. (eg. \0-terminated string, or a struct).
- * Inital count == 1, so _unref() after attaching somewhere.
- */
-static Area *
-area_new( VipsCallbackFn free_fn, void *data )
-{
-	Area *area;
-
-	if( !(area = VIPS_NEW( NULL, Area )) )
-		return( NULL );
-	area->count = 1;
-	area->length = 0;
-	area->data = data;
-	area->free_fn = free_fn;
-
-#ifdef DEBUG
-	area_number += 1;
-	printf( "area_new: %p count = %d (%d in total)\n", 
-		area, area->count, area_number );
-#endif /*DEBUG*/
-
-	return( area );
-}
-
-/* An area of mem with a free func and a length (some sort of binary object,
- * like an ICC profile).
- */
-static Area *
-area_new_blob( VipsCallbackFn free_fn, void *blob, size_t blob_length )
-{
-	Area *area;
-
-	if( !(area = area_new( free_fn, blob )) )
-		return( NULL );
-	area->length = blob_length;
-
-	return( area );
-}
-
-static Area *
-area_copy( Area *area )
-{
-	g_assert( area->count >= 0 );
-
-	area->count += 1;
-
-#ifdef DEBUG
-	printf( "area_copy: %p count = %d\n", area, area->count );
-#endif /*DEBUG*/
-
-	return( area );
-}
-
-static void
-area_unref( Area *area )
-{
-	g_assert( area->count > 0 );
-
-	area->count -= 1;
-
-#ifdef DEBUG
-	printf( "area_unref: %p count = %d\n", area, area->count );
-#endif /*DEBUG*/
-
-	if( area->count == 0 && area->free_fn ) {
-		(void) area->free_fn( area->data, NULL );
-		area->free_fn = NULL;
-		vips_free( area );
-
-#ifdef DEBUG
-		area_number -= 1;
-		printf( "area_unref: free .. total = %d\n", area_number );
-#endif /*DEBUG*/
-	}
-}
-
-/* Transform an area to a G_TYPE_STRING.
- */
-static void
-transform_area_g_string( const GValue *src_value, GValue *dest_value )
-{
-	Area *area;
-	char buf[256];
-
-	area = g_value_get_boxed( src_value );
-	vips_snprintf( buf, 256, "VIPS_TYPE_AREA, count = %d, data = %p",
-		area->count, area->data );
-	g_value_set_string( dest_value, buf );
-}
-
-GType
-vips_area_get_type( void )
-{
-	static GType type = 0;
-
-	if( !type ) {
-		type = g_boxed_type_register_static( "vips_area",
-			(GBoxedCopyFunc) area_copy, 
-			(GBoxedFreeFunc) area_unref );
-		g_value_register_transform_func( 
-			type,
-			G_TYPE_STRING,
-			transform_area_g_string );
-	}
-
-	return( type );
-}
-
-/* Set value to be a ref-counted area of memory with a free function.
- */
-static int
-value_set_area( VipsCallbackFn free_fn, void *data, GValue *value )
-{
-	Area *area;
-
-	if( !(area = area_new( free_fn, data )) )
-		return( -1 );
-
-	g_value_init( value, VIPS_TYPE_AREA );
-	g_value_set_boxed( value, area );
-	area_unref( area );
-
-	return( 0 );
-}
-
-/* Don't touch count (area is static).
- */
-static void *
-value_get_area_data( const GValue *value )
-{
-	Area *area;
-
-	area = g_value_get_boxed( value );
-
-	return( area->data );
-}
-
-static size_t
-value_get_area_length( const GValue *value )
-{
-	Area *area;
-
-	area = g_value_get_boxed( value );
-
-	return( area->length );
-}
-
-/* Helpers for set/get. Write a value and destroy it.
- */
-static int
-meta_set_value( VipsImage *image, const char *field, GValue *value )
-{
-	if( vips_image_set( image, field, value ) ) {
-		g_value_unset( value );
-		return( -1 );
-	}
-	g_value_unset( value );
-
-	return( 0 );
+	vips_value_set_area( &value, free_fn, data );
+	vips_image_set( image, field, &value );
+	g_value_unset( &value );
 }
 
 static int
-meta_get_value( VipsImage *image, 
+meta_get_value( VipsImage *image,
 	const char *field, GType type, GValue *value_copy )
 {
 	if( vips_image_get( image, field, value_copy ) )
 		return( -1 );
 	if( G_VALUE_TYPE( value_copy ) != type ) {
-		vips_error( "VipsImage", 
+		vips_error( "VipsImage",
 			_( "field \"%s\" is of type %s, not %s" ),
-			field, 
+			field,
 			g_type_name( G_VALUE_TYPE( value_copy ) ),
 			g_type_name( type ) );
 		g_value_unset( value_copy );
+
 		return( -1 );
 	}
 
 	return( 0 );
 }
 
-/** 
- * vips_image_set_area:
- * @image: image to attach the metadata to
- * @field: metadata name
- * @free_fn: free function for @data
- * @data: pointer to area of memory
- *
- * Attaches @data as a metadata item on @image under the name @field. When VIPS
- * no longer needs the metadata, it will be freed with @free_fn.
- *
- * See also: vips_image_get_double(), vips_image_set()
- *
- * Returns: 0 on success, -1 otherwise.
- */
-int
-vips_image_set_area( VipsImage *image, const char *field, 
-	VipsCallbackFn free_fn, void *data )
-{
-	GValue value = { 0 };
-
-	value_set_area( free_fn, data, &value );
-
-	return( meta_set_value( image, field, &value ) );
-}
-
-/** 
+/**
  * vips_image_get_area:
  * @image: image to get the metadata from
  * @field: metadata name
  * @data: return metadata value
  *
  * Gets @data from @image under the name @field. A convenience
- * function over vips_image_get(). Use vips_image_get_typeof() to test for the 
- * existance
- * of a piece of metadata.
+ * function over vips_image_get(). Use vips_image_get_typeof() to test for
+ * the existance of a piece of metadata.
  *
- * See also: vips_image_set_area(), vips_image_get(), vips_image_get_typeof()
+ * See also: vips_image_set_area(), vips_image_get(),
+ * vips_image_get_typeof()
  *
  * Returns: 0 on success, -1 otherwise.
  */
@@ -1278,7 +979,7 @@ vips_image_get_area( VipsImage *image, const char *field, void **data )
 	GValue value_copy = { 0 };
 
 	if( !meta_get_value( image, field, VIPS_TYPE_AREA, &value_copy ) ) {
-		*data = value_get_area_data( &value_copy );
+		*data = vips_value_get_area( &value_copy, NULL );
 		g_value_unset( &value_copy );
 		return( 0 );
 	}
@@ -1286,255 +987,11 @@ vips_image_get_area( VipsImage *image, const char *field, void **data )
 	return( -1 );
 }
 
-/** 
- * vips_ref_string_get:
- * @value: GValue to get from
- *
- * Get the C string held internally by the GValue.
- *
- * Returns: The C string held by @value. This must not be freed.
- */
-const char *
-vips_ref_string_get( const GValue *value )
-{
-	return( value_get_area_data( value ) );
-}
-
-/** 
- * vips_ref_string_get_length:
- * @value: GValue to get from
- *
- * Gets the cached string length held internally by the refstring.
- *
- * Returns: The length of the string.
- */
-size_t
-vips_ref_string_get_length( const GValue *value )
-{
-	return( value_get_area_length( value ) );
-}
-
-/** 
- * vips_ref_string_set:
- * @value: GValue to set
- * @str: C string to copy into the GValue
- *
- * Copies the C string @str into @value. 
- *
- * vips_ref_string are immutable C strings that are copied between images by
- * copying reference-counted pointers, making the much more efficient than
- * regular GValue strings.
- *
- * Returns: 0 on success, -1 otherwise.
- */
-int
-vips_ref_string_set( GValue *value, const char *str )
-{
-	Area *area;
-	char *str_copy;
-
-	g_assert( G_VALUE_TYPE( value ) == VIPS_TYPE_REF_STRING );
-
-	if( !(str_copy = vips_strdup( NULL, str )) )
-		return( -1 );
-	if( !(area = area_new( (VipsCallbackFn) vips_free, str_copy )) ) {
-		vips_free( str_copy );
-		return( -1 );
-	}
-
-	/* Handy place to cache this.
-	 */
-	area->length = strlen( str );
-
-	g_value_set_boxed( value, area );
-	area_unref( area );
-
-	return( 0 );
-}
-
-/* Transform a refstring to a G_TYPE_STRING and back.
- */
-static void
-transform_ref_string_g_string( const GValue *src_value, GValue *dest_value )
-{
-	g_value_set_string( dest_value, vips_ref_string_get( src_value ) );
-}
-
-static void
-transform_g_string_ref_string( const GValue *src_value, GValue *dest_value )
-{
-	vips_ref_string_set( dest_value, g_value_get_string( src_value ) );
-}
-
-/* To a save string.
- */
-static void
-transform_ref_string_save_string( const GValue *src_value, GValue *dest_value )
-{
-	vips_save_string_setf( dest_value, "%s", vips_ref_string_get( src_value ) );
-}
-
-static void
-transform_save_string_ref_string( const GValue *src_value, GValue *dest_value )
-{
-	vips_ref_string_set( dest_value, vips_save_string_get( src_value ) );
-}
-
-GType
-vips_ref_string_get_type( void )
-{
-	static GType type = 0;
-
-	if( !type ) {
-		type = g_boxed_type_register_static( "vips_ref_string",
-			(GBoxedCopyFunc) area_copy, 
-			(GBoxedFreeFunc) area_unref );
-		g_value_register_transform_func( type, G_TYPE_STRING,
-			transform_ref_string_g_string );
-		g_value_register_transform_func( G_TYPE_STRING, type,
-			transform_g_string_ref_string );
-		g_value_register_transform_func( type, VIPS_TYPE_SAVE_STRING,
-			transform_ref_string_save_string );
-		g_value_register_transform_func( VIPS_TYPE_SAVE_STRING, type,
-			transform_save_string_ref_string );
-	}
-
-	return( type );
-}
-
-/** 
- * vips_blob_get:
- * @value: GValue to get from
- * @length: return the blob length here, optionally
- *
- * Get the address of the blob (binary large object) being held in @value and
- * optionally return its length in @length.
- *
- * See also: vips_blob_set().
- *
- * Returns: The blob address.
- */
-void *
-vips_blob_get( const GValue *value, size_t *length )
-{
-	Area *area;
-
-	/* Can't check value type, because we may get called from
-	 * vips_blob_get_type().
-	 */
-
-	area = g_value_get_boxed( value );
-	if( length )
-		*length = area->length;
-
-	return( area->data );
-}
-
-/* Transform a blob to a G_TYPE_STRING.
- */
-static void
-transform_blob_g_string( const GValue *src_value, GValue *dest_value )
-{
-	void *blob;
-	size_t blob_length;
-	char buf[256];
-
-	blob = vips_blob_get( src_value, &blob_length );
-	vips_snprintf( buf, 256, "VIPS_TYPE_BLOB, data = %p, length = %zd",
-		blob, blob_length );
-	g_value_set_string( dest_value, buf );
-}
-
-/* Transform a blob to a save string and back.
- */
-static void
-transform_blob_save_string( const GValue *src_value, GValue *dest_value )
-{
-	void *blob;
-	size_t blob_length;
-	char *b64;
-
-	blob = vips_blob_get( src_value, &blob_length );
-	if( (b64 = vips__b64_encode( blob, blob_length )) ) {
-		vips_save_string_set( dest_value, b64 );
-		vips_free( b64 );
-	}
-}
-
-static void
-transform_save_string_blob( const GValue *src_value, GValue *dest_value )
-{
-	const char *b64;
-	void *blob;
-	size_t blob_length;
-
-	b64 = vips_save_string_get( src_value );
-	if( (blob = vips__b64_decode( b64, &blob_length )) )
-		vips_blob_set( dest_value, 
-			(VipsCallbackFn) vips_free, blob, blob_length );
-}
-
-GType
-vips_blob_get_type( void )
-{
-	static GType type = 0;
-
-	if( !type ) {
-		type = g_boxed_type_register_static( "vips_blob",
-			(GBoxedCopyFunc) area_copy, 
-			(GBoxedFreeFunc) area_unref );
-		g_value_register_transform_func( type, G_TYPE_STRING,
-			transform_blob_g_string );
-		g_value_register_transform_func( type, VIPS_TYPE_SAVE_STRING,
-			transform_blob_save_string );
-		g_value_register_transform_func( VIPS_TYPE_SAVE_STRING, type,
-			transform_save_string_blob );
-	}
-
-	return( type );
-}
-
-/** 
- * vips_blob_set:
- * @value: GValue to set
- * @free_fn: free function for @data
- * @data: pointer to area of memory
- * @length: length of memory area
- *
- * Sets @value to hold a pointer to @blob. When @value is freed, @blob will be
- * freed with @free_fn. @value also holds a note of the length of the memory
- * area.
- *
- * blobs are things like ICC profiles or EXIF data. They are relocatable, and
- * are saved to VIPS files for you coded as base64 inside the XML. They are
- * copied by copying reference-counted pointers.
- *
- * See also: vips_blob_get()
- *
- * Returns: 0 on success, -1 otherwise.
- */
-int
-vips_blob_set( GValue *value, 
-	VipsCallbackFn free_fn, void *data, size_t length ) 
-{
-	Area *area;
-
-	g_assert( G_VALUE_TYPE( value ) == VIPS_TYPE_BLOB );
-
-	if( !(area = area_new_blob( free_fn, data, length )) )
-		return( -1 );
-
-	g_value_set_boxed( value, area );
-	area_unref( area );
-
-	return( 0 );
-}
-
 /** 
  * vips_image_set_blob:
  * @image: image to attach the metadata to
  * @field: metadata name
- * @free_fn: free function for @data
+ * @free_fn: (scope async): free function for @data
  * @data: pointer to area of memory
  * @length: length of memory area
  *
@@ -1543,23 +1000,21 @@ vips_blob_set( GValue *value,
  * function over vips_image_set() using an vips_blob.
  *
  * See also: vips_image_get_blob(), vips_image_set().
- *
- * Returns: 0 on success, -1 otherwise.
  */
-int
+void
 vips_image_set_blob( VipsImage *image, const char *field, 
 	VipsCallbackFn free_fn, void *data, size_t length )
 {
 	GValue value = { 0 };
 
 	g_value_init( &value, VIPS_TYPE_BLOB );
-	vips_blob_set( &value, free_fn, data, length );
-
-	return( meta_set_value( image, field, &value ) );
+	vips_value_set_blob( &value, free_fn, data, length );
+	vips_image_set( image, field, &value );
+	g_value_unset( &value );
 }
 
 /** 
- * vips_image_get_blob:
+ * vips_image_get_blob: (skip)
  * @image: image to get the metadata from
  * @field: metadata name
  * @data: pointer to area of memory
@@ -1582,7 +1037,7 @@ vips_image_get_blob( VipsImage *image, const char *field,
 	GValue value_copy = { 0 };
 
 	if( !meta_get_value( image, field, VIPS_TYPE_BLOB, &value_copy ) ) {
-		*data = vips_blob_get( &value_copy, length );
+		*data = vips_value_get_blob( &value_copy, length );
 		g_value_unset( &value_copy );
 		return( 0 );
 	}
@@ -1643,18 +1098,16 @@ vips_image_get_int( VipsImage *image, const char *field, int *out )
  * function over vips_image_set().
  *
  * See also: vips_image_get_int(), vips_image_set()
- *
- * Returns: 0 on success, -1 otherwise.
  */
-int
+void
 vips_image_set_int( VipsImage *image, const char *field, int i )
 {
 	GValue value = { 0 };
 
 	g_value_init( &value, G_TYPE_INT );
 	g_value_set_int( &value, i );
-
-	return( meta_set_value( image, field, &value ) );
+	vips_image_set( image, field, &value );
+	g_value_unset( &value );
 }
 
 /** 
@@ -1679,13 +1132,13 @@ vips_image_get_double( VipsImage *image, const char *field, double *out )
 
 	for( i = 0; i < VIPS_NUMBER( double_field ); i++ )
 		if( strcmp( field, double_field[i].field ) == 0 ) {
-			*out = G_STRUCT_MEMBER( float, image, 
+			*out = G_STRUCT_MEMBER( double, image, 
 				double_field[i].offset );
 			return( 0 );
 		}
 	for( i = 0; i < VIPS_NUMBER( old_double_field ); i++ )
 		if( strcmp( field, old_double_field[i].field ) == 0 ) {
-			*out = G_STRUCT_MEMBER( float, image, 
+			*out = G_STRUCT_MEMBER( double, image, 
 				old_double_field[i].offset );
 			return( 0 );
 		}
@@ -1712,18 +1165,16 @@ vips_image_get_double( VipsImage *image, const char *field, double *out )
  * function over vips_image_set().
  *
  * See also: vips_image_get_double(), vips_image_set()
- *
- * Returns: 0 on success, -1 otherwise.
  */
-int
+void
 vips_image_set_double( VipsImage *image, const char *field, double d )
 {
 	GValue value = { 0 };
 
 	g_value_init( &value, G_TYPE_DOUBLE );
 	g_value_set_double( &value, d );
-
-	return( meta_set_value( image, field, &value ) );
+	vips_image_set( image, field, &value );
+	g_value_unset( &value );
 }
 
 /** 
@@ -1746,7 +1197,7 @@ vips_image_get_string( VipsImage *image, const char *field, char **out )
 {
 	int i;
 	GValue value_copy = { 0 };
-	Area *area;
+	VipsArea *area;
 
 	for( i = 0; i < VIPS_NUMBER( string_field ); i++ )
 		if( strcmp( field, string_field[i].field ) == 0 ) {
@@ -1778,18 +1229,16 @@ vips_image_get_string( VipsImage *image, const char *field, char **out )
  * function over vips_image_set() using an vips_ref_string.
  *
  * See also: vips_image_get_double(), vips_image_set(), vips_ref_string
- *
- * Returns: 0 on success, -1 otherwise.
  */
-int
+void
 vips_image_set_string( VipsImage *image, const char *field, const char *str )
 {
 	GValue value = { 0 };
 
 	g_value_init( &value, VIPS_TYPE_REF_STRING );
-	vips_ref_string_set( &value, str );
-
-	return( meta_set_value( image, field, &value ) );
+	vips_value_set_ref_string( &value, str );
+	vips_image_set( image, field, &value );
+	g_value_unset( &value );
 }
 
 /** 
@@ -1825,7 +1274,7 @@ vips_image_get_as_string( VipsImage *image, const char *field, char **out )
 		g_value_init( &save_value, VIPS_TYPE_SAVE_STRING );
 		if( !g_value_transform( &value, &save_value ) ) 
 			return( -1 );
-		*out = g_strdup( vips_save_string_get( &save_value ) );
+		*out = g_strdup( vips_value_get_save_string( &save_value ) );
 		g_value_unset( &save_value );
 	}
 	else 
@@ -1900,7 +1349,7 @@ vips_image_history_printf( VipsImage *image, const char *fmt, ... )
 
 /**
  * vips_image_history_args:
- * @out: image to attach history line to
+ * @image: image to attach history line to
  * @name: program name
  * @argc: number of program arguments
  * @argv: program arguments
@@ -1958,27 +1407,3 @@ vips_image_get_history( VipsImage *image )
 
 	return( image->Hist ? image->Hist : "" );
 }
-
-/* Make the types we need for basic functioning. Called from init_world().
- */
-void
-vips__meta_init_types( void )
-{
-	(void) vips_save_string_get_type();
-	(void) vips_area_get_type();
-	(void) vips_ref_string_get_type();
-	(void) vips_blob_get_type();
-
-	/* Register transform functions to go from built-in saveable types to 
-	 * a save string. Transform functions for our own types are set 
-	 * during type creation. 
-	 */
-	g_value_register_transform_func( G_TYPE_INT, VIPS_TYPE_SAVE_STRING,
-		transform_int_save_string );
-	g_value_register_transform_func( VIPS_TYPE_SAVE_STRING, G_TYPE_INT,
-		transform_save_string_int );
-	g_value_register_transform_func( G_TYPE_DOUBLE, VIPS_TYPE_SAVE_STRING,
-		transform_double_save_string );
-	g_value_register_transform_func( VIPS_TYPE_SAVE_STRING, G_TYPE_DOUBLE,
-		transform_save_string_double );
-}
