@@ -80,10 +80,6 @@
 #include <vips/internal.h>
 #include <vips/vector.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Use in various small places where we need a mutex and it's not worth 
  * making a private one.
  */
@@ -93,6 +89,10 @@ GMutex *vips__global_lock = NULL;
  */
 static char *vips__argv0 = NULL;
 
+/* Leak check on exit.
+ */
+static int vips__leak = 0;
+
 /**
  * vips_get_argv0:
  *
@@ -149,12 +149,14 @@ vips_get_argv0( void )
  *   if( vips_init( argv[0] ) )
  *     vips_error_exit( "unable to start VIPS" );
  *
+ *   vips_shutdown();
+ *
  *   return( 0 );
  * }
  * ]|
  *
- * See also: vips_get_option_group(), vips_version(), vips_guess_prefix(),
- * vips_guess_libdir().
+ * See also: vips_shutdown(), vips_get_option_group(), vips_version(), 
+ * vips_guess_prefix(), vips_guess_libdir().
  *
  * Returns: 0 on success, -1 otherwise
  */
@@ -183,12 +185,12 @@ vips_init( const char *argv0 )
 		return( 0 );
 	started = TRUE;
 
-	VIPS_SETSTR( vips__argv0, argv0 );
-
 	/* Need gobject etc.
 	 */
 	g_type_init();
 
+	/* Older glibs need this.
+	 */
 #ifdef G_THREADS_ENABLED
 	if( !g_thread_supported() ) 
 		g_thread_init( NULL );
@@ -197,6 +199,8 @@ vips_init( const char *argv0 )
 	if( !vips__global_lock )
 		vips__global_lock = g_mutex_new();
 
+	VIPS_SETSTR( vips__argv0, argv0 );
+
 	prgname = g_path_get_basename( argv0 );
 	g_set_prgname( prgname );
 	g_free( prgname );
@@ -226,6 +230,8 @@ vips_init( const char *argv0 )
 	/* Start up packages.
 	 */
 	vips_arithmetic_operation_init();
+	vips_conversion_operation_init();
+	vips_foreign_operation_init();
 
 	/* Load up any plugins in the vips libdir. We don't error on failure,
 	 * it's too annoying to have VIPS refuse to start because of a broken
@@ -257,6 +263,13 @@ vips_init( const char *argv0 )
 	 */
 	vips_vector_init();
 
+	/* Register vips_shutdown(). This may well not get called and many
+	 * platforms don't support it anyway.
+	 */
+#ifdef HAVE_ATEXIT
+	atexit( vips_shutdown );
+#endif /*HAVE_ATEXIT*/
+
 	done = TRUE;
 
 	return( 0 );
@@ -275,6 +288,62 @@ vips_check_init( void )
 		vips_error_clear();
 }
 
+static void
+vips_leak( void ) 
+{
+	char txt[1024];
+	VipsBuf buf = VIPS_BUF_STATIC( txt );
+
+	vips_object_print_all();
+
+	if( vips_tracked_get_allocs() || 
+		vips_tracked_get_mem() ||
+		vips_tracked_get_files() ) {
+		vips_buf_appendf( &buf, "memory: %d allocations, %zd bytes\n",
+			vips_tracked_get_allocs(), vips_tracked_get_mem() );
+		vips_buf_appendf( &buf, "\nfiles: %d open\n",
+			vips_tracked_get_files() );
+	}
+
+	vips_buf_appendf( &buf, "memory: high-water mark " );
+	vips_buf_append_size( &buf, vips_tracked_get_mem_highwater() );
+	vips_buf_appends( &buf, "\n" );
+
+	fprintf( stderr, "%s", vips_buf_all( &buf ) );
+}
+
+/**
+ * vips_shutdown:
+ *
+ * Call this to drop caches and close plugins. Run with "--vips-leak" to do 
+ * a leak check too. May be called many times.
+ */
+void
+vips_shutdown( void )
+{
+#ifdef DEBUG
+	printf( "vips_shutdown:\n" );
+#endif /*DEBUG*/
+
+	vips_cache_drop_all();
+	im_close_plugins();
+
+	/* In dev releases, always show leaks. But not more than once, it's
+	 * annoying.
+	 */
+#ifndef DEBUG_LEAK
+	if( vips__leak ) 
+#endif /*DEBUG_LEAK*/
+	{
+		static gboolean done = FALSE;
+
+		if( !done ) 
+			vips_leak();
+
+		done = TRUE;
+	}
+}
+
 const char *
 vips__gettext( const char *msgid )
 {
@@ -292,11 +361,14 @@ vips__ngettext( const char *msgid, const char *plural, unsigned long int n )
 }
 
 static GOptionEntry option_entries[] = {
-	{ "vips-concurrency", 'c', 0, G_OPTION_ARG_INT, &vips__concurrency, 
+	{ "vips-concurrency", 'c', 0, 
+		G_OPTION_ARG_INT, &vips__concurrency, 
 		N_( "evaluate with N concurrent threads" ), "N" },
-	{ "vips-tile-width", 'w', 0, G_OPTION_ARG_INT, &vips__tile_width, 
+	{ "vips-tile-width", 'w', 0, 
+		G_OPTION_ARG_INT, &vips__tile_width, 
 		N_( "set tile width to N (DEBUG)" ), "N" },
-	{ "vips-tile-height", 'h', 0, G_OPTION_ARG_INT, &vips__tile_height, 
+	{ "vips-tile-height", 'h', 0, 
+		G_OPTION_ARG_INT, &vips__tile_height, 
 		N_( "set tile height to N (DEBUG)" ), "N" },
 	{ "vips-thinstrip-height", 't', 0, 
 		G_OPTION_ARG_INT, &vips__thinstrip_height, 
@@ -304,19 +376,38 @@ static GOptionEntry option_entries[] = {
 	{ "vips-fatstrip-height", 'f', 0, 
 		G_OPTION_ARG_INT, &vips__fatstrip_height, 
 		N_( "set fatstrip height to N (DEBUG)" ), "N" },
-	{ "vips-progress", 'p', 0, G_OPTION_ARG_NONE, &vips__progress, 
+	{ "vips-progress", 'p', 0, 
+		G_OPTION_ARG_NONE, &vips__progress, 
 		N_( "show progress feedback" ), NULL },
-	{ "vips-disc-threshold", 'd', 0, G_OPTION_ARG_STRING, 
-		&vips__disc_threshold, 
-		N_( "image size above which to decompress to disc" ), NULL },
-	{ "vips-novector", 't', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, 
-			&vips__vector_enabled, 
+	{ "vips-leak", 'l', 0, 
+		G_OPTION_ARG_NONE, &vips__leak, 
+		N_( "leak-check on exit" ), NULL },
+	{ "vips-disc-threshold", 'd', 0, 
+		G_OPTION_ARG_STRING, &vips__disc_threshold, 
+		N_( "images larger than N are decompressed to disc" ), "N" },
+	{ "vips-novector", 't', G_OPTION_FLAG_REVERSE, 
+		G_OPTION_ARG_NONE, &vips__vector_enabled, 
 		N_( "disable vectorised versions of operations" ), NULL },
+	{ "vips-cache-max", 'x', 0, 
+		G_OPTION_ARG_STRING, &vips__cache_max, 
+		N_( "cache at most N operations" ), "N" },
+	{ "vips-cache-max-memory", 'm', 0, 
+		G_OPTION_ARG_STRING, &vips__cache_max_mem, 
+		N_( "cache at most N bytes in memory" ), "N" },
+	{ "vips-cache-max-files", 'l', 0, 
+		G_OPTION_ARG_STRING, &vips__cache_max_files, 
+		N_( "allow at most N open files" ), "N" },
+	{ "vips-cache-trace", 'c', 0, 
+		G_OPTION_ARG_NONE, &vips__cache_trace, 
+		N_( "trace operation cache" ), NULL },
+	{ "vips-cache-dump", 'r', 0, 
+		G_OPTION_ARG_NONE, &vips__cache_dump, 
+		N_( "dump operation cache on exit" ), NULL },
 	{ NULL }
 };
 
 /**
- * vips_get_option_group:
+ * vips_get_option_group: (skip)
  *
  * vips_get_option_group()  returns  a GOptionGroup containing various VIPS
  * command-line options. It  can  be  used  with  GOption  to  help
@@ -667,4 +758,3 @@ vips_guess_libdir( const char *argv0, const char *env_name )
 
 	return( libdir );
 }
-
