@@ -46,10 +46,6 @@
 
 #include <gobject/gvaluecollector.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Abstract base class for operations.
  */
 
@@ -78,46 +74,87 @@ typedef struct {
 	gboolean required;	/* show required args or optional */
 	gboolean oftype;	/* "is of type" message */
 	int n;			/* Arg number */
-} VipsOperationPrint;
+} VipsOperationClassUsage;
 
 static void *
-vips_operation_print_arg( VipsObject *object, GParamSpec *pspec,
-	VipsArgumentClass *argument_class,
-	VipsArgumentInstance *argument_instance,
-	VipsBuf *buf, VipsOperationPrint *print )
+vips_operation_class_usage_arg( VipsObjectClass *object_class, 
+	GParamSpec *pspec, VipsArgumentClass *argument_class,
+	VipsBuf *buf, VipsOperationClassUsage *usage )
 {
 	/* Only show construct args ... others are internal.
 	 */
-	if( print->required == 
+	if( usage->required == 
 		((argument_class->flags & VIPS_ARGUMENT_REQUIRED) != 0) &&
 		(argument_class->flags & VIPS_ARGUMENT_CONSTRUCT) ) {
-		if( print->message && print->n == 0 ) 
-			vips_buf_appendf( buf, "%s\n", print->message );
+		if( usage->message && usage->n == 0 ) 
+			vips_buf_appendf( buf, "%s\n", usage->message );
 
-		if( print->oftype ) 
-			vips_buf_appendf( buf, "   %s :: %s (%s)\n",
+		if( usage->oftype ) {
+			vips_buf_appendf( buf, "   %-12s - %s, %s %s\n",
 				g_param_spec_get_name( pspec ), 
-				g_type_name( 
-					G_PARAM_SPEC_VALUE_TYPE( pspec ) ),
+				g_param_spec_get_blurb( pspec ), 
 				(argument_class->flags & VIPS_ARGUMENT_INPUT) ?
-					_( "input" ) : _( "output" ) );
+					_( "input" ) : _( "output" ),
+				g_type_name( 
+					G_PARAM_SPEC_VALUE_TYPE( pspec ) ) );
+		}
 		else {
-			if( print->n > 0 )
+			if( usage->n > 0 )
 				vips_buf_appends( buf, " " );
 			vips_buf_appends( buf, g_param_spec_get_name( pspec ) );
 		}
 
-		print->n += 1;
+		usage->n += 1;
 	}
 
 	return( NULL );
 }
 
-#ifdef VIPS_DEBUG
+static void
+vips_operation_usage( VipsOperationClass *class, VipsBuf *buf )
+{
+	VipsObjectClass *object_class = VIPS_OBJECT_CLASS( class );
+
+	VipsOperationClassUsage usage;
+
+	/* First pass through args: show the required names.
+	 */
+	vips_buf_appendf( buf, "   %s ", object_class->nickname );
+	usage.message = NULL;
+	usage.required = TRUE;
+	usage.oftype = FALSE;
+	usage.n = 0;
+	vips_argument_class_map( object_class,
+		(VipsArgumentClassMapFn) vips_operation_class_usage_arg, 
+			buf, &usage );
+	vips_buf_appends( buf, "\n" );
+
+	/* Show required types.
+	 */
+	usage.message = "where:";
+	usage.required = TRUE;
+	usage.oftype = TRUE;
+	usage.n = 0;
+	vips_argument_class_map( object_class,
+		(VipsArgumentClassMapFn) vips_operation_class_usage_arg, 
+			buf, &usage );
+
+	/* Show optional args.
+	 */
+	usage.message = "optional arguments:";
+	usage.required = FALSE;
+	usage.oftype = TRUE;
+	usage.n = 0;
+	vips_argument_class_map( object_class,
+		(VipsArgumentClassMapFn) vips_operation_class_usage_arg, 
+			buf, &usage );
+}
+
 static void *
 vips_operation_call_argument( VipsObject *object, GParamSpec *pspec,
 	VipsArgumentClass *argument_class,
-	VipsArgumentInstance *argument_instance )
+	VipsArgumentInstance *argument_instance,
+	void *a, void *b )
 {
 	VipsArgument *argument = (VipsArgument *) argument_class;
 
@@ -136,58 +173,64 @@ vips_operation_call_argument( VipsObject *object, GParamSpec *pspec,
 
 	return( NULL );
 }
-#endif /*VIPS_DEBUG*/
 
 static void
-vips_operation_print( VipsObject *object, VipsBuf *buf )
+vips_operation_dump( VipsObject *object, VipsBuf *buf )
 {
 	VipsOperation *operation = VIPS_OPERATION( object );
 	VipsObjectClass *object_class = VIPS_OBJECT_GET_CLASS( object );
-	VipsOperationPrint print;
 
-#ifdef VIPS_DEBUG
 	printf( "%s args:\n", object_class->nickname );
 	vips_argument_map( VIPS_OBJECT( operation ),
-		(VipsArgumentMapFn) vips_operation_call_argument, NULL, NULL );
-#endif /*VIPS_DEBUG*/
+		vips_operation_call_argument, NULL, NULL );
 
-	/* First pass through args: show the required names.
-	 */
-	vips_buf_appendf( buf, "   %s ", object_class->nickname );
-	print.message = NULL;
-	print.required = TRUE;
-	print.oftype = FALSE;
-	print.n = 0;
-	vips_argument_map( VIPS_OBJECT( operation ),
-		(VipsArgumentMapFn) vips_operation_print_arg, buf, &print );
-	vips_buf_appends( buf, "\n" );
-
-	/* Show required types.
-	 */
-	print.message = "where:";
-	print.required = TRUE;
-	print.oftype = TRUE;
-	print.n = 0;
-	vips_argument_map( VIPS_OBJECT( operation ),
-		(VipsArgumentMapFn) vips_operation_print_arg, buf, &print );
-
-	/* Show optional args.
-	 */
-	print.message = "optional arguments:";
-	print.required = FALSE;
-	print.oftype = TRUE;
-	print.n = 0;
-	vips_argument_map( VIPS_OBJECT( operation ),
-		(VipsArgumentMapFn) vips_operation_print_arg, buf, &print );
+	VIPS_OBJECT_CLASS( vips_operation_parent_class )->dump( object, buf );
 }
 
-static int
-vips_operation_build( VipsObject *object )
+static void *
+vips_operation_vips_operation_print_summary_arg( VipsObject *object, 
+	GParamSpec *pspec,
+	VipsArgumentClass *argument_class,
+	VipsArgumentInstance *argument_instance,
+	void *a, void *b )
 {
-	if( VIPS_OBJECT_CLASS( vips_operation_parent_class )->build( object ) )
-		return( -1 );
+	VipsBuf *buf = (VipsBuf *) a;
 
-	return( 0 );
+	/* Just assigned required input construct args
+	 */
+	if( (argument_class->flags & VIPS_ARGUMENT_REQUIRED) &&
+		(argument_class->flags & VIPS_ARGUMENT_CONSTRUCT) &&
+		(argument_class->flags & VIPS_ARGUMENT_INPUT) && 
+		argument_instance->assigned ) {
+		const char *name = g_param_spec_get_name( pspec );
+		GType type = G_PARAM_SPEC_VALUE_TYPE( pspec );
+
+		GValue gvalue = { 0, };
+		char *str;
+
+		g_value_init( &gvalue, type );
+		g_object_get_property( G_OBJECT( object ), name, &gvalue ); 
+		str = g_strdup_value_contents( &gvalue );
+		vips_buf_appendf( buf, " %s", str );
+		g_free( str );
+		g_value_unset( &gvalue ); 
+	}
+
+	return( NULL );
+}
+
+static void
+vips_operation_summary( VipsObject *object, VipsBuf *buf )
+{
+	VipsOperation *operation = VIPS_OPERATION( object );
+	VipsObjectClass *object_class = VIPS_OBJECT_GET_CLASS( object );
+
+	vips_buf_appendf( buf, "%s", object_class->nickname ); 
+	vips_argument_map( VIPS_OBJECT( operation ),
+		vips_operation_vips_operation_print_summary_arg, buf, NULL );
+
+	VIPS_OBJECT_CLASS( vips_operation_parent_class )->
+		summary( object, buf );
 }
 
 static void
@@ -200,9 +243,11 @@ vips_operation_class_init( VipsOperationClass *class )
 	gobject_class->dispose = vips_operation_dispose;
 
 	vobject_class->nickname = "operation";
-	vobject_class->description = _( "VIPS operations" );
-	vobject_class->print = vips_operation_print;
-	vobject_class->build = vips_operation_build;
+	vobject_class->description = _( "operations" );
+	vobject_class->summary = vips_operation_summary;
+	vobject_class->dump = vips_operation_dump;
+
+	class->usage = vips_operation_usage;
 }
 
 static void
@@ -212,6 +257,23 @@ vips_operation_init( VipsOperation *operation )
 	 */
 }
 
+/**
+ * vips_operation_class_print_usage: (skip)
+ * @operation_class: class to print usage for
+ *
+ * Print a usage message for the operation to stdout.
+ */
+void
+vips_operation_class_print_usage( VipsOperationClass *operation_class )
+{
+	char str[2048];
+	VipsBuf buf = VIPS_BUF_STATIC( str );
+
+	operation_class->usage( operation_class, &buf );
+	printf( "%s", _( "usage:" ) );
+	printf( "\n%s", vips_buf_all( &buf ) );
+}
+
 VipsOperation *
 vips_operation_new( const char *name )
 {
@@ -220,8 +282,11 @@ vips_operation_new( const char *name )
 
 	vips_check_init();
 
-	if( !(type = vips_type_find( "VipsOperation", name )) )
+	if( !(type = vips_type_find( "VipsOperation", name )) ) {
+		vips_error( "VipsOperation", 
+			_( "class \"%s\" not found" ), name );
 		return( NULL );
+	}
 	operation = VIPS_OPERATION( g_object_new( type, NULL ) );
 
 	VIPS_DEBUG_MSG( "vips_operation_new: %s (%p)\n", name, operation );
@@ -229,9 +294,20 @@ vips_operation_new( const char *name )
 	return( operation );
 }
 
+/* Some systems do not have va_copy() ... this might work (it does on MSVC),
+ * apparently.
+ *
+ * FIXME ... this should be in configure.in
+ */
+#ifndef va_copy
+#define va_copy(d,s) ((d) = (s))
+#endif
+
 static int
 vips_operation_set_valist_required( VipsOperation *operation, va_list ap )
 {
+	VIPS_DEBUG_MSG( "vips_operation_set_valist_required:\n" );
+
 	/* Set required input arguments. Can't use vips_argument_map here 
 	 * :-( because passing va_list by reference is not portable. 
 	 */
@@ -240,30 +316,8 @@ vips_operation_set_valist_required( VipsOperation *operation, va_list ap )
 
 		g_assert( argument_instance );
 
-		if( (argument_class->flags & VIPS_ARGUMENT_REQUIRED) &&
-			(argument_class->flags & VIPS_ARGUMENT_INPUT) &&
-			!argument_instance->assigned ) {
-			GValue value = { 0 };
-			char *msg = NULL;
-
-			/* It'd be nice to use G_VALUE_COLLECT_INIT(), but
-			 * that's only available in very recent glib.
-			 */
-			g_value_init( &value, 
-				G_PARAM_SPEC_VALUE_TYPE( pspec ) );
-			G_VALUE_COLLECT( &value, ap, 0, &msg );
-
-			if( msg ) {
-				VipsObjectClass *class = 
-					VIPS_OBJECT_GET_CLASS( operation ); 
-
-				vips_error( class->description, 
-					"%s", _( msg ) );
-				g_value_unset( &value );
-				g_free( msg );
-
-				return( -1 );
-			}
+		if( (argument_class->flags & VIPS_ARGUMENT_REQUIRED) ) {
+			VIPS_ARGUMENT_COLLECT_SET( pspec, argument_class, ap );
 
 #ifdef VIPS_DEBUG
 			{
@@ -278,71 +332,70 @@ vips_operation_set_valist_required( VipsOperation *operation, va_list ap )
 
 			g_object_set_property( G_OBJECT( operation ),
 				g_param_spec_get_name( pspec ), &value );
-			g_value_unset( &value );
-		}
-		else if( (argument_class->flags & VIPS_ARGUMENT_REQUIRED) &&
-			(argument_class->flags & VIPS_ARGUMENT_OUTPUT) &&
-			!argument_instance->assigned ) {
-			void *arg __attribute((unused));
 
-			/* Output args are all pointers to places to write 
-			 * results. Skip here, we use these during the output 
-			 * phase.
-			 */
-			arg = va_arg( ap, void * );
+			VIPS_ARGUMENT_COLLECT_GET( pspec, argument_class, ap );
 
 #ifdef VIPS_DEBUG
 			printf( "\tskipping arg %p for %s\n", 
 				arg, g_param_spec_get_name( pspec ) );
 #endif /*VIPS_DEBUG */
+
+			VIPS_ARGUMENT_COLLECT_END
 		}
 	} VIPS_ARGUMENT_FOR_ALL_END
 
 	return( 0 );
 }
 
-/* See comments on calls below.
 static int
 vips_operation_set_valist_optional( VipsOperation *operation, va_list ap )
 {
-	char *first_property_name;
+	char *name;
 
-	first_property_name = va_arg( ap, char * );
-	g_object_set_valist( G_OBJECT (operation), first_property_name, ap );
+	VIPS_DEBUG_MSG( "vips_operation_set_valist_optional:\n" );
+
+	name = va_arg( ap, char * );
+
+	while( name ) {
+		GParamSpec *pspec;
+		VipsArgumentClass *argument_class;
+		VipsArgumentInstance *argument_instance;
+
+		VIPS_DEBUG_MSG( "\tname = '%s' (%p)\n", name, name );
+
+		if( vips_object_get_argument( VIPS_OBJECT( operation ), name,
+			&pspec, &argument_class, &argument_instance ) )
+			return( -1 );
+
+		VIPS_ARGUMENT_COLLECT_SET( pspec, argument_class, ap );
+
+		g_object_set_property( G_OBJECT( operation ), 
+			name, &value );
+
+		VIPS_ARGUMENT_COLLECT_GET( pspec, argument_class, ap );
+
+		VIPS_ARGUMENT_COLLECT_END
+
+		name = va_arg( ap, char * );
+	}
 
 	return( 0 );
 }
- */
 
-static void
-vips_operation_get_valist( VipsOperation *operation, va_list ap )
+static int
+vips_operation_get_valist_required( VipsOperation *operation, va_list ap )
 {
+	VIPS_DEBUG_MSG( "vips_operation_get_valist_required:\n" );
+
 	/* Extract output arguments. Can't use vips_argument_map here 
 	 * :-( because passing va_list by reference is not portable. 
 	 */
 	VIPS_ARGUMENT_FOR_ALL( operation, 
 		pspec, argument_class, argument_instance ) {
-		if( (argument_class->flags & VIPS_ARGUMENT_REQUIRED) &&
-			(argument_class->flags & VIPS_ARGUMENT_INPUT) ) {
-			GValue value = { 0 };
-			char *msg = NULL;
+		if( (argument_class->flags & VIPS_ARGUMENT_REQUIRED) ) {
+			VIPS_ARGUMENT_COLLECT_SET( pspec, argument_class, ap );
 
-			/* Collect the arg from valist to eat it up, but don't
-			 * do anything with it.
-			 *
-			 * It'd be nice to use G_VALUE_COLLECT_INIT(), but
-			 * that's only available in very recent glib.
-			 */
-			g_value_init( &value, 
-				G_PARAM_SPEC_VALUE_TYPE( pspec ) );
-			G_VALUE_COLLECT( &value, ap, 0, &msg );
-			g_value_unset( &value );
-		}
-		else if( (argument_class->flags & VIPS_ARGUMENT_REQUIRED) &&
-			(argument_class->flags & VIPS_ARGUMENT_OUTPUT) ) {
-			void **arg;
-
-			arg = va_arg( ap, void ** );
+			VIPS_ARGUMENT_COLLECT_GET( pspec, argument_class, ap );
 
 			if( !argument_instance->assigned ) 
 				continue;
@@ -365,8 +418,113 @@ vips_operation_get_valist( VipsOperation *operation, va_list ap )
 				object = *((GObject **) arg);
 				g_object_unref( object ); 
 			}
+
+			VIPS_ARGUMENT_COLLECT_END
 		}
 	} VIPS_ARGUMENT_FOR_ALL_END
+
+	return( 0 );
+}
+
+static int
+vips_operation_get_valist_optional( VipsOperation *operation, va_list ap )
+{
+	char *name;
+
+	VIPS_DEBUG_MSG( "vips_operation_get_valist_optional:\n" );
+
+	name = va_arg( ap, char * );
+
+	while( name ) {
+		GParamSpec *pspec;
+		VipsArgumentClass *argument_class;
+		VipsArgumentInstance *argument_instance;
+
+		VIPS_DEBUG_MSG( "\tname = '%s' (%p)\n", name, name );
+
+		if( vips_object_get_argument( VIPS_OBJECT( operation ), name,
+			&pspec, &argument_class, &argument_instance ) )
+			return( -1 );
+
+		VIPS_ARGUMENT_COLLECT_SET( pspec, argument_class, ap );
+
+		/* We must collect input args as we walk the name/value list,
+		 * but we don't do anything with them.
+		 */
+
+		VIPS_ARGUMENT_COLLECT_GET( pspec, argument_class, ap );
+
+		/* Here's an output arg.
+		 */
+
+#ifdef VIPS_DEBUG
+		printf( "\twriting %s to %p\n", 
+			g_param_spec_get_name( pspec ), arg );
+#endif /*VIPS_DEBUG */
+
+		/* If the dest pointer is NULL, skip the read.
+		 */
+		if( arg ) {
+			g_object_get( G_OBJECT( operation ), 
+				g_param_spec_get_name( pspec ), arg, 
+				NULL );
+
+			/* If the pspec is an object, that will up 
+			 * the ref count. We want to hand over the 
+			 * ref, so we have to knock it down again.
+			 */
+			if( G_IS_PARAM_SPEC_OBJECT( pspec ) ) {
+				GObject *object;
+
+				object = *((GObject **) arg);
+				g_object_unref( object ); 
+			}
+		}
+
+		VIPS_ARGUMENT_COLLECT_END
+
+		name = va_arg( ap, char * );
+	}
+
+	return( 0 );
+}
+
+/* This can change operation to point at an old, cached one.
+ */
+static int
+vips_call_required_optional( VipsOperation **operation,
+	va_list required, va_list optional ) 
+{
+	int result;
+	va_list a;
+	va_list b;
+
+	/* We need to be able to walk required and optional twice. On x64 gcc,
+	 * vips_operation_set_valist_required() etc. will destructively alter
+	 * the passed-in va_list. We make a copy and walk that instead.
+	 */
+	va_copy( a, required );
+	va_copy( b, optional );
+	result = vips_operation_set_valist_required( *operation, a ) ||
+		vips_operation_set_valist_optional( *operation, b );
+	va_end( a );
+	va_end( b );
+
+	/* Build from cache.
+	 */
+	if( vips_cache_operation_buildp( operation ) )
+		return( -1 );
+
+	/* Walk args again, writing output.
+	 */
+	va_copy( a, required );
+	va_copy( b, optional );
+	result = vips_operation_get_valist_required( *operation, required ) ||
+		vips_operation_get_valist_optional( *operation, optional );
+	va_end( a );
+	va_end( b );
+
+	return( result );
 }
 
 int
@@ -374,8 +532,8 @@ vips_call( const char *operation_name, ... )
 {
 	VipsOperation *operation;
 	int result;
-	va_list ap;
-
+	va_list required;
+	va_list optional;
 
 	VIPS_DEBUG_MSG( "vips_call: starting for %s ...\n", operation_name );
 
@@ -387,35 +545,45 @@ vips_call( const char *operation_name, ... )
 	vips_object_print( VIPS_OBJECT( operation ) );
 #endif /*VIPS_DEBUG*/
 
-	/* This code is broken. Fixed correctly in the next version, but for
-	 * here just ignore optional arguments.
-	va_start( ap, operation_name );
-	result = vips_operation_set_valist_required( operation, ap )  ||
-		vips_operation_set_valist_optional( operation, ap ) ||
-		vips_object_build( VIPS_OBJECT( operation ) );
-	va_end( ap );
+	/* We have to break the va_list into separate required and optional 
+	 * components.
+	 *
+	 * Note the start, grab the required, then copy and reuse.
 	 */
+	va_start( required, operation_name );
 
-	va_start( ap, operation_name );
-	result = vips_operation_set_valist_required( operation, ap )  ||
-		vips_object_build( VIPS_OBJECT( operation ) );
-	va_end( ap );
+	va_copy( optional, required );
 
-	/* Build failed: junk args and back out.
+	VIPS_ARGUMENT_FOR_ALL( operation, 
+		pspec, argument_class, argument_instance ) {
+
+		g_assert( argument_instance );
+
+		if( (argument_class->flags & VIPS_ARGUMENT_REQUIRED) ) {
+			VIPS_ARGUMENT_COLLECT_SET( pspec, argument_class, 
+				optional );
+
+			VIPS_ARGUMENT_COLLECT_GET( pspec, argument_class, 
+				optional );
+
+			VIPS_ARGUMENT_COLLECT_END
+		}
+	} VIPS_ARGUMENT_FOR_ALL_END
+
+	result = vips_call_required_optional( &operation, required, optional );
+
+	va_end( required );
+	va_end( optional );
+
+	/* Failed: junk args and back out.
 	 */
 	if( result ) {
-		vips_argument_dispose_all( VIPS_OBJECT( operation ) );
+		vips_object_unref_outputs( VIPS_OBJECT( operation ) );
 		g_object_unref( operation );
 
 		return( -1 );
 	}
 
-	/* Walk args again writing output.
-	 */
-	va_start( ap, operation_name );
-	vips_operation_get_valist( operation, ap );
-	va_end( ap );
-
 	/* The operation we have built should now have been reffed by one of 
 	 * its arguments or have finished its work. Either way, we can unref.
 	 */
@@ -432,45 +600,24 @@ vips_call_split( const char *operation_name, va_list optional, ... )
 	va_list required;
 
 	VIPS_DEBUG_MSG( "vips_call_split: starting for %s ...\n", 
-			operation_name );
+		operation_name );
 
 	if( !(operation = vips_operation_new( operation_name ) ) )
 		return( -1 );
 
-#ifdef VIPS_DEBUG
-	VIPS_DEBUG_MSG( "where:\n" );
-	vips_object_print( VIPS_OBJECT( operation ) );
-#endif /*VIPS_DEBUG*/
-
-	/* This code is broken. Fixed correctly in the next version, but for
-	 * here just ignore optional arguments.
 	va_start( required, optional );
-	result = vips_operation_set_valist_required( operation, required ) ||
-		vips_operation_set_valist_optional( operation, optional ) ||
-		vips_object_build( VIPS_OBJECT( operation ) );
-	va_end( required );
-	 */
-
-	va_start( required, optional );
-	result = vips_operation_set_valist_required( operation, required ) ||
-		vips_object_build( VIPS_OBJECT( operation ) );
+	result = vips_call_required_optional( &operation, required, optional );
 	va_end( required );
 
 	/* Build failed: junk args and back out.
 	 */
 	if( result ) {
-		vips_argument_dispose_all( VIPS_OBJECT( operation ) );
+		vips_object_unref_outputs( VIPS_OBJECT( operation ) );
 		g_object_unref( operation );
 
 		return( -1 );
 	}
 
-	/* Walk args again writing output.
-	 */
-	va_start( required, optional );
-	vips_operation_get_valist( operation, required );
-	va_end( required );
-
 	/* The operation we have built should now have been reffed by one of 
 	 * its arguments or have finished its work. Either way, we can unref.
 	 */
@@ -480,45 +627,50 @@ vips_call_split( const char *operation_name, va_list optional, ... )
 }
 
 static void *
-vips_call_char_option( VipsObject *object,
+vips_call_find_pspec( VipsObject *object,
 	GParamSpec *pspec,
 	VipsArgumentClass *argument_class,
 	VipsArgumentInstance *argument_instance,
 	void *a, void *b )
 {
 	const char *name = (const char *) a;
-	const char *value = (const char *) b;
 
+	/* One char names we assume are "-x" style abbreviations, longer names
+	 * we match the whole string.
+	 */
 	if( !(argument_class->flags & VIPS_ARGUMENT_REQUIRED) &&
 		(argument_class->flags & VIPS_ARGUMENT_CONSTRUCT) &&
-		!argument_instance->assigned &&
-		g_param_spec_get_name( pspec )[0] == name[0] ) 
-		if( vips_object_set_argument_from_string( object, 
-			g_param_spec_get_name( pspec ), value ) )
-			return( object );
+		!argument_instance->assigned ) 
+		if( (strlen( name ) == 1 && 
+			g_param_spec_get_name( pspec )[0] == name[0]) ||
+			strcmp( g_param_spec_get_name( pspec ), name  ) == 0 ) 
+			return( argument_instance );
 
 	return( NULL );
 }
 
-static void *
-vips_call_name_option( VipsObject *object,
-	GParamSpec *pspec,
-	VipsArgumentClass *argument_class,
-	VipsArgumentInstance *argument_instance,
-	void *a, void *b )
+/* Keep this stuff around for output args.
+ */
+typedef struct _VipsCallOptionOutput {
+	VipsArgumentInstance *argument_instance;
+	const char *value;
+} VipsCallOptionOutput;
+
+static void
+vips_call_option_output( VipsObject *object,
+	VipsCallOptionOutput *output )
 {
-	const char *name = (const char *) a;
-	const char *value = (const char *) b;
+	VipsArgumentInstance *argument_instance = output->argument_instance;
+	GParamSpec *pspec = ((VipsArgument *) argument_instance)->pspec;
 
-	if( !(argument_class->flags & VIPS_ARGUMENT_REQUIRED) &&
-		(argument_class->flags & VIPS_ARGUMENT_CONSTRUCT) &&
-		!argument_instance->assigned &&
-		strcmp( g_param_spec_get_name( pspec ), name  ) == 0 ) 
-		if( vips_object_set_argument_from_string( object, 
-			g_param_spec_get_name( pspec ), value ) )
-			return( object );
+	if( vips_object_get_argument_to_string( object, 
+		g_param_spec_get_name( pspec ), output->value ) ) {
+		/* FIXME .. Hmm what can we do here? If an arg is image
+		 * output, for example, we will lose the error.
+		 */
+	}
 
-	return( NULL );
+	g_free( output );
 }
 
 static gboolean
@@ -527,6 +679,9 @@ vips_call_options_set( const gchar *option_name, const gchar *value,
 {
 	VipsOperation *operation = (VipsOperation *) data;
 	const char *name;
+	VipsArgumentInstance *argument_instance;
+	VipsArgumentClass *argument_class;
+	GParamSpec *pspec;
 
 	VIPS_DEBUG_MSG( "vips_call_options_set: %s = %s\n", 
 		option_name, value );
@@ -536,24 +691,55 @@ vips_call_options_set( const gchar *option_name, const gchar *value,
 	for( name = option_name; *name == '-'; name++ )
 		;
 
-	/* If this is a single-character name, find the first unset pspec with
-	 * that initial. Otherwise, search for a spec of that nmae.
-	 */
-	if( strlen( name ) == 1 ) {
-		if( vips_argument_map( VIPS_OBJECT( operation ),
-			vips_call_char_option, 
-			(void *) name, (void *) value ) ) {
-			vips_error_g( error );
-			return( FALSE );
-		}
+	if( !(argument_instance = (VipsArgumentInstance *) 
+		vips_argument_map( 
+			VIPS_OBJECT( operation ),
+			vips_call_find_pspec, (void *) name, NULL )) ) {
+		vips_error( VIPS_OBJECT_GET_CLASS( operation )->nickname, 
+			_( "unknown argument '%s'" ), name );
+		return( FALSE );
 	}
-	else {
-		if( vips_argument_map( VIPS_OBJECT( operation ),
-			vips_call_name_option, 
-			(void *) name, (void *) value ) ) {
-			vips_error_g( error );
+	argument_class = argument_instance->argument_class;
+	pspec = ((VipsArgument *) argument_instance)->pspec;
+
+	if( (argument_class->flags & VIPS_ARGUMENT_INPUT) ) {
+		if( vips_object_set_argument_from_string( 
+			VIPS_OBJECT( operation ),
+			g_param_spec_get_name( pspec ), value ) )
 			return( FALSE );
-		}
+
+#ifdef VIPS_DEBUG
+{
+		GType type = G_PARAM_SPEC_VALUE_TYPE( pspec );
+		GValue gvalue = { 0, };
+		char *str;
+
+		g_value_init( &gvalue, type );
+		g_object_get_property( G_OBJECT( operation ), 
+			g_param_spec_get_name( pspec ), &gvalue ); 
+		str = g_strdup_value_contents( &gvalue );
+		VIPS_DEBUG_MSG( "\tGValue %s = %s\n", 
+			g_param_spec_get_name( pspec ), str );
+		g_free( str );
+		g_value_unset( &gvalue ); 
+}
+#endif /*VIPS_DEBUG*/
+	}
+	else if( (argument_class->flags & VIPS_ARGUMENT_OUTPUT) ) {
+		VipsCallOptionOutput *output;
+
+		/* We can't do output now, we have to attach a callback to do
+		 * the processing after the operation has run.
+		 *
+		 * FIXME ... something like posteval or postbuild might be
+		 * better for this?
+		 */
+		output = g_new( VipsCallOptionOutput, 1 );
+		output->argument_instance = argument_instance;
+		output->value = value;
+		g_signal_connect( operation, "preclose",
+			G_CALLBACK( vips_call_option_output ),
+			output );
 	}
 
 	return( TRUE );
@@ -571,27 +757,28 @@ vips_call_options_add( VipsObject *object,
 	if( !(argument_class->flags & VIPS_ARGUMENT_REQUIRED) &&
 		(argument_class->flags & VIPS_ARGUMENT_CONSTRUCT) &&
 		!argument_instance->assigned ) {
+		const char *name = g_param_spec_get_name( pspec );
+		gboolean needs_string = 
+			vips_object_get_argument_needs_string( object, name );
 		GOptionEntry entry[2];
 
-		entry[0].long_name = g_param_spec_get_name( pspec );
-		entry[0].short_name = g_param_spec_get_name( pspec )[0];
-		if( G_IS_PARAM_SPEC_BOOLEAN( pspec ) ) 
-			entry[0].flags = G_OPTION_FLAG_NO_ARG;
-		else
-			entry[0].flags = 0;
+		entry[0].long_name = name;
+		entry[0].short_name = name[0];
+		entry[0].flags = 0;
+		if( !needs_string ) 
+			entry[0].flags |= G_OPTION_FLAG_NO_ARG;
 		entry[0].arg = G_OPTION_ARG_CALLBACK;
 		entry[0].arg_data = (gpointer) vips_call_options_set;
 		entry[0].description = g_param_spec_get_blurb( pspec );
-		if( G_IS_PARAM_SPEC_BOOLEAN( pspec ) ) 
-			entry[0].arg_description = NULL;
-		else
+		if( needs_string ) 
 			entry[0].arg_description = 
 				g_type_name( G_PARAM_SPEC_VALUE_TYPE( pspec ) );
+		else
+			entry[0].arg_description = NULL;
 
 		entry[1].long_name = NULL;
 
-		VIPS_DEBUG_MSG( "vips_call_options_add: adding %s\n",
-			g_param_spec_get_name( pspec ) );
+		VIPS_DEBUG_MSG( "vips_call_options_add: adding %s\n", name );
 
 		g_option_group_add_entries( group, &entry[0] );
 	}
@@ -619,7 +806,7 @@ static const char *
 vips_call_get_arg( VipsCall *call, int i )
 {
 	if( i < 0 || i >= call->argc ) {
-		vips_error( VIPS_OBJECT( call->operation )->nickname, 
+		vips_error( VIPS_OBJECT_GET_CLASS( call->operation )->nickname, 
 			"%s", _( "too few arguments" ) );
 		return( NULL );
 	}
@@ -640,19 +827,21 @@ vips_call_argv_input( VipsObject *object,
 	 */
 	if( (argument_class->flags & VIPS_ARGUMENT_REQUIRED) &&
 		(argument_class->flags & VIPS_ARGUMENT_CONSTRUCT) ) {
+		const char *name = g_param_spec_get_name( pspec );
+
 		if( (argument_class->flags & VIPS_ARGUMENT_INPUT) ) {
 			const char *arg;
 
 			if( !(arg = vips_call_get_arg( call, call->i )) ||
 				vips_object_set_argument_from_string( object, 
-				g_param_spec_get_name( pspec ), arg ) ) 
+					name, arg ) ) 
 				return( pspec );
 
 			call->i += 1;
 		}
 		else if( (argument_class->flags & VIPS_ARGUMENT_OUTPUT) ) {
 			if( vips_object_get_argument_needs_string( object,
-				g_param_spec_get_name( pspec ) ) )
+				name ) )
 				call->i += 1;
 		}
 	}
@@ -676,11 +865,12 @@ vips_call_argv_output( VipsObject *object,
 		if( (argument_class->flags & VIPS_ARGUMENT_INPUT) ) 
 			call->i += 1;
 		else if( (argument_class->flags & VIPS_ARGUMENT_OUTPUT) ) {
+			const char *name = g_param_spec_get_name( pspec );
 			const char *arg;
 
 			arg = NULL;
 			if( vips_object_get_argument_needs_string( object,
-				g_param_spec_get_name( pspec ) ) ) {
+				name ) ) {
 				arg = vips_call_get_arg( call, call->i );
 				if( !arg )
 					return( pspec );
@@ -689,7 +879,7 @@ vips_call_argv_output( VipsObject *object,
 			}
 
 			if( vips_object_get_argument_to_string( object, 
-				g_param_spec_get_name( pspec ), arg ) ) 
+				name, arg ) ) 
 				return( pspec );
 		}
 	}
@@ -697,35 +887,12 @@ vips_call_argv_output( VipsObject *object,
 	return( NULL );
 }
 
-static void *
-vips_call_argv_unref_output( VipsObject *object,
-	GParamSpec *pspec,
-	VipsArgumentClass *argument_class,
-	VipsArgumentInstance *argument_instance,
-	void *a, void *b )
-{
-	if( (argument_class->flags & VIPS_ARGUMENT_OUTPUT) &&
-		G_IS_PARAM_SPEC_OBJECT( pspec ) ) { 
-		GObject *value;
-
-		g_object_get( object, 
-			g_param_spec_get_name( pspec ), &value, NULL );
-
-		/* Doing the get refs the object, so unref the get, then unref
-		 * again since this an an output object of the operation.
-		 */
-		g_object_unref( value );
-		g_object_unref( value );
-	}
-
-	return( NULL );
-}
-
 /* Our main command-line entry point. Optional args should have been set by
  * the GOption parser already, see above.
  *
  * We don't create the operation, so we must not unref it. The caller must
- * unref on error too.
+ * unref on error too. The caller must also call vips_object_unref_outputs() on
+ * all code paths.
  */
 int
 vips_call_argv( VipsOperation *operation, int argc, char **argv )
@@ -738,8 +905,12 @@ vips_call_argv( VipsOperation *operation, int argc, char **argv )
 	printf( "vips_call_argv: " );
 	vips_object_print_name( VIPS_OBJECT( operation ) );
 	printf( "\n" );
+{
+	int i;
+
 	for( i = 0; i < argc; i++ )
 		printf( "%d) %s\n", i, argv[i] );
+}
 #endif /*VIPS_DEBUG*/
 
 	call.operation = operation;
@@ -747,30 +918,30 @@ vips_call_argv( VipsOperation *operation, int argc, char **argv )
 	call.argv = argv;
 
 	call.i = 0;
-	(void) vips_argument_map( VIPS_OBJECT( operation ),
-		vips_call_argv_input, &call, NULL );
-
-	if( vips_object_build( VIPS_OBJECT( operation ) ) ) {
-		/* We must unref any output objects, they are holding refs to
-		 * the operation.
-		 */
-		(void) vips_argument_map( VIPS_OBJECT( operation ),
-			vips_call_argv_unref_output, NULL, NULL );
+	if( vips_argument_map( VIPS_OBJECT( operation ),
+		vips_call_argv_input, &call, NULL ) ) 
+		return( -1 );
 
+	/* Any unused arguments? We must fail. Consider eg. "vips bandjoin a b
+	 * c". This would overwrite b with a and ignore c, potentially
+	 * disasterous.
+	 */
+	if( argc > call.i ) {
+		vips_error( VIPS_OBJECT_GET_CLASS( operation )->nickname, 
+			"%s", _( "too many arguments" ) );
 		return( -1 );
 	}
 
+	/* We can't use the operation cache, we need to be able to change the
+	 * operation pointer. The cache probably wouldn't help anyway.
+	 */
+	if( vips_object_build( VIPS_OBJECT( operation ) ) ) 
+		return( -1 );
+
 	call.i = 0;
 	if( vips_argument_map( VIPS_OBJECT( operation ),
-		vips_call_argv_output, &call, NULL ) ) {
-		(void) vips_argument_map( VIPS_OBJECT( operation ),
-			vips_call_argv_unref_output, NULL, NULL );
-
+		vips_call_argv_output, &call, NULL ) ) 
 		return( -1 );
-	}
-
-	(void) vips_argument_map( VIPS_OBJECT( operation ),
-		vips_call_argv_unref_output, NULL, NULL );
 
 	return( 0 );
 }
