@@ -93,10 +93,6 @@
 #include <vips/thread.h>
 #include <vips/debug.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /**
  * SECTION: region
  * @short_description: small, rectangular parts of images
@@ -205,10 +201,10 @@ vips__region_start( VipsRegion *region )
 {
 	VipsImage *image = region->im;
 
-        if( !region->seq && image->start ) {
+        if( !region->seq && image->start_fn ) {
                 g_mutex_lock( image->sslock );
-                region->seq = 
-			image->start( image, image->client1, image->client2 );
+                region->seq = image->start_fn( image, 
+			image->client1, image->client2 );
                 g_mutex_unlock( image->sslock );
  
                 if( !region->seq ) {
@@ -229,11 +225,11 @@ vips__region_stop( VipsRegion *region )
 {
 	IMAGE *image = region->im;
 
-        if( region->seq && image->stop ) {
+        if( region->seq && image->stop_fn ) {
 		int result;
 
                 g_mutex_lock( image->sslock );
-               	result = image->stop( region->seq, 
+               	result = image->stop_fn( region->seq, 
 			image->client1, image->client2 );
                 g_mutex_unlock( image->sslock );
 
@@ -286,13 +282,14 @@ vips_region_dispose( GObject *gobject )
 	image->regions = g_slist_remove( image->regions, region );
 	g_mutex_unlock( image->sslock );
 	region->im = NULL;
+
 	g_object_unref( image );
 
 	G_OBJECT_CLASS( vips_region_parent_class )->dispose( gobject );
 }
 
 static void
-vips_region_print( VipsObject *object, VipsBuf *buf )
+vips_region_dump( VipsObject *object, VipsBuf *buf )
 {
 	VipsRegion *region = VIPS_REGION( object );
 
@@ -308,10 +305,25 @@ vips_region_print( VipsObject *object, VipsBuf *buf )
 	vips_buf_appendf( buf, "seq = %p, ", region->seq );
 	vips_buf_appendf( buf, "thread = %p, ", region->thread );
 	vips_buf_appendf( buf, "window = %p, ", region->window );
-	vips_buf_appendf( buf, "buffer = %p\n", region->buffer );
-	vips_buf_appendf( buf, "invalid = %d\n", region->invalid );
+	vips_buf_appendf( buf, "buffer = %p, ", region->buffer );
+	vips_buf_appendf( buf, "invalid = %d", region->invalid );
 
-	VIPS_OBJECT_CLASS( vips_region_parent_class )->print( object, buf );
+	VIPS_OBJECT_CLASS( vips_region_parent_class )->dump( object, buf );
+}
+
+static void
+vips_region_summary( VipsObject *object, VipsBuf *buf )
+{
+	VipsRegion *region = VIPS_REGION( object );
+
+	vips_buf_appendf( buf, "VipsRegion: %p, ", region );
+	vips_buf_appendf( buf, "im = %p, ", region->im );
+	vips_buf_appendf( buf, "left = %d, ", region->valid.left );
+	vips_buf_appendf( buf, "top = %d, ", region->valid.top );
+	vips_buf_appendf( buf, "width = %d, ", region->valid.width );
+	vips_buf_appendf( buf, "height = %d", region->valid.height );
+
+	VIPS_OBJECT_CLASS( vips_region_parent_class )->summary( object, buf );
 }
 
 static void
@@ -325,7 +337,7 @@ vips_region_sanity( VipsObject *object, VipsBuf *buf )
 	case VIPS_IMAGE_PARTIAL:
 		/* Start and stop can be NULL, but not generate.
 		 */
-		if( !region->im->generate )
+		if( !region->im->generate_fn )
 			vips_buf_appends( buf, "generate NULL in partial\n" );
 		break;
 	
@@ -424,8 +436,9 @@ vips_region_class_init( VipsRegionClass *class )
 	gobject_class->finalize = vips_region_finalize;
 	gobject_class->dispose = vips_region_dispose;
 
-	vobject_class->print = vips_region_print;
-	vobject_class->print = vips_region_sanity;
+	vobject_class->summary = vips_region_summary;
+	vobject_class->dump = vips_region_dump;
+	vobject_class->sanity = vips_region_sanity;
 	vobject_class->build = vips_region_build;
 }
 
@@ -449,22 +462,28 @@ vips_region_new( VipsImage *image )
 {
 	VipsRegion *region;
 
-	region = VIPS_REGION( g_object_new( VIPS_TYPE_REGION, NULL ) );
-
-	/* We can't use the property system, we need to be very threaded.
+	/* Ref quickly, we want to make sure we keep the image around.
+	 * We can't use the property system, we need to be very threaded.
 	 */
-	region->im = image;
 	g_object_ref( image );
 
+	g_assert( G_OBJECT( image )->ref_count > 1 );
+
+#ifdef DEBUG
+	g_assert( vips_object_sanity( VIPS_OBJECT( image ) ) );
+#endif /*DEBUG*/
+
+	region = VIPS_REGION( g_object_new( VIPS_TYPE_REGION, NULL ) );
+	region->im = image;
+
 	if( vips_object_build( VIPS_OBJECT( region ) ) ) {
 		VIPS_UNREF( region );
 		return( NULL );
 	}
 
 #ifdef DEBUG
-#endif /*DEBUG*/
-	g_assert( vips_object_sanity( VIPS_OBJECT( image ) ) );
 	g_assert( vips_object_sanity( VIPS_OBJECT( region ) ) );
+#endif /*DEBUG*/
 
 	return( region ); 
 }
@@ -555,7 +574,9 @@ vips_region_buffer( VipsRegion *reg, VipsRect *r )
 int
 vips_region_image( VipsRegion *reg, VipsRect *r )
 {
-	VipsRect image;
+	VipsImage *image = reg->im;
+
+	VipsRect all;
 	VipsRect clipped;
 
 	/* Sanity check.
@@ -564,11 +585,11 @@ vips_region_image( VipsRegion *reg, VipsRect *r )
 
 	/* Clip against image.
 	 */
-	image.top = 0;
-	image.left = 0;
-	image.width = reg->im->Xsize;
-	image.height = reg->im->Ysize;
-	vips_rect_intersectrect( r, &image, &clipped );
+	all.top = 0;
+	all.left = 0;
+	all.width = image->Xsize;
+	all.height = image->Ysize;
+	vips_rect_intersectrect( r, &all, &clipped );
 
 	/* Test for empty.
 	 */
@@ -578,7 +599,7 @@ vips_region_image( VipsRegion *reg, VipsRect *r )
 		return( -1 );
 	}
 
-	if( reg->im->data ) {
+	if( image->data ) {
 		/* We have the whole image available ... easy!
 		 */
 		vips_region_reset( reg );
@@ -587,22 +608,21 @@ vips_region_image( VipsRegion *reg, VipsRect *r )
 		 * incompletely calculated memory buffer. Just set valid to r.
 		 */
 		reg->valid = clipped;
-		reg->bpl = VIPS_IMAGE_SIZEOF_LINE( reg->im );
-		reg->data = reg->im->data +
-			clipped.top * VIPS_IMAGE_SIZEOF_LINE( reg->im ) +
-			clipped.left * VIPS_IMAGE_SIZEOF_PEL( reg->im );
+		reg->bpl = VIPS_IMAGE_SIZEOF_LINE( image );
+		reg->data = VIPS_IMAGE_ADDR( image, clipped.left, clipped.top );
 		reg->type = VIPS_REGION_OTHER_IMAGE;
 	}
-	else if( reg->im->dtype == VIPS_IMAGE_OPENIN ) {
+	else if( image->dtype == VIPS_IMAGE_OPENIN ) {
 		/* No complete image data ... but we can use a rolling window.
 		 */
-		if( reg->type != VIPS_REGION_WINDOW || !reg->window ||
+		if( reg->type != VIPS_REGION_WINDOW || 
+			!reg->window ||
 			reg->window->top > clipped.top ||
 			reg->window->top + reg->window->height < 
 				clipped.top + clipped.height ) {
 			vips_region_reset( reg );
 
-			if( !(reg->window = vips_window_ref( reg->im, 
+			if( !(reg->window = vips_window_ref( image, 
 				clipped.top, clipped.height )) )
 				return( -1 );
 
@@ -613,14 +633,13 @@ vips_region_image( VipsRegion *reg, VipsRect *r )
 		 */
 		reg->valid.left = 0;
 		reg->valid.top = reg->window->top;
-		reg->valid.width = reg->im->Xsize;
+		reg->valid.width = image->Xsize;
 		reg->valid.height = reg->window->height;
-		reg->bpl = VIPS_IMAGE_SIZEOF_LINE( reg->im );
+		reg->bpl = VIPS_IMAGE_SIZEOF_LINE( image );
 		reg->data = reg->window->data;
 	}
 	else {
-		vips_error( "VipsRegion", 
-			"%s", _( "bad image type" ) );
+		vips_error( "VipsRegion", "%s", _( "bad image type" ) );
 		return( -1 );
 	}
 
@@ -652,7 +671,8 @@ vips_region_image( VipsRegion *reg, VipsRect *r )
  * Returns: 0 on success, or -1 for error.
  */
 int
-vips_region_region( VipsRegion *reg, VipsRegion *dest, VipsRect *r, int x, int y )
+vips_region_region( VipsRegion *reg, 
+	VipsRegion *dest, VipsRect *r, int x, int y )
 {
 	VipsRect image;
 	VipsRect wanted;
@@ -662,11 +682,15 @@ vips_region_region( VipsRegion *reg, VipsRegion *dest, VipsRect *r, int x, int y
 
 	/* Sanity check.
 	 */
-	if( !dest->data || 
-		VIPS_IMAGE_SIZEOF_PEL( dest->im ) != 
-			VIPS_IMAGE_SIZEOF_PEL( reg->im ) ) {
+	if( !dest->data ) {
 		vips_error( "VipsRegion", 
-			"%s", _( "inappropriate region type" ) );
+			"%s", _( "no pixel data on attached image" ) );
+		return( -1 );
+	}
+	if( VIPS_IMAGE_SIZEOF_PEL( dest->im ) != 
+		VIPS_IMAGE_SIZEOF_PEL( reg->im ) ) {
+		vips_error( "VipsRegion", 
+			"%s", _( "images do not match in pixel size" ) );
 		return( -1 );
 	}
 	vips__region_check_ownership( reg );
@@ -800,7 +824,7 @@ int
 vips_region_fill( VipsRegion *reg, VipsRect *r, VipsRegionFillFn fn, void *a )
 {
 	g_assert( reg->im->dtype == VIPS_IMAGE_PARTIAL );
-	g_assert( reg->im->generate );
+	g_assert( reg->im->generate_fn );
 
 	/* Should have local memory.
 	 */
@@ -841,7 +865,7 @@ vips_region_paint( VipsRegion *reg, VipsRect *r, int value )
 
 	vips_rect_intersectrect( r, &reg->valid, &ovl );
 	if( !vips_rect_isempty( &ovl ) ) {
-		PEL *q = (PEL *) VIPS_REGION_ADDR( reg, ovl.left, ovl.top );
+		VipsPel *q = VIPS_REGION_ADDR( reg, ovl.left, ovl.top );
 		int wd = ovl.width * VIPS_IMAGE_SIZEOF_PEL( reg->im );
 		int ls = VIPS_REGION_LSKIP( reg );
 		int y;
@@ -853,6 +877,57 @@ vips_region_paint( VipsRegion *reg, VipsRect *r, int value )
 	}
 }
 
+/**
+ * vips_region_paint_pel:
+ * @reg: region to operate upon
+ * @r: area to paint
+ * @ink: value to paint
+ *
+ * Paints @ink into @reg covering rectangle @r. @r is clipped against 
+ * @reg->valid.
+ *
+ * @ink should be a byte array of the same size as an image pixel containing
+ * the binary value to write into the pixels.
+ *
+ * See also: vips_region_paint().
+ */
+void
+vips_region_paint_pel( VipsRegion *reg, VipsRect *r, VipsPel *ink )
+{
+	VipsRect ovl;
+
+	vips_rect_intersectrect( r, &reg->valid, &ovl );
+	if( !vips_rect_isempty( &ovl ) ) {
+		int ps = VIPS_IMAGE_SIZEOF_PEL( reg->im );
+		int ws = ovl.width * ps;
+		int ls = VIPS_REGION_LSKIP( reg );
+
+		VipsPel *to, *q;
+		int x, y, z;
+
+		/* We plot the first line pointwise, then memcpy() it for the
+		 * subsequent lines.
+		 */
+		to = VIPS_REGION_ADDR( reg, ovl.left, ovl.top );
+
+		q = to;
+		for( x = 0; x < ovl.width; x++ ) {
+			/* Faster than memcpy() for about n<20.
+			 */
+			for( z = 0; z < ps; z++ )
+				q[z] = ink[z];
+
+			q += ps;
+		}
+
+		q = to + ls;
+		for( y = 1; y < ovl.height; y++ ) {
+			memcpy( q, to, ws );
+			q += ls;
+		}
+	}
+}
+
 /**
  * vips_region_black:
  * @reg: region to operate upon
@@ -886,8 +961,8 @@ vips_region_copy( VipsRegion *reg, VipsRegion *dest, VipsRect *r, int x, int y )
 {
 	int z;
 	int len = VIPS_IMAGE_SIZEOF_PEL( reg->im ) * r->width;
-	char *p = VIPS_REGION_ADDR( reg, r->left, r->top );
-	char *q = VIPS_REGION_ADDR( dest, x, y );
+	VipsPel *p = VIPS_REGION_ADDR( reg, r->left, r->top );
+	VipsPel *q = VIPS_REGION_ADDR( dest, x, y );
 	int plsk = VIPS_REGION_LSKIP( reg );
 	int qlsk = VIPS_REGION_LSKIP( dest );
 
@@ -911,7 +986,7 @@ vips_region_copy( VipsRegion *reg, VipsRegion *dest, VipsRect *r, int x, int y )
 	 */
 	g_assert( vips_rect_includesrect( &reg->valid, r ) );
 
-	/* PEL size must be the same.
+	/* VipsPel size must be the same.
 	 */
 	g_assert( VIPS_IMAGE_SIZEOF_PEL( reg->im ) == 
 		VIPS_IMAGE_SIZEOF_PEL( dest->im ) );
@@ -939,7 +1014,7 @@ vips_region_generate( VipsRegion *reg )
 
 	/* Ask for evaluation.
 	 */
-	if( im->generate( reg, reg->seq, im->client1, im->client2 ) )
+	if( im->generate_fn( reg, reg->seq, im->client1, im->client2 ) )
 		return( -1 );
 
 	return( 0 );
@@ -1025,7 +1100,7 @@ vips_region_prepare( VipsRegion *reg, VipsRect *r )
 	default:
 		vips_error( "vips_region_prepare", 
 			_( "unable to input from a %s image" ),
-			VIPS_ENUM_STRING( VIPS_TYPE_DEMAND_STYLE, im->dtype ) );
+			vips_enum_string( VIPS_TYPE_DEMAND_STYLE, im->dtype ) );
 		return( -1 );
 	}
 
@@ -1044,9 +1119,9 @@ vips_region_prepare_to_generate( VipsRegion *reg,
 	VipsRegion *dest, VipsRect *r, int x, int y )
 {
 	IMAGE *im = reg->im;
-	char *p;
+	VipsPel *p;
 
-	if( !im->generate ) {
+	if( !im->generate_fn ) {
 		vips_error( "vips_region_prepare_to",
 			"%s", _( "incomplete header" ) );
 		return( -1 );
@@ -1201,7 +1276,7 @@ vips_region_prepare_to( VipsRegion *reg,
 		/* Could be either input or output. If there is a generate
 		 * function, we are outputting.
 		 */
-		if( im->generate ) {
+		if( im->generate_fn ) {
 			if( vips_region_prepare_to_generate( reg, 
 				dest, &final, x, y ) )
 				return( -1 );
@@ -1217,7 +1292,7 @@ vips_region_prepare_to( VipsRegion *reg,
 	default:
 		vips_error( "vips_region_prepare_to", 
 			_( "unable to input from a %s image" ), 
-			VIPS_ENUM_NICK( VIPS_TYPE_DEMAND_STYLE, im->dtype ) );
+			vips_enum_nick( VIPS_TYPE_DEMAND_STYLE, im->dtype ) );
 		return( -1 );
 	}
 
