@@ -49,10 +49,6 @@
 
 #include "sink.h"
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Per-call state.
  */
 typedef struct _Sink {
@@ -65,9 +61,9 @@ typedef struct _Sink {
 
 	/* Call params.
 	 */
-	VipsStartFn start;
-	VipsGenerateFn generate;
-	VipsStopFn stop;
+	VipsStartFn start_fn;
+	VipsGenerateFn generate_fn;
+	VipsStopFn stop_fn;
 	void *a;
 	void *b;
 } Sink;
@@ -99,10 +95,10 @@ G_DEFINE_TYPE( SinkThreadState, sink_thread_state, VIPS_TYPE_THREAD_STATE );
 static int
 sink_call_stop( Sink *sink, SinkThreadState *state )
 {
-	if( state->seq && sink->stop ) {
+	if( state->seq && sink->stop_fn ) {
 		VIPS_DEBUG_MSG( "sink_call_stop: state = %p\n", state );
 
-		if( sink->stop( state->seq, sink->a, sink->b ) ) {
+		if( sink->stop_fn( state->seq, sink->a, sink->b ) ) {
 			SinkBase *sink_base = (SinkBase *) sink;
 
 			vips_error( "vips_sink", 
@@ -134,10 +130,10 @@ sink_thread_state_dispose( GObject *gobject )
 static int
 sink_call_start( Sink *sink, SinkThreadState *state )
 {
-	if( !state->seq && sink->start ) {
+	if( !state->seq && sink->start_fn ) {
 		VIPS_DEBUG_MSG( "sink_call_start: state = %p\n", state );
 
-                state->seq = sink->start( sink->t, sink->a, sink->b );
+                state->seq = sink->start_fn( sink->t, sink->a, sink->b );
 
 		if( !state->seq ) {
 			SinkBase *sink_base = (SinkBase *) sink;
@@ -186,8 +182,8 @@ sink_thread_state_init( SinkThreadState *state )
 	state->reg = NULL;
 }
 
-static VipsThreadState *
-sink_thread_state_new( VipsImage *im, void *a )
+VipsThreadState *
+vips_sink_thread_state_new( VipsImage *im, void *a )
 {
 	return( VIPS_THREAD_STATE( vips_object_new( 
 		sink_thread_state_get_type(), 
@@ -215,22 +211,24 @@ vips_sink_base_init( SinkBase *sink_base, VipsImage *image )
 	vips_get_tile_size( image, 
 		&sink_base->tile_width, &sink_base->tile_height, 
 		&sink_base->nlines );
+
+	sink_base->processed = 0;
 }
 
 static int
 sink_init( Sink *sink, 
 	VipsImage *image, 
-	VipsStartFn start, VipsGenerateFn generate, VipsStopFn stop,
+	VipsStartFn start_fn, VipsGenerateFn generate_fn, VipsStopFn stop_fn,
 	void *a, void *b )
 {
-	g_assert( generate );
+	g_assert( generate_fn );
 
 	vips_sink_base_init( &sink->sink_base, image );
 
 	sink->t = NULL;
-	sink->start = start;
-	sink->generate = generate;
-	sink->stop = stop;
+	sink->start_fn = start_fn;
+	sink->generate_fn = generate_fn;
+	sink->stop_fn = stop_fn;
 	sink->a = a;
 	sink->b = b;
 
@@ -250,6 +248,14 @@ vips_sink_base_allocate( VipsThreadState *state, void *a, gboolean *stop )
 
 	VipsRect image, tile;
 
+	/* Has work requested early termination?
+	 */
+	if( state->stop ) {
+		*stop = TRUE;
+
+		return( 0 );
+	}
+
 	/* Is the state x/y OK? New line or maybe all done.
 	 */
 	if( sink_base->x >= sink_base->im->Xsize ) {
@@ -279,6 +285,10 @@ vips_sink_base_allocate( VipsThreadState *state, void *a, gboolean *stop )
 	 */
 	sink_base->x += sink_base->tile_width;
 
+	/* Add the number of pixels we've just allocated to progress.
+	 */
+	sink_base->processed += state->pos.width * state->pos.height;
+
 	return( 0 );
 }
 
@@ -289,7 +299,8 @@ sink_work( VipsThreadState *state, void *a )
 	Sink *sink = (Sink *) a;
 
 	if( vips_region_prepare( sstate->reg, &state->pos ) ||
-		sink->generate( sstate->reg, sstate->seq, sink->a, sink->b ) ) 
+		sink->generate_fn( sstate->reg, sstate->seq, 
+			sink->a, sink->b, &state->stop ) ) 
 		return( -1 );
 
 	return( 0 );
@@ -300,14 +311,12 @@ vips_sink_base_progress( void *a )
 {
 	SinkBase *sink_base = (SinkBase *) a;
 
-	VIPS_DEBUG_MSG( "vips_sink_base_progress: %d x %d\n",
-		sink_base->tile_width, sink_base->tile_height );
+	VIPS_DEBUG_MSG( "vips_sink_base_progress:\n" ); 
 
 	/* Trigger any eval callbacks on our source image and
 	 * check for errors.
 	 */
-	vips_image_eval( sink_base->im, 
-		sink_base->tile_width, sink_base->tile_height );
+	vips_image_eval( sink_base->im, sink_base->processed );
 	if( vips_image_get_kill( sink_base->im ) )
 		return( -1 );
 
@@ -319,16 +328,14 @@ vips_sink_base_progress( void *a )
  * @im: scan over this image
  * @tile_width: tile width
  * @tile_height: tile height
- * @start: start sequences with this function
- * @generate: generate pixels with this function
- * @stop: stop sequences with this function
+ * @start_fn: start sequences with this function
+ * @generate_fn: generate pixels with this function
+ * @stop_fn: stop sequences with this function
  * @a: user data
  * @b: user data
  *
  * Loops over an image. @generate is called for every pixel in the image, with
  * the @reg argument being a region of pixels for processing. 
- * vips_sink_tile() is
- * used to implement operations like im_avg() which have no image output.
  *
  * Each set of
  * pixels is @tile_width by @tile_height pixels (less at the image edges). 
@@ -343,7 +350,7 @@ vips_sink_base_progress( void *a )
 int
 vips_sink_tile( VipsImage *im, 
 	int tile_width, int tile_height,
-	VipsStart start, VipsGenerate generate, VipsStop stop,
+	VipsStartFn start_fn, VipsGenerateFn generate_fn, VipsStopFn stop_fn,
 	void *a, void *b )
 {
 	Sink sink;
@@ -356,7 +363,7 @@ vips_sink_tile( VipsImage *im,
 	 */
 	im->Bbits = vips_format_sizeof( im->BandFmt ) << 3;
  
-	if( sink_init( &sink, im, start, generate, stop, a, b ) )
+	if( sink_init( &sink, im, start_fn, generate_fn, stop_fn, a, b ) )
 		return( -1 );
 
 	if( tile_width > 0 ) {
@@ -370,7 +377,7 @@ vips_sink_tile( VipsImage *im,
 	vips_image_preeval( im );
 
 	result = vips_threadpool_run( im, 
-		sink_thread_state_new,
+		vips_sink_thread_state_new,
 		vips_sink_base_allocate, 
 		sink_work, 
 		vips_sink_base_progress, 
@@ -386,15 +393,15 @@ vips_sink_tile( VipsImage *im,
 /**
  * vips_sink:
  * @im: scan over this image
- * @start: start sequences with this function
- * @generate: generate pixels with this function
- * @stop: stop sequences with this function
+ * @start_fn: start sequences with this function
+ * @generate_fn: generate pixels with this function
+ * @stop_fn: stop sequences with this function
  * @a: user data
  * @b: user data
  *
  * Loops over an image. @generate is called for every pixel in the image, with
  * the @reg argument being a region of pixels for processing. vips_sink() is
- * used to implement operations like im_avg() which have no image output.
+ * used to implement operations like #VipsAvg which have no image output.
  *
  * Each set of pixels is sized according to the requirements of the image
  * pipeline that generated @im.
@@ -405,8 +412,9 @@ vips_sink_tile( VipsImage *im,
  */
 int
 vips_sink( VipsImage *im, 
-	VipsStart start, VipsGenerate generate, VipsStop stop,
+	VipsStartFn start_fn, VipsGenerateFn generate_fn, VipsStopFn stop_fn,
 	void *a, void *b )
 {
-	return( vips_sink_tile( im, -1, -1, start, generate, stop, a, b ) );
+	return( vips_sink_tile( im, -1, -1, 
+		start_fn, generate_fn, stop_fn, a, b ) );
 }
