@@ -7,6 +7,8 @@
  * 	- better buffer handling for single-line images
  * 17/7/10
  * 	- we could get stuck if allocate failed (thanks Tim)
+ * 23/2/12
+ * 	- we could deadlock if generate failed
  */
 
 /*
@@ -47,7 +49,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include <assert.h>
 #include <errno.h>
 #include <string.h>
 #include <sys/types.h>
@@ -63,10 +64,6 @@
 
 #include "sink.h"
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* A buffer we are going to write to disc in a background thread.
  */
 typedef struct _WriteBuffer {
@@ -353,15 +350,17 @@ wbuffer_allocate_fn( VipsThreadState *state, void *a, gboolean *stop )
 				return( 0 );
 			}
 
+			VIPS_DEBUG_MSG( "wbuffer_allocate_fn: "
+				"finished top = %d, height = %d\n",
+				write->buf->area.top, write->buf->area.height );
+
+			VIPS_DEBUG_MSG( "wbuffer_allocate_fn: "
+				"starting top = %d, height = %d\n",
+				sink_base->y, sink_base->nlines );
+
 			/* Swap buffers.
 			 */
-			{
-				WriteBuffer *t;
-
-				t = write->buf; 
-				write->buf = write->buf_back; 
-				write->buf_back = t;
-			}
+			VIPS_SWAP( WriteBuffer *, write->buf, write->buf_back );
 
 			/* Position buf at the new y.
 			 */
@@ -382,6 +381,9 @@ wbuffer_allocate_fn( VipsThreadState *state, void *a, gboolean *stop )
 	tile.width = sink_base->tile_width;
 	tile.height = sink_base->tile_height;
 	vips_rect_intersectrect( &image, &tile, &state->pos );
+
+	/* The thread needs to know which buffer it's writing to.
+	 */
 	wstate->buf = write->buf;
 
 	VIPS_DEBUG_MSG( "  allocated %d x %d:\n", tile.left, tile.top );
@@ -394,6 +396,10 @@ wbuffer_allocate_fn( VipsThreadState *state, void *a, gboolean *stop )
 	 */
 	sink_base->x += sink_base->tile_width;
 
+	/* Add the number of pixels we've just allocated to progress.
+	 */
+	sink_base->processed += state->pos.width * state->pos.height;
+
 	return( 0 );
 }
 
@@ -404,21 +410,21 @@ wbuffer_work_fn( VipsThreadState *state, void *a )
 {
 	WriteThreadState *wstate = (WriteThreadState *) state;
 
+	int result;
+
 	VIPS_DEBUG_MSG( "wbuffer_work_fn: %p %d x %d\n", 
 		state, state->pos.left, state->pos.top );
 
-	if( vips_region_prepare_to( state->reg, wstate->buf->region, 
-		&state->pos, state->pos.left, state->pos.top ) ) {
-		VIPS_DEBUG_MSG( "wbuffer_work_fn: %p error!\n", state );
-		return( -1 );
-	}
-	VIPS_DEBUG_MSG( "wbuffer_work_fn: %p done\n", state );
+	result = vips_region_prepare_to( state->reg, wstate->buf->region, 
+		&state->pos, state->pos.left, state->pos.top );
+
+	VIPS_DEBUG_MSG( "wbuffer_work_fn: %p result = %d\n", state, result );
 
 	/* Tell the bg write thread we've left.
 	 */
 	vips_semaphore_upn( &wstate->buf->nwrite, 1 );
 
-	return( 0 );
+	return( result );
 }
 
 static void
