@@ -66,10 +66,6 @@
 #include <vips/thread.h>
 #include <vips/debug.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* A have-threads we can test in if().
  */
 #ifdef HAVE_THREADS
@@ -885,9 +881,9 @@ tile_copy( Tile *tile, VipsRegion *to )
 			tile, tile->area.left, tile->area.top ); 
 
 		for( y = ovlap.top; y < VIPS_RECT_BOTTOM( &ovlap ); y++ ) {
-			PEL *p = (PEL *) VIPS_REGION_ADDR( tile->region, 
+			VipsPel *p = VIPS_REGION_ADDR( tile->region, 
 				ovlap.left, y );
-			PEL *q = (PEL *) VIPS_REGION_ADDR( to, ovlap.left, y );
+			VipsPel *q = VIPS_REGION_ADDR( to, ovlap.left, y );
 
 			memcpy( q, p, len );
 		}
@@ -910,7 +906,7 @@ image_start( IMAGE *out, void *a, void *b )
 /* Loop over the output region, filling with data from cache.
  */
 static int
-image_fill( VipsRegion *out, void *seq, void *a, void *b )
+image_fill( VipsRegion *out, void *seq, void *a, void *b, gboolean *stop )
 {
 	Render *render = (Render *) a;
 	VipsRegion *reg = (VipsRegion *) seq;
@@ -970,7 +966,7 @@ image_stop( void *seq, void *a, void *b )
 /* The mask image is 255 / 0 for the state of painted for each tile.
  */
 static int
-mask_fill( VipsRegion *out, void *seq, void *a, void *b )
+mask_fill( VipsRegion *out, void *seq, void *a, void *b, gboolean *stop )
 {
 #ifdef HAVE_THREADS
 	Render *render = (Render *) a;
@@ -1088,16 +1084,14 @@ vips_sink_screen( VipsImage *in, VipsImage *out, VipsImage *mask,
 	}
 
 	if( vips_image_pio_input( in ) ||
-		vips_image_pio_output( out ) ||
-		vips_image_copy_fields( out, in ) ||
-		vips_demand_hint( out, VIPS_DEMAND_STYLE_SMALLTILE, in, NULL ) )
+		vips_image_copy_fields( out, in ) )
 		return( -1 );
+	vips_demand_hint( out, VIPS_DEMAND_STYLE_SMALLTILE, in, NULL );
+
 	if( mask ) {
-		if( vips_image_pio_output( mask ) ||
-			vips_image_copy_fields( mask, in ) ||
-			vips_demand_hint( mask, 
-				VIPS_DEMAND_STYLE_SMALLTILE, in, NULL ) )
+		if( vips_image_copy_fields( mask, in ) )
 			return( -1 );
+		vips_demand_hint( mask, VIPS_DEMAND_STYLE_SMALLTILE, in, NULL );
 
 		mask->Bands = 1;
 		mask->BandFmt = VIPS_FORMAT_UCHAR;
@@ -1130,11 +1124,3 @@ vips__print_renders( void )
 #endif /*VIPS_DEBUG_AMBER*/
 	printf( "%d dirty renders\n", g_slist_length( render_dirty_all ) );
 }
-
-int
-vips_image_cache( VipsImage *in, VipsImage *out, 
-	int width, int height, int max )
-{
-	return( vips_sink_screen( in, out, NULL, 
-		width, height, max, 0, NULL, NULL ) );
-}
