@@ -68,10 +68,6 @@
 #include <windows.h>
 #endif /*OS_WIN32*/
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /**
  * SECTION: threadpool
  * @short_description: pools of worker threads 
@@ -291,6 +287,7 @@ vips_thread_state_init( VipsThreadState *state )
 	VIPS_DEBUG_MSG( "vips_thread_state_init:\n" );
 
 	state->reg = NULL;
+	state->stop = FALSE;
 }
 
 void *
@@ -356,9 +353,9 @@ typedef struct _VipsThreadpool {
 	 * a unit of work (serial). Plus the mutex we use to serialize work 
 	 * allocation.
 	 */
-	VipsThreadStart start; 
-	VipsThreadpoolAllocate allocate;
-	VipsThreadpoolWork work;
+	VipsThreadStartFn start; 
+	VipsThreadpoolAllocateFn allocate;
+	VipsThreadpoolWorkFn work;
 	GMutex *allocate_lock;
         void *a; 		/* User argument to start / allocate / etc. */
 
@@ -592,7 +589,7 @@ vips_thread_new( VipsThreadpool *pool )
 	thr->btime = VIPS_ARRAY( pool->im, TBUF_SIZE, double );
 	thr->etime = VIPS_ARRAY( pool->im, TBUF_SIZE, double );
 	if( !thr->btime || !thr->etime ) {
-		thread_free( thr );
+		vips_thread_free( thr );
 		return( NULL );
 	}
 #endif /*TIME_THREAD*/
@@ -715,7 +712,7 @@ vips_threadpool_create_threads( VipsThreadpool *pool )
 }
 
 /**
- * VipsThreadpoolStart:
+ * VipsThreadpoolStartFn:
  * @a: client data
  * @b: client data
  * @c: client data
@@ -733,7 +730,7 @@ vips_threadpool_create_threads( VipsThreadpool *pool )
  */
 
 /**
- * _VipsThreadpoolAllocate:
+ * VipsThreadpoolAllocateFn:
  * @state: per-thread state
  * @a: client data
  * @b: client data
@@ -756,7 +753,7 @@ vips_threadpool_create_threads( VipsThreadpool *pool )
  */
 
 /**
- * _VipsThreadpoolWork:
+ * VipsThreadpoolWorkFn:
  * @state: per-thread state
  * @a: client data
  * @b: client data
@@ -775,7 +772,7 @@ vips_threadpool_create_threads( VipsThreadpool *pool )
  */
 
 /**
- * _VipsThreadpoolProgress:
+ * VipsThreadpoolProgressFn:
  * @a: client data
  * @b: client data
  * @c: client data
@@ -819,10 +816,10 @@ vips_threadpool_create_threads( VipsThreadpool *pool )
  */
 int
 vips_threadpool_run( VipsImage *im, 
-	VipsThreadStart start, 
-	VipsThreadpoolAllocate allocate, 
-	VipsThreadpoolWork work,
-	VipsThreadpoolProgress progress, 
+	VipsThreadStartFn start, 
+	VipsThreadpoolAllocateFn allocate, 
+	VipsThreadpoolWorkFn work,
+	VipsThreadpoolProgressFn progress, 
 	void *a )
 {
 	VipsThreadpool *pool; 
@@ -885,6 +882,14 @@ vips_threadpool_run( VipsImage *im,
 	return( result );
 }
 
+/* Round N down to P boundary. 
+ */
+#define ROUND_DOWN(N,P) ((N) - ((N) % P)) 
+
+/* Round N up to P boundary. 
+ */
+#define ROUND_UP(N,P) (ROUND_DOWN( (N) + (P) - 1, (P) ))
+
 /**
  * vips_get_tile_size:
  * @im: image to guess for
@@ -908,31 +913,36 @@ vips_get_tile_size( VipsImage *im,
 	case VIPS_DEMAND_STYLE_SMALLTILE:
 		*tile_width = vips__tile_width;
 		*tile_height = vips__tile_height;
-
-		/* Enough lines of tiles that we can expect to be able to keep
-		 * nthr busy. Then double it.
-		 */
-		*nlines = *tile_height * 
-			(1 + nthr / VIPS_MAX( 1, im->Xsize / *tile_width )) * 2;
 		break;
 
 	case VIPS_DEMAND_STYLE_ANY:
 	case VIPS_DEMAND_STYLE_FATSTRIP:
 		*tile_width = im->Xsize;
 		*tile_height = vips__fatstrip_height;
-		*nlines = *tile_height * nthr * 2;
 		break;
 
 	case VIPS_DEMAND_STYLE_THINSTRIP:
 		*tile_width = im->Xsize;
 		*tile_height = vips__thinstrip_height;
-		*nlines = *tile_height * nthr * 2;
 		break;
 
 	default:
 		g_assert( 0 );
 	}
 
+	/* We can't set nlines for the current demand style: a later bit of
+	 * the pipeline might see a different hint and we need to synchronise
+	 * buffer sizes everywhere.
+	 *
+	 * Pick the maximum buffer size we might possibly need, then round up
+	 * to a multiple of tileheight.
+	 */
+	*nlines = vips__tile_height * 
+		(1 + nthr / VIPS_MAX( 1, im->Xsize / vips__tile_width )) * 2;
+	*nlines = VIPS_MAX( *nlines, vips__fatstrip_height * nthr * 2 );
+	*nlines = VIPS_MAX( *nlines, vips__thinstrip_height * nthr * 2 );
+	*nlines = ROUND_UP( *nlines, *tile_height );
+
 	/* We make this assumption in several places.
 	 */
 	g_assert( *nlines % *tile_height == 0 );
