@@ -55,10 +55,7 @@
 
 #include <vips/vips.h>
 #include <vips/internal.h>
-
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
+#include <vips/debug.h>
 
 /* Try to make an O_BINARY ... sometimes need the leading '_'.
  */
@@ -92,7 +89,7 @@
 
 /* Temp buffer for snprintf() layer on old systems.
  */
-#define MAX_BUF (32768)
+#define MAX_BUF (100000)
 
 /* Test two lists for eqality.
  */
@@ -273,6 +270,8 @@ vips_hash_table_map( GHashTable *hash, VipsSListMap2Fn fn, void *a, void *b )
 }
 
 /* Like strncpy(), but always NULL-terminate, and don't pad with NULLs.
+ * If @n is 100 and @src is more than 99 characters, 99 are copied and the
+ * final byte of @dest is set to '\0'.
  */
 char *
 vips_strncpy( char *dest, const char *src, int n )
@@ -855,7 +854,7 @@ vips__file_write( void *data, size_t size, size_t nmemb, FILE *stream )
 
 	if( (n = fwrite( data, size, nmemb, stream )) != nmemb ) {
 		vips_error( "vips__file_write", 
-			_( "writing error (%zd out of %zd blocks written) "
+			_( "write error (%zd out of %zd blocks written) "
 			"... disc full?" ), n, nmemb );
 		return( -1 );
 	}
@@ -929,7 +928,7 @@ vips__gvalue_ref_string_new( const char *text )
 	GValue *value;
 
 	value = vips__gvalue_new( VIPS_TYPE_REF_STRING );
-	vips_ref_string_set( value, text );
+	vips_value_set_ref_string( value, text );
 
 	return( value );
 }
@@ -988,8 +987,8 @@ vips__gslist_gvalue_merge( GSList *a, const GSList *b )
 			/* Just do a pointer compare ... good enough 99.9% of 
 			 * the time.
 			 */
-			if( vips_ref_string_get( value ) ==
-				vips_ref_string_get( value2 ) )
+			if( vips_value_get_ref_string( value, NULL ) ==
+				vips_value_get_ref_string( value2, NULL ) )
 				break;
 		}
 
@@ -1019,12 +1018,14 @@ vips__gslist_gvalue_get( const GSList *list )
 	length = 0;
 	for( p = list; p; p = p->next ) {
 		GValue *value = (GValue *) p->data;
+		size_t l2;
 
 		g_assert( G_VALUE_TYPE( value ) == VIPS_TYPE_REF_STRING );
 
 		/* +1 for the newline we will add for each item.
 		 */
-		length += vips_ref_string_get_length( value ) + 1;
+		(void) vips_value_get_ref_string( value, &l2 );
+		length += l2 + 1;
 	}
 
 	if( length == 0 )
@@ -1042,9 +1043,10 @@ vips__gslist_gvalue_get( const GSList *list )
 	q = all;
 	for( p = list; p; p = p->next ) {
 		GValue *value = (GValue *) p->data;
+		size_t l2;
 
-		strcpy( q, vips_ref_string_get( value ) );
-		q += vips_ref_string_get_length( value );
+		strcpy( q, vips_value_get_ref_string( value, &l2 ) );
+		q += l2;
 		strcpy( q, "\n" );
 		q += 1;
 	}
@@ -1166,10 +1168,13 @@ vips_popenf( const char *fmt, const char *mode, ... )
 	return( fp );
 }
 
-/* Break a command-line argument into tokens separated by whitespace. Strings
- * can't be adjacent, so "hello world" (without quotes) is a single string.
- * Strings are written (with \" escaped) into string, which must be size 
- * characters large.
+/* Break a command-line argument into tokens separated by whitespace. 
+ *
+ * Strings can't be adjacent, so "hello world" (without quotes) is a single 
+ * string.  Strings are written (with \" escaped) into @string. If the string
+ * is larger than @size, it is silently null-termionated and truncated. 
+ *
+ * Return NULL for end of tokens.
  */
 const char *
 vips__token_get( const char *p, VipsToken *token, char *string, int size )
@@ -1177,6 +1182,7 @@ vips__token_get( const char *p, VipsToken *token, char *string, int size )
 	const char *q;
 	int ch;
 	int n;
+	int i;
 
 	/* Parse this token with p.
 	 */
@@ -1232,19 +1238,20 @@ vips__token_get( const char *p, VipsToken *token, char *string, int size )
 			else
 				n = strlen( p + 1 );
 
-			g_assert( size > n + 1 );
-			memcpy( string, p + 1, n );
-			string[n] = '\0';
-
-			/* p[n + 1] might not be " if there's no closing ".
+			/* How much can we copy to the buffer?
 			 */
-			if( p[n + 1] == ch && p[n] == '\\' ) 
-				/* An escaped ": overwrite the '\' with '"'
-				 */
-				string[n - 1] = ch;
+			i = VIPS_MIN( n, size );
+			vips_strncpy( string, p + 1, i );
 
-			string += n;
-			size -= n;
+			/* We might have stopped at an escaped quote. If the
+			 * string was not truncated, swap the preceding 
+			 * backslash for a quote.
+			 */
+			if( p[n + 1] == ch && p[n] == '\\' && i == n )
+				string[i - 1] = ch;
+
+			string += i;
+			size -= i;
 			p += n + 1;
 		} while( p[0] && p[-1] == '\\' );
 
@@ -1259,18 +1266,19 @@ vips__token_get( const char *p, VipsToken *token, char *string, int size )
 		 */
 		*token = VIPS_TOKEN_STRING;
 		n = strcspn( p, "<[{()}]>=," );
-		g_assert( size > n + 1 );
-		memcpy( string, p, n );
-		string[n] = '\0';
+		i = VIPS_MIN( n, size );
+		vips_strncpy( string, p, i + 1 );
 		p += n;
 
 		/* We remove leading whitespace, so we trim trailing
-		 * whitespace from unquoted strings too.
+		 * whitespace from unquoted strings too. Only if the string
+		 * hasn't been truncated.
 		 */
-		while( isspace( string[n - 1] ) ) {
-			string[n - 1] = '\0';
-			n -= 1;
-		}
+		if( i == n ) 
+			while( i > 0 && isspace( string[i - 1] ) ) {
+				string[i - 1] = '\0';
+				i--;
+			}
 
 		break;
 	}
@@ -1305,14 +1313,82 @@ vips__token_need( const char *p, VipsToken need_token,
 		return( NULL );
 	if( token != need_token ) {
 		vips_error( "get_token", _( "expected %s, saw %s" ), 
-			VIPS_ENUM_NICK( VIPS_TYPE_TOKEN, need_token ),
-			VIPS_ENUM_NICK( VIPS_TYPE_TOKEN, token ) );
+			vips_enum_nick( VIPS_TYPE_TOKEN, need_token ),
+			vips_enum_nick( VIPS_TYPE_TOKEN, token ) );
 		return( NULL );
 	}
 
 	return( p );
 }
 
+/* Maximum number of tokens we allow in a filename. Surely this will be
+ * plenty.
+ */
+#define MAX_TOKENS (1000)
+
+/* Find the start of the right-most pair of brackets in the string.
+ *
+ * A string can be of the form:
+ *
+ * 	"hello world! (no really).tif[fred=12]"
+ *
+ * we need to be able to find the fred=12 at the end.
+ *
+ * We lex the whole string noting the position of each token, then, if the 
+ * final token is a right-bracket, search left for the matching left-bracket.
+ *
+ * This can get confused if the lefts are hidden inside another token :-( But
+ * a fixing that would require us to write a separate right-to-left lexer, 
+ * argh.
+ */
+const char *
+vips__find_rightmost_brackets( const char *p )
+{
+	const char *start[MAX_TOKENS];
+	VipsToken tokens[MAX_TOKENS];
+	char str[PATH_MAX];
+	int n, i;
+	int nest;
+
+	start[0] = p;
+	for( n = 0; 
+		n < MAX_TOKENS &&
+		(p = vips__token_get( start[n], &tokens[n], str, PATH_MAX )); 
+		n++, start[n] = p )
+		;
+
+	/* Too many tokens?
+	 */
+	if( n == MAX_TOKENS )
+		return( NULL );
+
+	/* No rightmost close bracket?
+	 */
+	if( n == 0 ||
+		tokens[n - 1] != VIPS_TOKEN_RIGHT ) 
+		return( NULL );
+
+	nest = 0;
+	for( i = n - 1; i >= 0; i-- ) {
+		if( tokens[i] == VIPS_TOKEN_RIGHT )
+			nest += 1;
+		else if( tokens[i] == VIPS_TOKEN_LEFT )
+			nest -= 1;
+
+		if( nest == 0 )
+			break;
+	}
+
+	/* No matching left bracket?
+	 */
+	if( nest != 0 )
+		return( NULL );
+
+	/* This should be the matching left.
+	 */
+	return( start[i] );
+}
+
 /* True if an int is a power of two ... 1, 2, 4, 8, 16, 32, etc. Do with just
  * integer arithmetic for portability. A previous Nicos version using doubles
  * and log/log failed on x86 with rounding problems. Return 0 for not
@@ -1455,3 +1531,66 @@ vips__change_suffix( const char *name, char *out, int mx,
 	vips_strncpy( out + len, new, mx - len );
 }
 
+typedef struct {
+	const char unit;
+	int multiplier;
+} Unit;
+
+guint64
+vips__parse_size( const char *size_string )
+{
+	static Unit units[] = {
+		{ 'k', 1024 },
+		{ 'm', 1024 * 1024 },
+		{ 'g', 1024 * 1024 * 1024 }
+	};
+
+	guint64 size;
+	int n;
+	int i, j;
+	char *unit;
+
+	/* An easy way to alloc a buffer large enough.
+	 */
+	unit = g_strdup( size_string );
+	n = sscanf( size_string, "%d %s", &i, unit );
+	if( n > 0 )
+		size = i;
+	if( n > 1 ) {
+		for( j = 0; j < VIPS_NUMBER( units ); j++ )
+			if( tolower( unit[0] ) == units[j].unit ) {
+				size *= units[j].multiplier;
+				break;
+			}
+	}
+	g_free( unit );
+
+	VIPS_DEBUG_MSG( "parse_size: parsed \"%s\" as %lld\n", 
+		size_string, size );
+
+	return( size );
+}
+
+/* Look up the const char * for an enum value.
+ */
+const char *
+vips_enum_string( GType enm, int v )
+{
+	GEnumValue *value;
+
+	if( !(value = g_enum_get_value( g_type_class_ref( enm ), v )) )
+		return( "(null)" );
+
+	return( value->value_name );
+}
+
+const char *
+vips_enum_nick( GType enm, int v )
+{
+	GEnumValue *value;
+
+	if( !(value = g_enum_get_value( g_type_class_ref( enm ), v )) )
+		return( "(null)" );
+
+	return( value->value_nick );
+}
