@@ -56,10 +56,6 @@
 #include <vips/vips.h>
 #include <vips/vector.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Cleared by the command-line --vips-novector switch and the IM_NOVECTOR env
  * var.
  */
@@ -274,9 +270,11 @@ vips_vector_full( VipsVector *vector )
 	if( vector->n_source + vector->n_scanline + 1 > 7 )
 		return( TRUE );
 
-	/* I seem to get segvs with I counts over about 50 :-( argh.
+	/* I seem to get segvs with I counts over about 50 :-( argh. After
+	 * signalling full, some operations will add up to 4 more instructions
+	 * as they finish up. Leave a margin.
 	 */
-	if( vector->n_instruction > 45 )
+	if( vector->n_instruction > 42 )
 		return( TRUE );
 
 	return( FALSE );
@@ -352,7 +350,7 @@ vips_executor_set_scanline( VipsExecutor *executor,
 	VipsRegion *ir, int x, int y )
 {
 	VipsVector *vector = executor->vector;
-	PEL *base = (PEL *) VIPS_REGION_ADDR( ir, x, y );
+	VipsPel *base = VIPS_REGION_ADDR( ir, x, y );
 	int lsk = VIPS_REGION_LSKIP( ir );
 
 	int i;
