@@ -47,6 +47,7 @@
 
 /*
 #define DEBUG
+#define SHOW_HEADER
  */
 
 #ifdef HAVE_CONFIG_H
@@ -86,10 +87,6 @@
 #include <vips/internal.h>
 #include <vips/debug.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Try to make an O_BINARY ... sometimes need the leading '_'.
  */
 #ifdef BINARY_OPEN
@@ -135,14 +132,45 @@ vips__open_image_read( const char *filename )
 	 * work. When we later mmap this file, we set read-only, so there 
 	 * is little danger of scrubbing over files we own.
 	 */
-	if( (fd = open( filename, MODE_READWRITE )) == -1 ) {
+	fd = vips_tracked_open( filename, MODE_READWRITE );
+	if( fd == -1 ) 
 		/* Open read-write failed. Fall back to open read-only.
 		 */
-		if( (fd = open( filename, MODE_READONLY )) == -1 ) {
-			vips_error_system( errno, "VipsImage", 
-				_( "unable to open \"%s\"" ), filename );
-			return( -1 );
-		}
+		fd = vips_tracked_open( filename, MODE_READONLY );
+	
+	if( fd == -1 ) {
+		vips_error_system( errno, "VipsImage", 
+			_( "unable to open \"%s\"" ), filename );
+		return( -1 );
+	}
+
+	return( fd );
+}
+
+/* Open for write for image files. 
+ */
+int
+vips__open_image_write( const char *filename, gboolean temp )
+{
+	int flags;
+	int fd;
+
+	flags = MODE_WRITE;
+
+#ifdef _O_TEMPORARY
+	/* On Windows, setting O_TEMP gets the file automatically
+	 * deleted on process exit, even if the processes crashes. See
+	 * vips_image_rewind() for what we do to help on *nix.
+	 */
+	if( temp )
+		flags |= _O_TEMPORARY;
+#endif /*_O_TEMPORARY*/
+
+	if( (fd = vips_tracked_open( filename, flags, 0666 )) < 0 ) {
+		vips_error_system( errno, "VipsImage", 
+			_( "unable to write to \"%s\"" ), 
+			filename );
+		return( -1 );
 	}
 
 	return( fd );
@@ -172,146 +200,161 @@ image_pixel_length( VipsImage *image )
 	return( psize + image->sizeof_header );
 }
 
-/* Read short/int/float LSB and MSB first.
+/* Copy 2 and 4 bytes, optionally swapping byte order.
  */
 void
-vips__read_4byte( int msb_first, unsigned char *to, unsigned char **from )
+vips__copy_4byte( int swap, unsigned char *to, unsigned char *from )
 {
-	unsigned char *p = *from;
-	int out;
+	guint32 *in = (guint32 *) from;
+	guint32 *out = (guint32 *) to;
 
-	if( msb_first )
-		out = p[0] << 24 | p[1] << 16 | p[2] << 8 | p[3];
+	if( swap )
+		*out = GUINT32_SWAP_LE_BE( *in );
 	else
-		out = p[3] << 24 | p[2] << 16 | p[1] << 8 | p[0];
-
-	*from += 4;
-	*((guint32 *) to) = out;
+		*out = *in;
 }
 
 void
-vips__read_2byte( int msb_first, unsigned char *to, unsigned char **from )
+vips__copy_2byte( gboolean swap, unsigned char *to, unsigned char *from )
 {
-	int out;
-	unsigned char *p = *from;
+	guint16 *in = (guint16 *) from;
+	guint16 *out = (guint16 *) to;
 
-	if( msb_first )
-		out = p[0] << 8 | p[1];
+	if( swap )
+		*out = GUINT16_SWAP_LE_BE( *in );
 	else
-		out = p[1] << 8 | p[0];
-
-	*from += 2;
-	*((guint16 *) to) = out;
+		*out = *in;
 }
 
-/* We always write in native byte order.
- */
-void
-vips__write_4byte( unsigned char **to, unsigned char *from )
+guint32
+vips__file_magic( const char *filename )
 {
-	*((guint32 *) *to) = *((guint32 *) from);
-	*to += 4;
-}
+	guint32 magic;
 
-void
-vips__write_2byte( unsigned char **to, unsigned char *from )
-{
-	*((guint16 *) *to) = *((guint16 *) from);
-	*to += 2;
+	if( vips__get_bytes( filename, (unsigned char *) &magic, 4 ) &&
+		(magic == VIPS_MAGIC_INTEL || 
+		 magic == VIPS_MAGIC_SPARC ) )
+		return( magic );
+
+	return( 0 );
 }
 
 /* offset, read, write functions.
  */
 typedef struct _FieldIO {
 	glong offset;
-	void (*read)( int msb_first, unsigned char *to, unsigned char **from );
-	void (*write)( unsigned char **to, unsigned char *from );
+	int size;
+	void (*copy)( gboolean swap, unsigned char *to, unsigned char *from );
 } FieldIO;
 
 static FieldIO fields[] = {
-	{ G_STRUCT_OFFSET( VipsImage, Xsize ), 
-		vips__read_4byte, vips__write_4byte },
-	{ G_STRUCT_OFFSET( VipsImage, Ysize ), 
-		vips__read_4byte, vips__write_4byte },
-	{ G_STRUCT_OFFSET( VipsImage, Bands ), 
-		vips__read_4byte, vips__write_4byte },
-	{ G_STRUCT_OFFSET( VipsImage, Bbits ), 
-		vips__read_4byte, vips__write_4byte },
-	{ G_STRUCT_OFFSET( VipsImage, BandFmt ), 
-		vips__read_4byte, vips__write_4byte },
-	{ G_STRUCT_OFFSET( VipsImage, Coding ), 
-		vips__read_4byte, vips__write_4byte },
-	{ G_STRUCT_OFFSET( VipsImage, Type ), 
-		vips__read_4byte, vips__write_4byte },
-	{ G_STRUCT_OFFSET( VipsImage, Xres ), 
-		vips__read_4byte, vips__write_4byte },
-	{ G_STRUCT_OFFSET( VipsImage, Yres ), 
-		vips__read_4byte, vips__write_4byte },
-	{ G_STRUCT_OFFSET( VipsImage, Length ), 
-		vips__read_4byte, vips__write_4byte },
-	{ G_STRUCT_OFFSET( VipsImage, Compression ), 
-		vips__read_2byte, vips__write_2byte },
-	{ G_STRUCT_OFFSET( VipsImage, Level ), 
-		vips__read_2byte, vips__write_2byte },
-	{ G_STRUCT_OFFSET( VipsImage, Xoffset ), 
-		vips__read_4byte, vips__write_4byte },
-	{ G_STRUCT_OFFSET( VipsImage, Yoffset ), 
-		vips__read_4byte, vips__write_4byte }
+	{ G_STRUCT_OFFSET( VipsImage, Xsize ), 4, vips__copy_4byte },
+	{ G_STRUCT_OFFSET( VipsImage, Ysize ), 4, vips__copy_4byte },
+	{ G_STRUCT_OFFSET( VipsImage, Bands ), 4, vips__copy_4byte },
+	{ G_STRUCT_OFFSET( VipsImage, Bbits ), 4, vips__copy_4byte },
+	{ G_STRUCT_OFFSET( VipsImage, BandFmt ), 4, vips__copy_4byte },
+	{ G_STRUCT_OFFSET( VipsImage, Coding ), 4, vips__copy_4byte },
+	{ G_STRUCT_OFFSET( VipsImage, Type ), 4, vips__copy_4byte },
+	{ G_STRUCT_OFFSET( VipsImage, Xres_float ), 4, vips__copy_4byte },
+	{ G_STRUCT_OFFSET( VipsImage, Yres_float ), 4, vips__copy_4byte },
+	{ G_STRUCT_OFFSET( VipsImage, Length ), 4, vips__copy_4byte },
+	{ G_STRUCT_OFFSET( VipsImage, Compression ), 2, vips__copy_2byte },
+	{ G_STRUCT_OFFSET( VipsImage, Level ), 2, vips__copy_2byte },
+	{ G_STRUCT_OFFSET( VipsImage, Xoffset ), 4, vips__copy_4byte },
+	{ G_STRUCT_OFFSET( VipsImage, Yoffset ), 4, vips__copy_4byte }
 };
 
 int
 vips__read_header_bytes( VipsImage *im, unsigned char *from )
 {
-	int msb_first;
+	gboolean swap;
 	int i;
 
-	vips__read_4byte( 1, (unsigned char *) &im->magic, &from );
+#ifdef SHOW_HEADER
+	printf( "vips__read_header_bytes: file bytes:\n" ); 
+	for( i = 0; i < im->sizeof_header; i++ )
+		printf( "%2d - 0x%02x\n", i, from[i] );
+#endif /*SHOW_HEADER*/
+
+	/* The magic number is always written MSB first, we may need to swap.
+	 */
+	vips__copy_4byte( !vips_amiMSBfirst(), 
+		(unsigned char *) &im->magic, from );
+	from += 4;
 	if( im->magic != VIPS_MAGIC_INTEL && 
 		im->magic != VIPS_MAGIC_SPARC ) {
 		vips_error( "VipsImage", _( "\"%s\" is not a VIPS image" ), 
 			im->filename );
 		return( -1 );
 	}
-	msb_first = im->magic == VIPS_MAGIC_SPARC;
 
-	for( i = 0; i < VIPS_NUMBER( fields ); i++ )
-		fields[i].read( msb_first,
+	/* We need to swap for other fields if the file byte order is 
+	 * different from ours.
+	 */
+	swap = vips_amiMSBfirst() != (im->magic == VIPS_MAGIC_SPARC);
+
+	for( i = 0; i < VIPS_NUMBER( fields ); i++ ) {
+		fields[i].copy( swap,
 			&G_STRUCT_MEMBER( unsigned char, im, fields[i].offset ),
-			&from );
+			from );
+		from += fields[i].size;
+	}
 
 	/* Set this ourselves ... bbits is deprecated in the file format.
 	 */
 	im->Bbits = vips_format_sizeof( im->BandFmt ) << 3;
 
+	/* We read xres/yres as floats to a staging area, then copy to double
+	 * in the main fields.
+	 */
+	im->Xres = im->Xres_float;
+	im->Yres = im->Yres_float;
+
 	return( 0 );
 }
 
 int
 vips__write_header_bytes( VipsImage *im, unsigned char *to )
 {
-	guint32 magic;
+	/* Swap if the byte order we are asked to write the header in is
+	 * different from ours.
+	 */
+	gboolean swap = vips_amiMSBfirst() != (im->magic == VIPS_MAGIC_SPARC);
+
 	int i;
 	unsigned char *q;
 
+	/* We set xres/yres as floats in a staging area, then copy those
+	 * smaller values to the file. 
+	 */
+	im->Xres_float = im->Xres;
+	im->Yres_float = im->Yres;
+
 	/* Always write the magic number MSB first.
 	 */
-	magic = vips_amiMSBfirst() ? VIPS_MAGIC_SPARC : VIPS_MAGIC_INTEL;
-	to[0] = magic >> 24;
-	to[1] = magic >> 16;
-	to[2] = magic >> 8;
-	to[3] = magic;
+	vips__copy_4byte( !vips_amiMSBfirst(), 
+		to, (unsigned char *) &im->magic );
 	q = to + 4;
 
-	for( i = 0; i < VIPS_NUMBER( fields ); i++ )
-		fields[i].write( &q,
+	for( i = 0; i < VIPS_NUMBER( fields ); i++ ) {
+		fields[i].copy( swap,
+			q, 
 			&G_STRUCT_MEMBER( unsigned char, im, 
 				fields[i].offset ) );
+		q += fields[i].size;
+	}
 
 	/* Pad spares with zeros.
 	 */
 	while( q - to < im->sizeof_header )
 		*q++ = 0;
 
+#ifdef SHOW_HEADER
+	printf( "vips__write_header_bytes: file bytes:\n" ); 
+	for( i = 0; i < im->sizeof_header; i++ )
+		printf( "%2d - 0x%02x\n", i, to[i] );
+#endif /*SHOW_HEADER*/
+
 	return( 0 );
 }
 
@@ -526,7 +569,7 @@ rebuild_header_meta( VipsImage *im, xmlNode *i )
 
 			content = (char *) xmlNodeGetContent( i );
 			g_value_init( &save_value, VIPS_TYPE_SAVE_STRING );
-			vips_save_string_set( &save_value, content );
+			vips_value_set_save_string( &save_value, content );
 			xmlFree( content );
 
 			g_value_init( &value, gtype );
@@ -537,11 +580,7 @@ rebuild_header_meta( VipsImage *im, xmlNode *i )
 					"save format" ) );
 				return( -1 );
 			}
-			if( vips_image_set( im, name, &value ) ) {
-				g_value_unset( &save_value );
-				g_value_unset( &value );
-				return( -1 );
-			}
+			vips_image_set( im, name, &value );
 			g_value_unset( &save_value );
 			g_value_unset( &value );
 		}
@@ -615,11 +654,8 @@ readhist( VipsImage *im )
 
 		if( !(doc = read_xml( im )) )
 			return( -1 );
-		if( vips_image_set_area( im, VIPS_META_XML, 
-			(VipsCallbackFn) xmlFreeDoc, doc ) ) {
-			xmlFreeDoc( doc );
-			return( -1 );
-		}
+		vips_image_set_area( im, VIPS_META_XML, 
+			(VipsCallbackFn) xmlFreeDoc, doc );
 	}
 
 	if( rebuild_header( im ) )
@@ -693,7 +729,7 @@ save_fields_meta( VipsMeta *meta, xmlNode *node )
 			return( node );
 		}
 		if( set_field( node, meta->field, g_type_name( type ), 
-			vips_save_string_get( &save_value ) ) ) {
+			vips_value_get_save_string( &save_value ) ) ) {
 			g_value_unset( &save_value );
 			return( node );
 		}
@@ -962,13 +998,15 @@ vips_image_open_output( VipsImage *image )
 		 */
 		unsigned char header[VIPS_SIZEOF_HEADER];
 
-		if( (image->fd = open( image->filename, 
-			MODE_WRITE, 0666 )) < 0 ) {
-			vips_error_system( errno, "VipsImage", 
-				_( "unable to write to \"%s\"" ), 
-				image->filename );
+		if( (image->fd = vips__open_image_write( image->filename, 
+			image->delete_on_close )) < 0 )
 			return( -1 );
-		}
+
+		/* We always write in native mode, so we must overwrite the
+		 * magic we read from the file originally.
+		 */
+		image->magic = vips_amiMSBfirst() ? 
+			VIPS_MAGIC_SPARC : VIPS_MAGIC_INTEL;
 
 		if( vips__write_header_bytes( image, header ) ||
 			vips__write( image->fd, header, VIPS_SIZEOF_HEADER ) )
