@@ -39,10 +39,6 @@
 
 #include "mosaic.h"
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Given a pair of points, return scale, angle, dx, dy to resample the 2nd
  * image with.
  */
