@@ -46,10 +46,6 @@
 #include <vips/internal.h>
 #include <vips/transform.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Calculate the inverse transformation.
  */
 int
