@@ -45,7 +45,6 @@
 #include <unistd.h>
 #endif /*HAVE_UNISTD_H*/
 #include <errno.h>
-#include <assert.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -58,10 +57,6 @@
 
 #include <vips/vips.h>
 
-#ifdef WITH_DMALLOC
-#include <dmalloc.h>
-#endif /*WITH_DMALLOC*/
-
 /* Zero freed mem to help catch stray pointers.
  */
 #ifdef NDEBUG
