@@ -35,6 +35,9 @@
  * 24/6/10
  * 	- less chatty error messages
  * 	- oops, don't rename "copy_set" as "copy_"
+ * 6/2/12
+ * 	- long arg names in decls to help SWIG
+ * 	- don't wrap im_remainderconst_vec()
  */
 
 /*
@@ -64,14 +67,12 @@
  */
 
 /*
-#define DEBUG_FATAL
-#define DEBUG_LEAK
 #define DEBUG
+#define DEBUG_FATAL
  */
 
 /* Need to disable these sometimes.
 #undef DEBUG_FATAL
-#undef DEBUG_LEAK
  */
 
 #ifdef HAVE_CONFIG_H
@@ -167,14 +168,15 @@ list_function( im_function *func )
 }
 
 static void *
-list_class( VipsObjectClass *class )
+list_class( GType type )
 {
-	int depth = vips_class_depth( class );
+	int depth = vips_type_depth( type );
 	int i;
 
 	for( i = 0; i < depth * 2; i++ )
 		printf( " " );
-	vips_object_print_class( class );
+	vips_object_print_summary_class( 
+		VIPS_OBJECT_CLASS( g_type_class_ref( type ) ) );
 
 	return( NULL );
 }
@@ -185,8 +187,13 @@ print_list( int argc, char **argv )
 	if( !argv[0] || strcmp( argv[0], "packages" ) == 0 ) 
 		im_map_packages( (VSListMap2Fn) list_package, NULL );
 	else if( strcmp( argv[0], "classes" ) == 0 ) 
-		vips_class_map_all( g_type_from_name( "VipsObject" ), 
-			(VipsClassMap) list_class, NULL );
+		vips_type_map_all( g_type_from_name( "VipsObject" ), 
+			(VipsTypeMapFn) list_class, NULL );
+	else if( g_type_from_name( argv[0] ) &&
+		g_type_is_a( g_type_from_name( argv[0] ), VIPS_TYPE_OBJECT ) ) {
+		vips_type_map_all( g_type_from_name( argv[0] ), 
+			(VipsTypeMapFn) list_class, NULL );
+	}
 	else {
 		if( map_name( argv[0], list_function ) )
 			error_exit( "unknown package \"%s\"", argv[0] ); 
@@ -195,33 +202,6 @@ print_list( int argc, char **argv )
 	return( 0 );
 }
 
-/* Is s1 a prefix of s2?
- */
-static int
-isprefix( const char *s1, const char *s2 )
-{
-	while( *s1 && *s1 == *s2 ) {
-		s1++;
-		s2++;
-	}
-
-	return( *s1 == '\0' );
-}
-
-/* Is s1 a postfix of s2?
- */
-static int
-ispostfix( const char *s1, const char *s2 )
-{
-	int l1 = strlen( s1 );
-	int l2 = strlen( s2 );
-
-	if( l2 < l1 )
-		return( 0 );
-	
-	return( strcasecmp( s1, s2 + l2 - l1 ) == 0 );
-}
-
 /* Print "ln -s" lines for this package.
  */
 static void *
@@ -385,6 +365,14 @@ is_cppable( im_function *fn )
 {
 	int j;
 
+	/* Don't wrap im_remainderconst_vec().
+	 *
+	 * This has been replaced by the saner name im_remainder_vec(). If we
+	 * generate wrappers for both names we get a overloading clash.
+	 */
+	if( strcmp( fn->name, "im_remainderconst_vec" ) == 0 )
+		return( 0 );
+
 	/* Check we know all the types.
 	 */
 	for( j = 0; j < fn->argc; j++ ) {
@@ -456,7 +444,7 @@ find_ioargs( im_function *fn, int *ia, int *oa )
 static gboolean
 drop_postfix( char *str, const char *postfix )
 {
-	if( ispostfix( postfix, str ) ) {
+	if( vips_ispostfix( str, postfix ) ) {
 		str[strlen( str ) - strlen( postfix )] = '\0';
 
 		return( TRUE );
@@ -486,7 +474,7 @@ c2cpp_name( const char *in, char *out )
 
 	/* Copy, chopping off "im_" prefix.
 	 */
-	if( isprefix( "im_", in ) )
+	if( vips_isprefix( "im_", in ) )
 		strcpy( out, in + 3 );
 	else
 		strcpy( out, in );
@@ -499,7 +487,7 @@ c2cpp_name( const char *in, char *out )
 
 		found = FALSE;
 		for( i = 0; i < IM_NUMBER( dont_drop ); i++ )
-			if( ispostfix( dont_drop[i], out ) ) {
+			if( vips_ispostfix( out, dont_drop[i] ) ) {
 				found = TRUE;
 				break;
 			}
@@ -654,6 +642,14 @@ print_cppdecl( im_function *fn )
 		 */
 		if( ty->flags & IM_TYPE_OUTPUT )
 			printf( "&" );
+
+		/* Print arg name. 
+		 *
+		 * Prepend the member name to make the arg
+		 * unique. This is important for SWIG since it needs to have
+		 * unique names for %apply.
+		 */
+		printf( " %s_%s", name, fn->argv[j].name );
 	}
 
 	/* End of arg list!
@@ -1014,14 +1010,9 @@ main( int argc, char **argv )
 		G_LOG_LEVEL_ERROR |
 		G_LOG_LEVEL_CRITICAL |
 		G_LOG_LEVEL_WARNING );
-
-	fprintf( stderr, "*** DEBUG_FATAL: will abort() on first warning\n" );
+	fprintf( stderr, "** DEBUG_FATAL\n" );
 #endif /*!DEBUG_FATAL*/
 
-#ifdef DEBUG_LEAK
-	fprintf( stderr, "*** DEBUG_LEAK: will leak test on exit\n" );
-#endif /*!DEBUG_LEAK*/
-
 	/* Try to find our action.
 	 */
 	action = NULL;
@@ -1070,28 +1061,9 @@ main( int argc, char **argv )
 				break;
 			}
 
-	/* Could be a vips8 VipsOperation.
-	 */
-	if( action && !handled && 
-		(operation = vips_operation_new( action )) ) {
-		main_group = add_main_group( context, operation );
-		vips_call_options( main_group, operation );
-		parse_options( context, &argc, argv );
-
-		if( vips_call_argv( operation, argc - 1, argv + 1 ) ) {
-			if( argc == 1 ) 
-				vips_object_print( VIPS_OBJECT( operation ) );
-			else
-				error_exit( NULL );
-		}
-
-		g_object_unref( operation );
-
-		handled = TRUE;
-	}
-	im_error_clear();
-
-	/* Could be a vips7 im_function.
+	/* Could be a vips7 im_function. We need to test for vips7 first,
+	 * since we don't want to use the vips7 compat wrappers in vips8
+	 * unless we have to. They don't support all args types.
 	 */
 	if( action && !handled && 
 		(fn = im_find_function( action )) ) {
@@ -1109,6 +1081,32 @@ main( int argc, char **argv )
 	}
 	im_error_clear();
 
+	/* Could be a vips8 VipsOperation.
+	 */
+	if( action && !handled && 
+		(operation = vips_operation_new( action )) ) {
+		main_group = add_main_group( context, operation );
+		vips_call_options( main_group, operation );
+		parse_options( context, &argc, argv );
+
+		if( vips_call_argv( operation, argc - 1, argv + 1 ) ) {
+			if( argc == 1 ) 
+				vips_operation_class_print_usage( 
+					VIPS_OPERATION_GET_CLASS( operation ) );
+
+			vips_object_unref_outputs( VIPS_OBJECT( operation ) );
+			g_object_unref( operation );
+
+			error_exit( NULL );
+		}
+
+		vips_object_unref_outputs( VIPS_OBJECT( operation ) );
+		g_object_unref( operation );
+
+		handled = TRUE;
+	}
+	im_error_clear();
+
 	if( action && !handled ) {
 		printf( "%s", _( "possible actions:\n" ) );
 		for( i = 0; i < VIPS_NUMBER( actions ); i++ )
@@ -1127,13 +1125,7 @@ main( int argc, char **argv )
 
 	g_option_context_free( context );
 
-	im_close_plugins();
-
-#ifdef DEBUG_LEAK
-	printf( "** leak test on exit:\n" );
-	im__print_all();
-	printf( "** leak test done\n" );
-#endif /*DEBUG_LEAK*/
+	vips_shutdown();
 
 	return( 0 );
 }
